@@ -1,6 +1,7 @@
 #!/bin/bash
 
 for var in \
+    ACO_DEBUG \
     ASAN_OPTIONS \
     BASE_SYSTEM_FORK_HOST_PREFIX \
     BASE_SYSTEM_MAINLINE_HOST_PREFIX \
@@ -26,7 +27,6 @@ for var in \
     CI_SERVER_URL \
     CROSVM_GALLIUM_DRIVER \
     CROSVM_GPU_ARGS \
-    CROSVM_TEST_SCRIPT \
     DEQP_BIN_DIR \
     DEQP_CASELIST_FILTER \
     DEQP_CASELIST_INV_FILTER \
@@ -49,6 +49,7 @@ for var in \
     FDO_UPSTREAM_REPO \
     FD_MESA_DEBUG \
     FLAKES_CHANNEL \
+    FREEDRENO_HANGCHECK_MS \
     GALLIUM_DRIVER \
     GALLIVM_PERF \
     GPU_VERSION \
@@ -60,12 +61,15 @@ for var in \
     GTEST_SKIPS \
     HWCI_FREQ_MAX \
     HWCI_KERNEL_MODULES \
+    HWCI_KVM \
     HWCI_START_XORG \
     HWCI_TEST_SCRIPT \
     IR3_SHADER_DEBUG \
     JOB_ARTIFACTS_BASE \
     JOB_RESULTS_PATH \
     JOB_ROOTFS_OVERLAY_PATH \
+    KERNEL_IMAGE_BASE_URL \
+    KERNEL_IMAGE_NAME \
     LD_LIBRARY_PATH \
     LP_NUM_THREADS \
     MESA_BASE_TAG \
@@ -77,9 +81,12 @@ for var in \
     MESA_IMAGE \
     MESA_IMAGE_PATH \
     MESA_IMAGE_TAG \
+    MESA_LOADER_DRIVER_OVERRIDE \
     MESA_TEMPLATES_COMMIT \
     MESA_VK_IGNORE_CONFORMANCE_WARNING \
+    MESA_SPIRV_LOG_LEVEL \
     MINIO_HOST \
+    MINIO_RESULTS_UPLOAD \
     NIR_DEBUG \
     PAN_I_WANT_A_BROKEN_VULKAN_DRIVER \
     PAN_MESA_DEBUG \
@@ -92,11 +99,14 @@ for var in \
     PIGLIT_REPLAY_DESCRIPTION_FILE \
     PIGLIT_REPLAY_DEVICE_NAME \
     PIGLIT_REPLAY_EXTRA_ARGS \
+    PIGLIT_REPLAY_LOOP_TIMES \
     PIGLIT_REPLAY_REFERENCE_IMAGES_BASE \
     PIGLIT_REPLAY_SUBCOMMAND \
     PIGLIT_RESULTS \
     PIGLIT_TESTS \
     PIPELINE_ARTIFACTS_BASE \
+    RADV_DEBUG \
+    RADV_PERFTEST \
     SKQP_ASSETS_DIR \
     SKQP_BACKENDS \
     TU_DEBUG \
@@ -104,6 +114,7 @@ for var in \
     VK_CPU \
     VK_DRIVER \
     VK_ICD_FILENAMES \
+    VKD3D_PROTON_RESULTS \
     ; do
   if [ -n "${!var+x}" ]; then
     echo "export $var=${!var@Q}"
