@@ -65,16 +65,16 @@ init()
 }
 
 void
-init_program(Program* program, Stage stage, const struct radv_shader_info* info,
-             enum chip_class chip_class, enum radeon_family family, bool wgp_mode,
+init_program(Program* program, Stage stage, const struct aco_shader_info* info,
+             enum amd_gfx_level gfx_level, enum radeon_family family, bool wgp_mode,
              ac_shader_config* config)
 {
    program->stage = stage;
    program->config = config;
-   program->info = info;
-   program->chip_class = chip_class;
+   program->info = *info;
+   program->gfx_level = gfx_level;
    if (family == CHIP_UNKNOWN) {
-      switch (chip_class) {
+      switch (gfx_level) {
       case GFX6: program->family = CHIP_TAHITI; break;
       case GFX7: program->family = CHIP_BONAIRE; break;
       case GFX8: program->family = CHIP_POLARIS10; break;
@@ -88,10 +88,10 @@ init_program(Program* program, Stage stage, const struct radv_shader_info* info,
    program->wave_size = info->wave_size;
    program->lane_mask = program->wave_size == 32 ? s1 : s2;
 
-   program->dev.lds_encoding_granule = chip_class >= GFX7 ? 512 : 256;
-   program->dev.lds_alloc_granule =
-      chip_class >= GFX10_3 ? 1024 : program->dev.lds_encoding_granule;
-   program->dev.lds_limit = chip_class >= GFX7 ? 65536 : 32768;
+   program->dev.lds_encoding_granule = gfx_level >= GFX11 && stage == fragment_fs ? 1024 :
+                                       gfx_level >= GFX7 ? 512 : 256;
+   program->dev.lds_alloc_granule = gfx_level >= GFX10_3 ? 1024 : program->dev.lds_encoding_granule;
+   program->dev.lds_limit = gfx_level >= GFX7 ? 65536 : 32768;
    /* apparently gfx702 also has 16-bank LDS but I can't find a family for that */
    program->dev.has_16bank_lds = family == CHIP_KABINI || family == CHIP_STONEY;
 
@@ -99,17 +99,17 @@ init_program(Program* program, Stage stage, const struct radv_shader_info* info,
    program->dev.physical_vgprs = 256;
    program->dev.vgpr_alloc_granule = 4;
 
-   if (chip_class >= GFX10) {
+   if (gfx_level >= GFX10) {
       program->dev.physical_sgprs = 5120; /* doesn't matter as long as it's at least 128 * 40 */
       program->dev.physical_vgprs = program->wave_size == 32 ? 1024 : 512;
       program->dev.sgpr_alloc_granule = 128;
       program->dev.sgpr_limit =
          108; /* includes VCC, which can be treated as s[106-107] on GFX10+ */
-      if (chip_class >= GFX10_3)
+      if (gfx_level == GFX10_3)
          program->dev.vgpr_alloc_granule = program->wave_size == 32 ? 16 : 8;
       else
          program->dev.vgpr_alloc_granule = program->wave_size == 32 ? 8 : 4;
-   } else if (program->chip_class >= GFX8) {
+   } else if (program->gfx_level >= GFX8) {
       program->dev.physical_sgprs = 800;
       program->dev.sgpr_alloc_granule = 16;
       program->dev.sgpr_limit = 102;
@@ -122,14 +122,14 @@ init_program(Program* program, Stage stage, const struct radv_shader_info* info,
    }
 
    program->dev.max_wave64_per_simd = 10;
-   if (program->chip_class >= GFX10_3)
+   if (program->gfx_level >= GFX10_3)
       program->dev.max_wave64_per_simd = 16;
-   else if (program->chip_class == GFX10)
+   else if (program->gfx_level == GFX10)
       program->dev.max_wave64_per_simd = 20;
    else if (program->family >= CHIP_POLARIS10 && program->family <= CHIP_VEGAM)
       program->dev.max_wave64_per_simd = 8;
 
-   program->dev.simd_per_cu = program->chip_class >= GFX10 ? 2 : 4;
+   program->dev.simd_per_cu = program->gfx_level >= GFX10 ? 2 : 4;
 
    switch (program->family) {
    /* GFX8 APUs */
@@ -144,10 +144,28 @@ init_program(Program* program, Stage stage, const struct radv_shader_info* info,
 
    program->dev.sram_ecc_enabled = program->family == CHIP_ARCTURUS;
    /* apparently gfx702 also has fast v_fma_f32 but I can't find a family for that */
-   program->dev.has_fast_fma32 = program->chip_class >= GFX9;
+   program->dev.has_fast_fma32 = program->gfx_level >= GFX9;
    if (program->family == CHIP_TAHITI || program->family == CHIP_CARRIZO ||
        program->family == CHIP_HAWAII)
       program->dev.has_fast_fma32 = true;
+   program->dev.has_mac_legacy32 = program->gfx_level <= GFX7 || program->gfx_level >= GFX10;
+
+   program->dev.fused_mad_mix = program->gfx_level >= GFX10;
+   if (program->family == CHIP_VEGA12 || program->family == CHIP_VEGA20 ||
+       program->family == CHIP_ARCTURUS || program->family == CHIP_ALDEBARAN)
+      program->dev.fused_mad_mix = true;
+
+   if (program->gfx_level >= GFX11) {
+      program->dev.scratch_global_offset_min = -4096;
+      program->dev.scratch_global_offset_max = 4095;
+   } else if (program->gfx_level >= GFX10 || program->gfx_level == GFX8) {
+      program->dev.scratch_global_offset_min = -2048;
+      program->dev.scratch_global_offset_max = 2047;
+   } else if (program->gfx_level == GFX9) {
+      /* The minimum is actually -4096, but negative offsets are broken when SADDR is used. */
+      program->dev.scratch_global_offset_min = 0;
+      program->dev.scratch_global_offset_max = 4095;
+   }
 
    program->wgp_mode = wgp_mode;
 
@@ -182,12 +200,12 @@ get_sync_info(const Instruction* instr)
 }
 
 bool
-can_use_SDWA(chip_class chip, const aco_ptr<Instruction>& instr, bool pre_ra)
+can_use_SDWA(amd_gfx_level gfx_level, const aco_ptr<Instruction>& instr, bool pre_ra)
 {
    if (!instr->isVALU())
       return false;
 
-   if (chip < GFX8 || instr->isDPP() || instr->isVOP3P())
+   if (gfx_level < GFX8 || gfx_level >= GFX11 || instr->isDPP() || instr->isVOP3P())
       return false;
 
    if (instr->isSDWA())
@@ -197,9 +215,9 @@ can_use_SDWA(chip_class chip, const aco_ptr<Instruction>& instr, bool pre_ra)
       VOP3_instruction& vop3 = instr->vop3();
       if (instr->format == Format::VOP3)
          return false;
-      if (vop3.clamp && instr->isVOPC() && chip != GFX8)
+      if (vop3.clamp && instr->isVOPC() && gfx_level != GFX8)
          return false;
-      if (vop3.omod && chip < GFX9)
+      if (vop3.omod && gfx_level < GFX9)
          return false;
 
       // TODO: return true if we know we will use vcc
@@ -209,7 +227,7 @@ can_use_SDWA(chip_class chip, const aco_ptr<Instruction>& instr, bool pre_ra)
       for (unsigned i = 1; i < instr->operands.size(); i++) {
          if (instr->operands[i].isLiteral())
             return false;
-         if (chip < GFX9 && !instr->operands[i].isOfType(RegType::vgpr))
+         if (gfx_level < GFX9 && !instr->operands[i].isOfType(RegType::vgpr))
             return false;
       }
    }
@@ -220,7 +238,7 @@ can_use_SDWA(chip_class chip, const aco_ptr<Instruction>& instr, bool pre_ra)
    if (!instr->operands.empty()) {
       if (instr->operands[0].isLiteral())
          return false;
-      if (chip < GFX9 && !instr->operands[0].isOfType(RegType::vgpr))
+      if (gfx_level < GFX9 && !instr->operands[0].isOfType(RegType::vgpr))
          return false;
       if (instr->operands[0].bytes() > 4)
          return false;
@@ -231,11 +249,11 @@ can_use_SDWA(chip_class chip, const aco_ptr<Instruction>& instr, bool pre_ra)
    bool is_mac = instr->opcode == aco_opcode::v_mac_f32 || instr->opcode == aco_opcode::v_mac_f16 ||
                  instr->opcode == aco_opcode::v_fmac_f32 || instr->opcode == aco_opcode::v_fmac_f16;
 
-   if (chip != GFX8 && is_mac)
+   if (gfx_level != GFX8 && is_mac)
       return false;
 
    // TODO: return true if we know we will use vcc
-   if (!pre_ra && instr->isVOPC() && chip == GFX8)
+   if (!pre_ra && instr->isVOPC() && gfx_level == GFX8)
       return false;
    if (!pre_ra && instr->operands.size() >= 3 && !is_mac)
       return false;
@@ -248,7 +266,7 @@ can_use_SDWA(chip_class chip, const aco_ptr<Instruction>& instr, bool pre_ra)
 
 /* updates "instr" and returns the old instruction (or NULL if no update was needed) */
 aco_ptr<Instruction>
-convert_to_SDWA(chip_class chip, aco_ptr<Instruction>& instr)
+convert_to_SDWA(amd_gfx_level gfx_level, aco_ptr<Instruction>& instr)
 {
    if (instr->isSDWA())
       return NULL;
@@ -281,23 +299,25 @@ convert_to_SDWA(chip_class chip, aco_ptr<Instruction>& instr)
 
    sdwa.dst_sel = SubdwordSel(instr->definitions[0].bytes(), 0, false);
 
-   if (instr->definitions[0].getTemp().type() == RegType::sgpr && chip == GFX8)
+   if (instr->definitions[0].getTemp().type() == RegType::sgpr && gfx_level == GFX8)
       instr->definitions[0].setFixed(vcc);
    if (instr->definitions.size() >= 2)
       instr->definitions[1].setFixed(vcc);
    if (instr->operands.size() >= 3)
       instr->operands[2].setFixed(vcc);
 
+   instr->pass_flags = tmp->pass_flags;
+
    return tmp;
 }
 
 bool
-can_use_DPP(const aco_ptr<Instruction>& instr, bool pre_ra)
+can_use_DPP(const aco_ptr<Instruction>& instr, bool pre_ra, bool dpp8)
 {
    assert(instr->isVALU() && !instr->operands.empty());
 
    if (instr->isDPP())
-      return true;
+      return instr->isDPP8() == dpp8;
 
    if (instr->operands.size() && instr->operands[0].isLiteral())
       return false;
@@ -316,6 +336,8 @@ can_use_DPP(const aco_ptr<Instruction>& instr, bool pre_ra)
       const VOP3_instruction* vop3 = &instr->vop3();
       if (vop3->clamp || vop3->omod || vop3->opsel)
          return false;
+      if (dpp8)
+         return false;
       if (instr->format == Format::VOP3)
          return false;
       if (instr->operands.size() > 1 && !instr->operands[1].isOfType(RegType::vgpr))
@@ -331,29 +353,39 @@ can_use_DPP(const aco_ptr<Instruction>& instr, bool pre_ra)
 }
 
 aco_ptr<Instruction>
-convert_to_DPP(aco_ptr<Instruction>& instr)
+convert_to_DPP(aco_ptr<Instruction>& instr, bool dpp8)
 {
    if (instr->isDPP())
       return NULL;
 
    aco_ptr<Instruction> tmp = std::move(instr);
-   Format format =
-      (Format)(((uint32_t)tmp->format & ~(uint32_t)Format::VOP3) | (uint32_t)Format::DPP);
-   instr.reset(create_instruction<DPP_instruction>(tmp->opcode, format, tmp->operands.size(),
-                                                   tmp->definitions.size()));
+   Format format = (Format)(((uint32_t)tmp->format & ~(uint32_t)Format::VOP3) |
+                            (dpp8 ? (uint32_t)Format::DPP8 : (uint32_t)Format::DPP16));
+   if (dpp8)
+      instr.reset(create_instruction<DPP8_instruction>(tmp->opcode, format, tmp->operands.size(),
+                                                       tmp->definitions.size()));
+   else
+      instr.reset(create_instruction<DPP16_instruction>(tmp->opcode, format, tmp->operands.size(),
+                                                        tmp->definitions.size()));
    std::copy(tmp->operands.cbegin(), tmp->operands.cend(), instr->operands.begin());
    for (unsigned i = 0; i < instr->definitions.size(); i++)
       instr->definitions[i] = tmp->definitions[i];
 
-   DPP_instruction* dpp = &instr->dpp();
-   dpp->dpp_ctrl = dpp_quad_perm(0, 1, 2, 3);
-   dpp->row_mask = 0xf;
-   dpp->bank_mask = 0xf;
+   if (dpp8) {
+      DPP8_instruction* dpp = &instr->dpp8();
+      for (unsigned i = 0; i < 8; i++)
+         dpp->lane_sel[i] = i;
+   } else {
+      DPP16_instruction* dpp = &instr->dpp16();
+      dpp->dpp_ctrl = dpp_quad_perm(0, 1, 2, 3);
+      dpp->row_mask = 0xf;
+      dpp->bank_mask = 0xf;
 
-   if (tmp->isVOP3()) {
-      const VOP3_instruction* vop3 = &tmp->vop3();
-      memcpy(dpp->neg, vop3->neg, sizeof(dpp->neg));
-      memcpy(dpp->abs, vop3->abs, sizeof(dpp->abs));
+      if (tmp->isVOP3()) {
+         const VOP3_instruction* vop3 = &tmp->vop3();
+         memcpy(dpp->neg, vop3->neg, sizeof(dpp->neg));
+         memcpy(dpp->abs, vop3->abs, sizeof(dpp->abs));
+      }
    }
 
    if (instr->isVOPC() || instr->definitions.size() > 1)
@@ -362,14 +394,16 @@ convert_to_DPP(aco_ptr<Instruction>& instr)
    if (instr->operands.size() >= 3)
       instr->operands[2].setFixed(vcc);
 
+   instr->pass_flags = tmp->pass_flags;
+
    return tmp;
 }
 
 bool
-can_use_opsel(chip_class chip, aco_opcode op, int idx, bool high)
+can_use_opsel(amd_gfx_level gfx_level, aco_opcode op, int idx)
 {
    /* opsel is only GFX9+ */
-   if ((high || idx == -1) && chip < GFX9)
+   if (gfx_level < GFX9)
       return false;
 
    switch (op) {
@@ -409,10 +443,10 @@ can_use_opsel(chip_class chip, aco_opcode op, int idx, bool high)
 }
 
 bool
-instr_is_16bit(chip_class chip, aco_opcode op)
+instr_is_16bit(amd_gfx_level gfx_level, aco_opcode op)
 {
    /* partial register writes are GFX9+, only */
-   if (chip < GFX9)
+   if (gfx_level < GFX9)
       return false;
 
    switch (op) {
@@ -424,10 +458,11 @@ instr_is_16bit(chip_class chip, aco_opcode op)
    case aco_opcode::v_div_fixup_f16:
    case aco_opcode::v_interp_p2_f16:
    case aco_opcode::v_fma_mixlo_f16:
+   case aco_opcode::v_fma_mixhi_f16:
    /* VOP2 */
    case aco_opcode::v_mac_f16:
    case aco_opcode::v_madak_f16:
-   case aco_opcode::v_madmk_f16: return chip >= GFX9;
+   case aco_opcode::v_madmk_f16: return gfx_level >= GFX9;
    case aco_opcode::v_add_f16:
    case aco_opcode::v_sub_f16:
    case aco_opcode::v_subrev_f16:
@@ -455,7 +490,7 @@ instr_is_16bit(chip_class chip, aco_opcode op)
    case aco_opcode::v_rndne_f16:
    case aco_opcode::v_fract_f16:
    case aco_opcode::v_sin_f16:
-   case aco_opcode::v_cos_f16: return chip >= GFX10;
+   case aco_opcode::v_cos_f16: return gfx_level >= GFX10;
    // TODO: confirm whether these write 16 or 32 bit on GFX10+
    // case aco_opcode::v_cvt_u16_f16:
    // case aco_opcode::v_cvt_i16_f16:
@@ -463,7 +498,7 @@ instr_is_16bit(chip_class chip, aco_opcode op)
    // case aco_opcode::v_cvt_norm_i16_f16:
    // case aco_opcode::v_cvt_norm_u16_f16:
    /* on GFX10, all opsel instructions preserve the high bits */
-   default: return chip >= GFX10 && can_use_opsel(chip, op, -1, false);
+   default: return gfx_level >= GFX10 && can_use_opsel(gfx_level, op, -1);
    }
 }
 
@@ -554,9 +589,11 @@ needs_exec_mask(const Instruction* instr)
          return instr->reads_exec();
       case aco_opcode::p_spill:
       case aco_opcode::p_reload:
+      case aco_opcode::p_end_linear_vgpr:
       case aco_opcode::p_logical_start:
       case aco_opcode::p_logical_end:
-      case aco_opcode::p_startpgm: return instr->reads_exec();
+      case aco_opcode::p_startpgm:
+      case aco_opcode::p_init_scratch: return instr->reads_exec();
       default: break;
       }
    }
@@ -567,9 +604,9 @@ needs_exec_mask(const Instruction* instr)
 struct CmpInfo {
    aco_opcode ordered;
    aco_opcode unordered;
-   aco_opcode ordered_swapped;
-   aco_opcode unordered_swapped;
+   aco_opcode swapped;
    aco_opcode inverse;
+   aco_opcode vcmpx;
    aco_opcode f32;
    unsigned size;
 };
@@ -579,8 +616,9 @@ get_cmp_info(aco_opcode op, CmpInfo* info)
 {
    info->ordered = aco_opcode::num_opcodes;
    info->unordered = aco_opcode::num_opcodes;
-   info->ordered_swapped = aco_opcode::num_opcodes;
-   info->unordered_swapped = aco_opcode::num_opcodes;
+   info->swapped = aco_opcode::num_opcodes;
+   info->inverse = aco_opcode::num_opcodes;
+   info->f32 = aco_opcode::num_opcodes;
    switch (op) {
       // clang-format off
 #define CMP2(ord, unord, ord_swap, unord_swap, sz)                                                 \
@@ -588,12 +626,14 @@ get_cmp_info(aco_opcode op, CmpInfo* info)
    case aco_opcode::v_cmp_n##unord##_f##sz:                                                        \
       info->ordered = aco_opcode::v_cmp_##ord##_f##sz;                                             \
       info->unordered = aco_opcode::v_cmp_n##unord##_f##sz;                                        \
-      info->ordered_swapped = aco_opcode::v_cmp_##ord_swap##_f##sz;                                \
-      info->unordered_swapped = aco_opcode::v_cmp_n##unord_swap##_f##sz;                           \
+      info->swapped = op == aco_opcode::v_cmp_##ord##_f##sz ? aco_opcode::v_cmp_##ord_swap##_f##sz \
+                                                      : aco_opcode::v_cmp_n##unord_swap##_f##sz;   \
       info->inverse = op == aco_opcode::v_cmp_n##unord##_f##sz ? aco_opcode::v_cmp_##unord##_f##sz \
                                                                : aco_opcode::v_cmp_n##ord##_f##sz; \
       info->f32 = op == aco_opcode::v_cmp_##ord##_f##sz ? aco_opcode::v_cmp_##ord##_f32            \
                                                         : aco_opcode::v_cmp_n##unord##_f32;        \
+      info->vcmpx = op == aco_opcode::v_cmp_##ord##_f##sz ? aco_opcode::v_cmpx_##ord##_f##sz       \
+                                                          : aco_opcode::v_cmpx_n##unord##_f##sz;   \
       info->size = sz;                                                                             \
       return true;
 #define CMP(ord, unord, ord_swap, unord_swap)                                                      \
@@ -603,7 +643,7 @@ get_cmp_info(aco_opcode op, CmpInfo* info)
       CMP(lt, /*n*/ge, gt, /*n*/le)
       CMP(eq, /*n*/lg, eq, /*n*/lg)
       CMP(le, /*n*/gt, ge, /*n*/lt)
-      CMP(gt, /*n*/le, lt, /*n*/le)
+      CMP(gt, /*n*/le, lt, /*n*/ge)
       CMP(lg, /*n*/eq, lg, /*n*/eq)
       CMP(ge, /*n*/lt, le, /*n*/gt)
 #undef CMP
@@ -611,18 +651,53 @@ get_cmp_info(aco_opcode op, CmpInfo* info)
 #define ORD_TEST(sz)                                                                               \
    case aco_opcode::v_cmp_u_f##sz:                                                                 \
       info->f32 = aco_opcode::v_cmp_u_f32;                                                         \
+      info->swapped = aco_opcode::v_cmp_u_f##sz;                                                   \
       info->inverse = aco_opcode::v_cmp_o_f##sz;                                                   \
+      info->vcmpx = aco_opcode::v_cmpx_u_f##sz;                                                    \
       info->size = sz;                                                                             \
       return true;                                                                                 \
    case aco_opcode::v_cmp_o_f##sz:                                                                 \
       info->f32 = aco_opcode::v_cmp_o_f32;                                                         \
+      info->swapped = aco_opcode::v_cmp_o_f##sz;                                                   \
       info->inverse = aco_opcode::v_cmp_u_f##sz;                                                   \
+      info->vcmpx = aco_opcode::v_cmpx_o_f##sz;                                                    \
       info->size = sz;                                                                             \
       return true;
       ORD_TEST(16)
       ORD_TEST(32)
       ORD_TEST(64)
 #undef ORD_TEST
+#define CMPI2(op, swap, inv, type, sz)                                                             \
+   case aco_opcode::v_cmp_##op##_##type##sz:                                                       \
+      info->swapped = aco_opcode::v_cmp_##swap##_##type##sz;                                       \
+      info->inverse = aco_opcode::v_cmp_##inv##_##type##sz;                                        \
+      info->vcmpx = aco_opcode::v_cmpx_##op##_##type##sz;                                          \
+      info->size = sz;                                                                             \
+      return true;
+#define CMPI(op, swap, inv)                                                                        \
+   CMPI2(op, swap, inv, i, 16)                                                                     \
+   CMPI2(op, swap, inv, u, 16)                                                                     \
+   CMPI2(op, swap, inv, i, 32)                                                                     \
+   CMPI2(op, swap, inv, u, 32)                                                                     \
+   CMPI2(op, swap, inv, i, 64)                                                                     \
+   CMPI2(op, swap, inv, u, 64)
+      CMPI(lt, gt, ge)
+      CMPI(eq, eq, lg)
+      CMPI(le, ge, gt)
+      CMPI(gt, lt, le)
+      CMPI(lg, lg, eq)
+      CMPI(ge, le, lt)
+#undef CMPI
+#undef CMPI2
+#define CMPCLASS(sz)                                                                               \
+   case aco_opcode::v_cmp_class_f##sz:                                                             \
+      info->vcmpx = aco_opcode::v_cmpx_class_f##sz;                                                \
+      info->size = sz;                                                                             \
+      return true;
+      CMPCLASS(16)
+      CMPCLASS(32)
+      CMPCLASS(64)
+#undef CMPCLASS
       // clang-format on
    default: return false;
    }
@@ -656,6 +731,13 @@ get_f32_cmp(aco_opcode op)
    return get_cmp_info(op, &info) ? info.f32 : aco_opcode::num_opcodes;
 }
 
+aco_opcode
+get_vcmpx(aco_opcode op)
+{
+   CmpInfo info;
+   return get_cmp_info(op, &info) ? info.vcmpx : aco_opcode::num_opcodes;
+}
+
 unsigned
 get_cmp_bitsize(aco_opcode op)
 {
@@ -715,13 +797,8 @@ can_swap_operands(aco_ptr<Instruction>& instr, aco_opcode* new_op)
    case aco_opcode::v_sub_u32: *new_op = aco_opcode::v_subrev_u32; return true;
    default: {
       CmpInfo info;
-      get_cmp_info(instr->opcode, &info);
-      if (info.ordered == instr->opcode) {
-         *new_op = info.ordered_swapped;
-         return true;
-      }
-      if (info.unordered == instr->opcode) {
-         *new_op = info.unordered_swapped;
+      if (get_cmp_info(instr->opcode, &info) && info.swapped != aco_opcode::num_opcodes) {
+         *new_op = info.swapped;
          return true;
       }
       return false;
@@ -735,25 +812,30 @@ wait_imm::wait_imm(uint16_t vm_, uint16_t exp_, uint16_t lgkm_, uint16_t vs_)
     : vm(vm_), exp(exp_), lgkm(lgkm_), vs(vs_)
 {}
 
-wait_imm::wait_imm(enum chip_class chip, uint16_t packed) : vs(unset_counter)
+wait_imm::wait_imm(enum amd_gfx_level gfx_level, uint16_t packed) : vs(unset_counter)
 {
    vm = packed & 0xf;
-   if (chip >= GFX9)
+   if (gfx_level >= GFX9)
       vm |= (packed >> 10) & 0x30;
 
    exp = (packed >> 4) & 0x7;
 
    lgkm = (packed >> 8) & 0xf;
-   if (chip >= GFX10)
+   if (gfx_level >= GFX10)
       lgkm |= (packed >> 8) & 0x30;
 }
 
 uint16_t
-wait_imm::pack(enum chip_class chip) const
+wait_imm::pack(enum amd_gfx_level gfx_level) const
 {
    uint16_t imm = 0;
    assert(exp == unset_counter || exp <= 0x7);
-   switch (chip) {
+   switch (gfx_level) {
+   case GFX11:
+      assert(lgkm == unset_counter || lgkm <= 0x3f);
+      assert(vm == unset_counter || vm <= 0x3f);
+      imm = ((vm & 0x3f) << 10) | ((lgkm & 0x3f) << 4) | (exp & 0x7);
+      break;
    case GFX10:
    case GFX10_3:
       assert(lgkm == unset_counter || lgkm <= 0x3f);
@@ -771,10 +853,10 @@ wait_imm::pack(enum chip_class chip) const
       imm = ((lgkm & 0xf) << 8) | ((exp & 0x7) << 4) | (vm & 0xf);
       break;
    }
-   if (chip < GFX9 && vm == wait_imm::unset_counter)
+   if (gfx_level < GFX9 && vm == wait_imm::unset_counter)
       imm |= 0xc000; /* should have no effect on pre-GFX9 and now we won't have to worry about the
                         architecture when interpreting the immediate */
-   if (chip < GFX10 && lgkm == wait_imm::unset_counter)
+   if (gfx_level < GFX10 && lgkm == wait_imm::unset_counter)
       imm |= 0x3000; /* should have no effect on pre-GFX10 and now we won't have to worry about the
                         architecture when interpreting the immediate */
    return imm;
