@@ -42,6 +42,9 @@
 struct radeon_info;
 struct ac_surf_info;
 struct radeon_surf;
+struct vk_sync_type;
+struct vk_sync_wait;
+struct vk_sync_signal;
 
 enum radeon_bo_domain { /* bitfield */
                         RADEON_DOMAIN_GTT = 2,
@@ -63,6 +66,7 @@ enum radeon_bo_flag { /* bitfield */
                       RADEON_FLAG_32BIT = (1 << 8),
                       RADEON_FLAG_PREFER_LOCAL_BO = (1 << 9),
                       RADEON_FLAG_ZERO_VRAM = (1 << 10),
+                      RADEON_FLAG_REPLAYABLE = (1 << 11),
 };
 
 enum radeon_ctx_priority {
@@ -73,6 +77,14 @@ enum radeon_ctx_priority {
    RADEON_CTX_PRIORITY_REALTIME,
 };
 
+enum radeon_ctx_pstate {
+   RADEON_CTX_PSTATE_NONE = 0,
+   RADEON_CTX_PSTATE_STANDARD,
+   RADEON_CTX_PSTATE_MIN_SCLK,
+   RADEON_CTX_PSTATE_MIN_MCLK,
+   RADEON_CTX_PSTATE_PEAK,
+};
+
 enum radeon_value_id {
    RADEON_ALLOCATED_VRAM,
    RADEON_ALLOCATED_VRAM_VIS,
@@ -168,26 +180,21 @@ struct radeon_winsys_bo {
    bool use_global_list;
    enum radeon_bo_domain initial_domain;
 };
-struct radv_winsys_sem_counts {
-   uint32_t syncobj_count;
-   uint32_t syncobj_reset_count; /* for wait only, whether to reset the syncobj */
-   uint32_t timeline_syncobj_count;
-   uint32_t *syncobj;
-   uint64_t *points;
-};
-
-struct radv_winsys_sem_info {
-   bool cs_emit_signal;
-   bool cs_emit_wait;
-   struct radv_winsys_sem_counts wait;
-   struct radv_winsys_sem_counts signal;
-};
 
 struct radv_winsys_bo_list {
    struct radeon_winsys_bo **bos;
    unsigned count;
 };
 
+struct radv_winsys_submit_info {
+   enum amd_ip_type ip_type;
+   int queue_index;
+   unsigned cs_count;
+   struct radeon_cmdbuf **cs_array;
+   struct radeon_cmdbuf *initial_preamble_cs;
+   struct radeon_cmdbuf *continue_preamble_cs;
+};
+
 /* Kernel effectively allows 0-31. This sets some priorities for fixed
  * functionality buffers */
 enum {
@@ -222,18 +229,18 @@ struct radeon_winsys {
 
    const char *(*get_chip_name)(struct radeon_winsys *ws);
 
-   struct radeon_winsys_bo *(*buffer_create)(struct radeon_winsys *ws, uint64_t size,
-                                             unsigned alignment, enum radeon_bo_domain domain,
-                                             enum radeon_bo_flag flags, unsigned priority);
+   VkResult (*buffer_create)(struct radeon_winsys *ws, uint64_t size, unsigned alignment,
+                             enum radeon_bo_domain domain, enum radeon_bo_flag flags,
+                             unsigned priority, uint64_t address, struct radeon_winsys_bo **out_bo);
 
    void (*buffer_destroy)(struct radeon_winsys *ws, struct radeon_winsys_bo *bo);
    void *(*buffer_map)(struct radeon_winsys_bo *bo);
 
-   struct radeon_winsys_bo *(*buffer_from_ptr)(struct radeon_winsys *ws, void *pointer,
-                                               uint64_t size, unsigned priority);
+   VkResult (*buffer_from_ptr)(struct radeon_winsys *ws, void *pointer, uint64_t size,
+                               unsigned priority, struct radeon_winsys_bo **out_bo);
 
-   struct radeon_winsys_bo *(*buffer_from_fd)(struct radeon_winsys *ws, int fd, unsigned priority,
-                                              uint64_t *alloc_size);
+   VkResult (*buffer_from_fd)(struct radeon_winsys *ws, int fd, unsigned priority,
+                              struct radeon_winsys_bo **out_bo, uint64_t *alloc_size);
 
    bool (*buffer_get_fd)(struct radeon_winsys *ws, struct radeon_winsys_bo *bo, int *fd);
 
@@ -258,11 +265,13 @@ struct radeon_winsys {
                           struct radeon_winsys_ctx **ctx);
    void (*ctx_destroy)(struct radeon_winsys_ctx *ctx);
 
-   bool (*ctx_wait_idle)(struct radeon_winsys_ctx *ctx, enum ring_type ring_type, int ring_index);
+   bool (*ctx_wait_idle)(struct radeon_winsys_ctx *ctx, enum amd_ip_type amd_ip_type, int ring_index);
+
+   int (*ctx_set_pstate)(struct radeon_winsys_ctx *ctx, uint32_t pstate);
 
    enum radeon_bo_domain (*cs_domain)(const struct radeon_winsys *ws);
 
-   struct radeon_cmdbuf *(*cs_create)(struct radeon_winsys *ws, enum ring_type ring_type);
+   struct radeon_cmdbuf *(*cs_create)(struct radeon_winsys *ws, enum amd_ip_type amd_ip_type);
 
    void (*cs_destroy)(struct radeon_cmdbuf *cs);
 
@@ -272,14 +281,15 @@ struct radeon_winsys {
 
    void (*cs_grow)(struct radeon_cmdbuf *cs, size_t min_size);
 
-   VkResult (*cs_submit)(struct radeon_winsys_ctx *ctx, int queue_index,
-                         struct radeon_cmdbuf **cs_array, unsigned cs_count,
-                         struct radeon_cmdbuf *initial_preamble_cs,
-                         struct radeon_cmdbuf *continue_preamble_cs,
-                         struct radv_winsys_sem_info *sem_info, bool can_patch);
+   VkResult (*cs_submit)(struct radeon_winsys_ctx *ctx, uint32_t submit_count,
+                         const struct radv_winsys_submit_info *submits, uint32_t wait_count,
+                         const struct vk_sync_wait *waits, uint32_t signal_count,
+                         const struct vk_sync_signal *signals, bool can_patch);
 
    void (*cs_add_buffer)(struct radeon_cmdbuf *cs, struct radeon_winsys_bo *bo);
 
+   void (*cs_add_buffers)(struct radeon_cmdbuf *to, struct radeon_cmdbuf *from);
+
    void (*cs_execute_secondary)(struct radeon_cmdbuf *parent, struct radeon_cmdbuf *child,
                                 bool allow_ib2);
 
@@ -292,25 +302,9 @@ struct radeon_winsys {
    int (*surface_init)(struct radeon_winsys *ws, const struct ac_surf_info *surf_info,
                        struct radeon_surf *surf);
 
-   int (*create_syncobj)(struct radeon_winsys *ws, bool create_signaled, uint32_t *handle);
-   void (*destroy_syncobj)(struct radeon_winsys *ws, uint32_t handle);
+   int (*get_fd)(struct radeon_winsys *ws);
 
-   void (*reset_syncobj)(struct radeon_winsys *ws, uint32_t handle);
-   void (*signal_syncobj)(struct radeon_winsys *ws, uint32_t handle, uint64_t point);
-   VkResult (*query_syncobj)(struct radeon_winsys *ws, uint32_t handle, uint64_t *point);
-   bool (*wait_syncobj)(struct radeon_winsys *ws, const uint32_t *handles, uint32_t handle_count,
-                        bool wait_all, uint64_t timeout);
-   bool (*wait_timeline_syncobj)(struct radeon_winsys *ws, const uint32_t *handles,
-                                 const uint64_t *points, uint32_t handle_count, bool wait_all,
-                                 bool available, uint64_t timeout);
-
-   int (*export_syncobj)(struct radeon_winsys *ws, uint32_t syncobj, int *fd);
-   int (*import_syncobj)(struct radeon_winsys *ws, int fd, uint32_t *syncobj);
-
-   int (*export_syncobj_to_sync_file)(struct radeon_winsys *ws, uint32_t syncobj, int *fd);
-
-   /* Note that this, unlike the normal import, uses an existing syncobj. */
-   int (*import_syncobj_from_sync_file)(struct radeon_winsys *ws, uint32_t syncobj, int fd);
+   const struct vk_sync_type *const *(*get_sync_types)(struct radeon_winsys *ws);
 };
 
 static inline void
