@@ -38,7 +38,6 @@
 #include "glsl_parser_extras.h"
 #include "glsl_parser.h"
 #include "ir_optimization.h"
-#include "loop_analysis.h"
 #include "builtin_functions.h"
 
 /**
@@ -61,7 +60,8 @@ static const unsigned known_desktop_gl_versions[] =
 _mesa_glsl_parse_state::_mesa_glsl_parse_state(struct gl_context *_ctx,
 					       gl_shader_stage stage,
                                                void *mem_ctx)
-   : ctx(_ctx), cs_input_local_size_specified(false), cs_input_local_size(),
+   : ctx(_ctx), exts(&_ctx->Extensions), consts(&_ctx->Const),
+     api(_ctx->API), cs_input_local_size_specified(false), cs_input_local_size(),
      switch_state(), warnings_enabled(true)
 {
    assert(stage < MESA_SHADER_STAGES);
@@ -417,10 +417,10 @@ _mesa_glsl_parse_state::set_valid_gl_and_glsl_versions(YYLTYPE *locp)
        * Later calls to _mesa_glsl_initialize_types will misbehave if
        * the version is invalid.
        */
-      switch (this->ctx->API) {
+      switch (this->api) {
       case API_OPENGL_COMPAT:
       case API_OPENGL_CORE:
-	 this->language_version = this->ctx->Const.GLSLVersion;
+	 this->language_version = this->consts->GLSLVersion;
 	 break;
 
       case API_OPENGLES:
@@ -458,7 +458,8 @@ _mesa_glsl_parse_state::process_version_directive(YYLTYPE *locp, int version,
          } else if (strcmp(ident, "compatibility") == 0) {
             compat_token_present = true;
 
-            if (this->ctx->API != API_OPENGL_COMPAT) {
+            if (this->api != API_OPENGL_COMPAT &&
+                !this->consts->AllowGLSLCompatShaders) {
                _mesa_glsl_error(locp, this,
                                 "the compatibility profile is not supported");
             }
@@ -494,7 +495,8 @@ _mesa_glsl_parse_state::process_version_directive(YYLTYPE *locp, int version,
       this->language_version = version;
 
    this->compat_shader = compat_token_present ||
-                         (this->ctx->API == API_OPENGL_COMPAT &&
+                         this->consts->ForceCompatShaders ||
+                         (this->api == API_OPENGL_COMPAT &&
                           this->language_version == 140) ||
                          (!this->es_shader && this->language_version < 140);
 
@@ -596,7 +598,7 @@ struct _mesa_glsl_extension {
     * Predicate that checks whether the relevant extension is available for
     * this context.
     */
-   bool (*available_pred)(const struct gl_context *,
+   bool (*available_pred)(const struct gl_extensions *,
                           gl_api api, uint8_t version);
 
    /**
@@ -626,9 +628,9 @@ struct _mesa_glsl_extension {
 /** Checks if the context supports a user-facing extension */
 #define EXT(name_str, driver_cap, ...) \
 static UNUSED bool \
-has_##name_str(const struct gl_context *ctx, gl_api api, uint8_t version) \
+has_##name_str(const struct gl_extensions *exts, gl_api api, uint8_t version) \
 { \
-   return ctx->Extensions.driver_cap && (version >= \
+   return exts->driver_cap && (version >= \
           _mesa_extension_table[MESA_EXTENSION_##name_str].version[api]); \
 }
 #include "main/extensions_table.h"
@@ -694,6 +696,8 @@ static const _mesa_glsl_extension _mesa_glsl_supported_extensions[] = {
    EXT(ARB_shading_language_420pack),
    EXT(ARB_shading_language_include),
    EXT(ARB_shading_language_packing),
+   EXT(ARB_sparse_texture2),
+   EXT(ARB_sparse_texture_clamp),
    EXT(ARB_tessellation_shader),
    EXT(ARB_texture_cube_map_array),
    EXT(ARB_texture_gather),
@@ -740,6 +744,7 @@ static const _mesa_glsl_extension _mesa_glsl_supported_extensions[] = {
    EXT(AMD_vertex_shader_layer),
    EXT(AMD_vertex_shader_viewport_index),
    EXT(ANDROID_extension_pack_es31a),
+   EXT(ARM_shader_framebuffer_fetch_depth_stencil),
    EXT(EXT_blend_func_extended),
    EXT(EXT_demote_to_helper_invocation),
    EXT(EXT_frag_depth),
@@ -790,7 +795,7 @@ static const _mesa_glsl_extension _mesa_glsl_supported_extensions[] = {
 bool _mesa_glsl_extension::compatible_with_state(
       const _mesa_glsl_parse_state *state, gl_api api, uint8_t gl_version) const
 {
-   return this->available_pred(state->ctx, api, gl_version);
+   return this->available_pred(state->exts, api, gl_version);
 }
 
 /**
@@ -827,8 +832,8 @@ _mesa_glsl_process_extension(const char *name, YYLTYPE *name_locp,
 			     const char *behavior_string, YYLTYPE *behavior_locp,
 			     _mesa_glsl_parse_state *state)
 {
-   uint8_t gl_version = state->ctx->Extensions.Version;
-   gl_api api = state->ctx->API;
+   uint8_t gl_version = state->exts->Version;
+   gl_api api = state->api;
    ext_behavior behavior;
    if (strcmp(behavior_string, "warn") == 0) {
       behavior = extension_warn;
@@ -874,7 +879,10 @@ _mesa_glsl_process_extension(const char *name, YYLTYPE *name_locp,
       }
    } else {
       const _mesa_glsl_extension *extension = find_extension(name);
-      if (extension && extension->compatible_with_state(state, api, gl_version)) {
+      if (extension &&
+          (extension->compatible_with_state(state, api, gl_version) ||
+           (state->consts->AllowGLSLCompatShaders &&
+            extension->compatible_with_state(state, API_OPENGL_COMPAT, gl_version)))) {
          extension->set_flags(state, behavior);
          if (extension->available_pred == has_ANDROID_extension_pack_es31a) {
             for (unsigned i = 0;
@@ -1595,6 +1603,7 @@ ast_switch_statement::ast_switch_statement(ast_expression *test_expression,
 {
    this->test_expression = test_expression;
    this->body = body;
+   this->test_val = NULL;
 }
 
 
@@ -1822,9 +1831,21 @@ set_shader_inout_layout(struct gl_shader *shader,
       }
       break;
    case MESA_SHADER_TESS_EVAL:
-      shader->info.TessEval.PrimitiveMode = PRIM_UNKNOWN;
-      if (state->in_qualifier->flags.q.prim_type)
-         shader->info.TessEval.PrimitiveMode = state->in_qualifier->prim_type;
+      shader->OES_tessellation_point_size_enable = state->OES_tessellation_point_size_enable || state->EXT_tessellation_point_size_enable;
+      shader->info.TessEval._PrimitiveMode = TESS_PRIMITIVE_UNSPECIFIED;
+      if (state->in_qualifier->flags.q.prim_type) {
+         switch (state->in_qualifier->prim_type) {
+         case GL_TRIANGLES:
+            shader->info.TessEval._PrimitiveMode = TESS_PRIMITIVE_TRIANGLES;
+            break;
+         case GL_QUADS:
+            shader->info.TessEval._PrimitiveMode = TESS_PRIMITIVE_QUADS;
+            break;
+         case GL_ISOLINES:
+            shader->info.TessEval._PrimitiveMode = TESS_PRIMITIVE_ISOLINES;
+            break;
+         }
+      }
 
       shader->info.TessEval.Spacing = TESS_SPACING_UNSPECIFIED;
       if (state->in_qualifier->flags.q.vertex_spacing)
@@ -1839,6 +1860,7 @@ set_shader_inout_layout(struct gl_shader *shader,
          shader->info.TessEval.PointMode = state->in_qualifier->point_mode;
       break;
    case MESA_SHADER_GEOMETRY:
+      shader->OES_geometry_point_size_enable = state->OES_geometry_point_size_enable || state->EXT_geometry_point_size_enable;
       shader->info.Geom.VerticesOut = -1;
       if (state->out_qualifier->flags.q.max_vertices) {
          unsigned qual_max_vertices;
@@ -1858,15 +1880,15 @@ set_shader_inout_layout(struct gl_shader *shader,
       }
 
       if (state->gs_input_prim_type_specified) {
-         shader->info.Geom.InputType = state->in_qualifier->prim_type;
+         shader->info.Geom.InputType = (enum shader_prim)state->in_qualifier->prim_type;
       } else {
-         shader->info.Geom.InputType = PRIM_UNKNOWN;
+         shader->info.Geom.InputType = SHADER_PRIM_UNKNOWN;
       }
 
       if (state->out_qualifier->flags.q.prim_type) {
-         shader->info.Geom.OutputType = state->out_qualifier->prim_type;
+         shader->info.Geom.OutputType = (enum shader_prim)state->out_qualifier->prim_type;
       } else {
-         shader->info.Geom.OutputType = PRIM_UNKNOWN;
+         shader->info.Geom.OutputType = SHADER_PRIM_UNKNOWN;
       }
 
       shader->info.Geom.Invocations = 0;
@@ -2033,8 +2055,8 @@ add_builtin_defines(struct _mesa_glsl_parse_state *state,
                     unsigned version,
                     bool es)
 {
-   unsigned gl_version = state->ctx->Extensions.Version;
-   gl_api api = state->ctx->API;
+   unsigned gl_version = state->exts->Version;
+   gl_api api = state->api;
 
    if (gl_version != 0xff) {
       unsigned i;
@@ -2076,29 +2098,22 @@ do_late_parsing_checks(struct _mesa_glsl_parse_state *state)
 }
 
 static void
-opt_shader_and_create_symbol_table(struct gl_context *ctx,
+opt_shader_and_create_symbol_table(const struct gl_constants *consts,
                                    struct glsl_symbol_table *source_symbols,
                                    struct gl_shader *shader)
 {
    assert(shader->CompileStatus != COMPILE_FAILURE &&
           !shader->ir->is_empty());
 
-   struct gl_shader_compiler_options *options =
-      &ctx->Const.ShaderCompilerOptions[shader->Stage];
+   const struct gl_shader_compiler_options *options =
+      &consts->ShaderCompilerOptions[shader->Stage];
 
    /* Do some optimization at compile time to reduce shader IR size
-    * and reduce later work if the same shader is linked multiple times
+    * and reduce later work if the same shader is linked multiple times.
+    *
+    * Run it just once, since NIR will do the real optimization.
     */
-   if (ctx->Const.GLSLOptimizeConservatively) {
-      /* Run it just once. */
-      do_common_optimization(shader->ir, false, false, options,
-                             ctx->Const.NativeIntegers);
-   } else {
-      /* Repeat it until it stops making changes. */
-      while (do_common_optimization(shader->ir, false, false, options,
-                                    ctx->Const.NativeIntegers))
-         ;
-   }
+   do_common_optimization(shader->ir, false, options, consts->NativeIntegers);
 
    validate_ir_tree(shader->ir);
 
@@ -2141,18 +2156,19 @@ opt_shader_and_create_symbol_table(struct gl_context *ctx,
 
 static bool
 can_skip_compile(struct gl_context *ctx, struct gl_shader *shader,
-                 const char *source, bool force_recompile,
-                 bool source_has_shader_include)
+                 const char *source,
+                 const uint8_t source_sha1[SHA1_DIGEST_LENGTH],
+                 bool force_recompile, bool source_has_shader_include)
 {
    if (!force_recompile) {
       if (ctx->Cache) {
          char buf[41];
          disk_cache_compute_key(ctx->Cache, source, strlen(source),
-                                shader->sha1);
-         if (disk_cache_has_key(ctx->Cache, shader->sha1)) {
+                                shader->disk_cache_sha1);
+         if (disk_cache_has_key(ctx->Cache, shader->disk_cache_sha1)) {
             /* We've seen this shader before and know it compiles */
             if (ctx->_Shader->Flags & GLSL_CACHE_INFO) {
-               _mesa_sha1_format(buf, shader->sha1);
+               _mesa_sha1_format(buf, shader->disk_cache_sha1);
                fprintf(stderr, "deferring compile of shader: %s\n", buf);
             }
             shader->CompileStatus = COMPILE_SKIPPED;
@@ -2163,8 +2179,15 @@ can_skip_compile(struct gl_context *ctx, struct gl_shader *shader,
              * we have no guarantee the shader include source tree has not
              * changed.
              */
-            shader->FallbackSource = source_has_shader_include ?
-               strdup(source) : NULL;
+            if (source_has_shader_include) {
+               shader->FallbackSource = strdup(source);
+               memcpy(shader->fallback_source_sha1, source_sha1,
+                      SHA1_DIGEST_LENGTH);
+            } else {
+               shader->FallbackSource = NULL;
+            }
+            memcpy(shader->compiled_source_sha1, source_sha1,
+                   SHA1_DIGEST_LENGTH);
             return true;
          }
       }
@@ -2184,8 +2207,16 @@ void
 _mesa_glsl_compile_shader(struct gl_context *ctx, struct gl_shader *shader,
                           bool dump_ast, bool dump_hir, bool force_recompile)
 {
-   const char *source = force_recompile && shader->FallbackSource ?
-      shader->FallbackSource : shader->Source;
+   const char *source;
+   const uint8_t *source_sha1;
+
+   if (force_recompile && shader->FallbackSource) {
+      source = shader->FallbackSource;
+      source_sha1 = shader->fallback_source_sha1;
+   } else {
+      source = shader->Source;
+      source_sha1 = shader->source_sha1;
+   }
 
    /* Note this will be true for shaders the have #include inside comments
     * however that should be rare enough not to worry about.
@@ -2199,7 +2230,8 @@ _mesa_glsl_compile_shader(struct gl_context *ctx, struct gl_shader *shader,
     * keep duplicate copies of the shader include source tree and paths.
     */
    if (!source_has_shader_include &&
-       can_skip_compile(ctx, shader, source, force_recompile, false))
+       can_skip_compile(ctx, shader, source, source_sha1, force_recompile,
+                        false))
       return;
 
     struct _mesa_glsl_parse_state *state =
@@ -2219,7 +2251,8 @@ _mesa_glsl_compile_shader(struct gl_context *ctx, struct gl_shader *shader,
     * include.
     */
    if (source_has_shader_include &&
-       can_skip_compile(ctx, shader, source, force_recompile, true))
+       can_skip_compile(ctx, shader, source, source_sha1, force_recompile,
+                        true))
       return;
 
    if (!state->error) {
@@ -2272,7 +2305,7 @@ _mesa_glsl_compile_shader(struct gl_context *ctx, struct gl_shader *shader,
       lower_builtins(shader->ir);
       assign_subroutine_indexes(state);
       lower_subroutine(shader->ir, state);
-      opt_shader_and_create_symbol_table(ctx, state->symbols, shader);
+      opt_shader_and_create_symbol_table(&ctx->Const, state->symbols, shader);
    }
 
    if (!force_recompile) {
@@ -2281,18 +2314,25 @@ _mesa_glsl_compile_shader(struct gl_context *ctx, struct gl_shader *shader,
       /* Copy pre-processed shader include to fallback source otherwise we
        * have no guarantee the shader include source tree has not changed.
        */
-      shader->FallbackSource = source_has_shader_include ?
-         strdup(source) : NULL;
+      if (source_has_shader_include) {
+         shader->FallbackSource = strdup(source);
+         memcpy(shader->fallback_source_sha1, source_sha1, SHA1_DIGEST_LENGTH);
+      } else {
+         shader->FallbackSource = NULL;
+      }
    }
 
    delete state->symbols;
    ralloc_free(state);
 
+   if (shader->CompileStatus == COMPILE_SUCCESS)
+      memcpy(shader->compiled_source_sha1, source_sha1, SHA1_DIGEST_LENGTH);
+
    if (ctx->Cache && shader->CompileStatus == COMPILE_SUCCESS) {
       char sha1_buf[41];
-      disk_cache_put_key(ctx->Cache, shader->sha1);
+      disk_cache_put_key(ctx->Cache, shader->disk_cache_sha1);
       if (ctx->_Shader->Flags & GLSL_CACHE_INFO) {
-         _mesa_sha1_format(sha1_buf, shader->sha1);
+         _mesa_sha1_format(sha1_buf, shader->disk_cache_sha1);
          fprintf(stderr, "marking shader: %s\n", sha1_buf);
       }
    }
@@ -2320,7 +2360,6 @@ _mesa_glsl_compile_shader(struct gl_context *ctx, struct gl_shader *shader,
  */
 bool
 do_common_optimization(exec_list *ir, bool linked,
-		       bool uniform_locations_assigned,
                        const struct gl_shader_compiler_options *options,
                        bool native_integers)
 {
@@ -2351,18 +2390,13 @@ do_common_optimization(exec_list *ir, bool linked,
    OPT(propagate_invariance, ir);
    OPT(do_if_simplification, ir);
    OPT(opt_flatten_nested_if_blocks, ir);
-   OPT(opt_conditional_discard, ir);
    OPT(do_copy_propagation_elements, ir);
 
    if (options->OptimizeForAOS && !linked)
       OPT(opt_flip_matrices, ir);
 
-   if (linked && options->OptimizeForAOS) {
-      OPT(do_vectorize, ir);
-   }
-
    if (linked)
-      OPT(do_dead_code, ir, uniform_locations_assigned);
+      OPT(do_dead_code, ir);
    else
       OPT(do_dead_code_unlinked, ir);
    OPT(do_dead_code_local, ir);
@@ -2377,10 +2411,9 @@ do_common_optimization(exec_list *ir, bool linked,
    OPT(do_rebalance_tree, ir);
    OPT(do_algebraic, ir, native_integers, options);
    OPT(do_lower_jumps, ir, true, true, options->EmitNoMainReturn,
-       options->EmitNoCont, options->EmitNoLoops);
+       options->EmitNoCont);
    OPT(do_vec_index_to_swizzle, ir);
    OPT(lower_vector_insert, ir, false);
-   OPT(optimize_swizzles, ir);
 
    /* Some drivers only call do_common_optimization() once rather than in a
     * loop, and split arrays causes each element of a constant array to
@@ -2396,46 +2429,8 @@ do_common_optimization(exec_list *ir, bool linked,
       do_constant_propagation(ir);
    progress |= array_split;
 
-   OPT(optimize_redundant_jumps, ir);
-
-   if (options->MaxUnrollIterations) {
-      loop_state *ls = analyze_loop_variables(ir);
-      if (ls->loop_found) {
-         bool loop_progress = unroll_loops(ir, ls, options);
-         while (loop_progress) {
-            loop_progress = false;
-            loop_progress |= do_constant_propagation(ir);
-            loop_progress |= do_if_simplification(ir);
-
-            /* Some drivers only call do_common_optimization() once rather
-             * than in a loop. So we must call do_lower_jumps() after
-             * unrolling a loop because for drivers that use LLVM validation
-             * will fail if a jump is not the last instruction in the block.
-             * For example the following will fail LLVM validation:
-             *
-             *   (loop (
-             *      ...
-             *   break
-             *   (assign  (x) (var_ref v124)  (expression int + (var_ref v124)
-             *      (constant int (1)) ) )
-             *   ))
-             */
-            loop_progress |= do_lower_jumps(ir, true, true,
-                                            options->EmitNoMainReturn,
-                                            options->EmitNoCont,
-                                            options->EmitNoLoops);
-         }
-         progress |= loop_progress;
-      }
-      delete ls;
-   }
-
-   /* If the PIPE_CAP_GLSL_OPTIMIZE_CONSERVATIVELY cap is set, this pass will
-    * only be called once rather than repeatedly until no further progress is
-    * made.
-    *
-    * If an optimization pass fails to preserve the invariant flag, calling
-    * the pass only once may result in incorrect code generation. Always call
+   /* If an optimization pass fails to preserve the invariant flag, calling
+    * the pass only once earlier may result in incorrect code generation. Always call
     * propagate_invariance() last to avoid this possibility.
     */
    OPT(propagate_invariance, ir);
