@@ -25,6 +25,8 @@
 #include "nir_deref.h"
 #include "main/menums.h"
 
+#include "util/set.h"
+
 static bool
 src_is_invocation_id(const nir_src *src)
 {
@@ -36,6 +38,17 @@ src_is_invocation_id(const nir_src *src)
              nir_intrinsic_load_invocation_id;
 }
 
+static bool
+src_is_local_invocation_index(const nir_src *src)
+{
+   assert(src->is_ssa);
+   if (src->ssa->parent_instr->type != nir_instr_type_intrinsic)
+      return false;
+
+   return nir_instr_as_intrinsic(src->ssa->parent_instr)->intrinsic ==
+             nir_intrinsic_load_local_invocation_index;
+}
+
 static void
 get_deref_info(nir_shader *shader, nir_variable *var, nir_deref_instr *deref,
                bool *cross_invocation, bool *indirect)
@@ -53,7 +66,10 @@ get_deref_info(nir_shader *shader, nir_variable *var, nir_deref_instr *deref,
    /* Vertex index is the outermost array index. */
    if (is_arrayed) {
       assert((*p)->deref_type == nir_deref_type_array);
-      *cross_invocation = !src_is_invocation_id(&(*p)->arr.index);
+      if (shader->info.stage == MESA_SHADER_TESS_CTRL)
+         *cross_invocation = !src_is_invocation_id(&(*p)->arr.index);
+      else if (shader->info.stage == MESA_SHADER_MESH)
+         *cross_invocation = !src_is_local_invocation_index(&(*p)->arr.index);
       p++;
    }
 
@@ -79,7 +95,9 @@ set_io_mask(nir_shader *shader, nir_variable *var, int offset, int len,
             nir_deref_instr *deref, bool is_output_read)
 {
    for (int i = 0; i < len; i++) {
-      assert(var->data.location != -1);
+      /* Varyings might not have been assigned values yet so abort. */
+      if (var->data.location == -1)
+         return;
 
       int idx = var->data.location + offset + i;
       bool is_patch_generic = var->data.patch &&
@@ -90,11 +108,17 @@ set_io_mask(nir_shader *shader, nir_variable *var, int offset, int len,
       uint64_t bitfield;
 
       if (is_patch_generic) {
-         assert(idx >= VARYING_SLOT_PATCH0 && idx < VARYING_SLOT_TESS_MAX);
+         /* Varyings might still have temp locations so abort */
+         if (idx < VARYING_SLOT_PATCH0 || idx >= VARYING_SLOT_TESS_MAX)
+            return;
+
          bitfield = BITFIELD64_BIT(idx - VARYING_SLOT_PATCH0);
       }
       else {
-         assert(idx < VARYING_SLOT_MAX);
+         /* Varyings might still have temp locations so abort */
+         if (idx >= VARYING_SLOT_MAX)
+            return;
+
          bitfield = BITFIELD64_BIT(idx);
       }
 
@@ -146,6 +170,8 @@ set_io_mask(nir_shader *shader, nir_variable *var, int offset, int len,
             }
          }
 
+         if (cross_invocation && shader->info.stage == MESA_SHADER_MESH)
+            shader->info.mesh.ms_cross_invocation_output_access |= bitfield;
 
          if (var->data.fb_fetch_output) {
             shader->info.outputs_read |= bitfield;
@@ -173,7 +199,8 @@ mark_whole_variable(nir_shader *shader, nir_variable *var,
    if (nir_is_arrayed_io(var, shader->info.stage) ||
        /* For NV_mesh_shader. */
        (shader->info.stage == MESA_SHADER_MESH &&
-        var->data.location == VARYING_SLOT_PRIMITIVE_INDICES)) {
+        var->data.location == VARYING_SLOT_PRIMITIVE_INDICES &&
+        !var->data.per_primitive)) {
       assert(glsl_type_is_array(type));
       type = glsl_get_array_element(type);
    }
@@ -195,6 +222,10 @@ get_io_offset(nir_deref_instr *deref, nir_variable *var, bool is_arrayed,
               bool skip_non_arrayed)
 {
    if (var->data.compact) {
+      if (deref->deref_type == nir_deref_type_var) {
+         assert(glsl_type_is_array(var->type));
+         return 0;
+      }
       assert(deref->deref_type == nir_deref_type_array);
       return nir_src_is_const(deref->arr.index) ?
              (nir_src_as_uint(deref->arr.index) + var->data.location_frac) / 4u :
@@ -404,6 +435,7 @@ nir_intrinsic_writes_external_memory(const nir_intrinsic_instr *instr)
    case nir_intrinsic_ssbo_atomic_xor_ir3:
    case nir_intrinsic_store_global:
    case nir_intrinsic_store_global_ir3:
+   case nir_intrinsic_store_global_amd:
    case nir_intrinsic_store_ssbo:
    case nir_intrinsic_store_ssbo_ir3:
       return true;
@@ -490,7 +522,8 @@ gather_intrinsic_info(nir_intrinsic_instr *instr, nir_shader *shader,
    case nir_intrinsic_interp_deref_at_offset:
    case nir_intrinsic_interp_deref_at_vertex:
    case nir_intrinsic_load_deref:
-   case nir_intrinsic_store_deref:{
+   case nir_intrinsic_store_deref:
+   case nir_intrinsic_copy_deref:{
       nir_deref_instr *deref = nir_src_as_deref(instr->src[0]);
       if (nir_deref_mode_is_one_of(deref, nir_var_shader_in |
                                           nir_var_shader_out)) {
@@ -518,6 +551,18 @@ gather_intrinsic_info(nir_intrinsic_instr *instr, nir_shader *shader,
          shader->info.writes_memory = true;
       break;
    }
+   case nir_intrinsic_image_deref_load: {
+      nir_deref_instr *deref = nir_src_as_deref(instr->src[0]);
+      nir_variable *var = nir_deref_instr_get_variable(deref);
+      enum glsl_sampler_dim dim = glsl_get_sampler_dim(glsl_without_array(var->type));
+      if (dim != GLSL_SAMPLER_DIM_SUBPASS &&
+          dim != GLSL_SAMPLER_DIM_SUBPASS_MS)
+         break;
+
+      var->data.fb_fetch_output = true;
+      shader->info.fs.uses_fbfetch_output = true;
+      break;
+   }
 
    case nir_intrinsic_load_input:
    case nir_intrinsic_load_per_vertex_input:
@@ -565,6 +610,13 @@ gather_intrinsic_info(nir_intrinsic_instr *instr, nir_shader *shader,
           !src_is_invocation_id(nir_get_io_arrayed_index_src(instr)))
          shader->info.tess.tcs_cross_invocation_outputs_read |= slot_mask;
 
+      /* NV_mesh_shader: mesh shaders can load their outputs. */
+      if (shader->info.stage == MESA_SHADER_MESH &&
+          (instr->intrinsic == nir_intrinsic_load_per_vertex_output ||
+           instr->intrinsic == nir_intrinsic_load_per_primitive_output) &&
+          !src_is_local_invocation_index(nir_get_io_arrayed_index_src(instr)))
+         shader->info.mesh.ms_cross_invocation_output_access |= slot_mask;
+
       if (shader->info.stage == MESA_SHADER_FRAGMENT &&
           nir_intrinsic_io_semantics(instr).fb_fetch_output)
          shader->info.fs.uses_fbfetch_output = true;
@@ -587,6 +639,12 @@ gather_intrinsic_info(nir_intrinsic_instr *instr, nir_shader *shader,
          }
       }
 
+      if (shader->info.stage == MESA_SHADER_MESH &&
+          (instr->intrinsic == nir_intrinsic_store_per_vertex_output ||
+           instr->intrinsic == nir_intrinsic_store_per_primitive_output) &&
+          !src_is_local_invocation_index(nir_get_io_arrayed_index_src(instr)))
+         shader->info.mesh.ms_cross_invocation_output_access |= slot_mask;
+
       if (shader->info.stage == MESA_SHADER_FRAGMENT &&
           nir_intrinsic_io_semantics(instr).dual_source_blend_index)
          shader->info.fs.color_is_dual_source = true;
@@ -639,12 +697,29 @@ gather_intrinsic_info(nir_intrinsic_instr *instr, nir_shader *shader,
    case nir_intrinsic_load_base_global_invocation_id:
    case nir_intrinsic_load_global_invocation_index:
    case nir_intrinsic_load_workgroup_id:
+   case nir_intrinsic_load_workgroup_index:
    case nir_intrinsic_load_num_workgroups:
    case nir_intrinsic_load_workgroup_size:
    case nir_intrinsic_load_work_dim:
    case nir_intrinsic_load_user_data_amd:
    case nir_intrinsic_load_view_index:
    case nir_intrinsic_load_barycentric_model:
+   case nir_intrinsic_load_ray_launch_id:
+   case nir_intrinsic_load_ray_launch_size:
+   case nir_intrinsic_load_ray_launch_size_addr_amd:
+   case nir_intrinsic_load_ray_world_origin:
+   case nir_intrinsic_load_ray_world_direction:
+   case nir_intrinsic_load_ray_object_origin:
+   case nir_intrinsic_load_ray_object_direction:
+   case nir_intrinsic_load_ray_t_min:
+   case nir_intrinsic_load_ray_t_max:
+   case nir_intrinsic_load_ray_object_to_world:
+   case nir_intrinsic_load_ray_world_to_object:
+   case nir_intrinsic_load_ray_hit_kind:
+   case nir_intrinsic_load_ray_flags:
+   case nir_intrinsic_load_ray_geometry_index:
+   case nir_intrinsic_load_ray_instance_custom_index:
+   case nir_intrinsic_load_mesh_view_count:
    case nir_intrinsic_load_gs_header_ir3:
    case nir_intrinsic_load_tcs_header_ir3:
       BITSET_SET(shader->info.system_values_read,
@@ -815,24 +890,37 @@ gather_alu_info(nir_alu_instr *instr, nir_shader *shader)
 }
 
 static void
-gather_info_block(nir_block *block, nir_shader *shader, void *dead_ctx)
+gather_func_info(nir_function_impl *func, nir_shader *shader,
+                 struct set *visited_funcs, void *dead_ctx)
 {
-   nir_foreach_instr(instr, block) {
-      switch (instr->type) {
-      case nir_instr_type_alu:
-         gather_alu_info(nir_instr_as_alu(instr), shader);
-         break;
-      case nir_instr_type_intrinsic:
-         gather_intrinsic_info(nir_instr_as_intrinsic(instr), shader, dead_ctx);
-         break;
-      case nir_instr_type_tex:
-         gather_tex_info(nir_instr_as_tex(instr), shader);
-         break;
-      case nir_instr_type_call:
-         assert(!"nir_shader_gather_info only works if functions are inlined");
-         break;
-      default:
-         break;
+   if (_mesa_set_search(visited_funcs, func))
+      return;
+
+   _mesa_set_add(visited_funcs, func);
+
+   nir_foreach_block(block, func) {
+      nir_foreach_instr(instr, block) {
+         switch (instr->type) {
+         case nir_instr_type_alu:
+            gather_alu_info(nir_instr_as_alu(instr), shader);
+            break;
+         case nir_instr_type_intrinsic:
+            gather_intrinsic_info(nir_instr_as_intrinsic(instr), shader, dead_ctx);
+            break;
+         case nir_instr_type_tex:
+            gather_tex_info(nir_instr_as_tex(instr), shader);
+            break;
+         case nir_instr_type_call: {
+            nir_call_instr *call = nir_instr_as_call(instr);
+            nir_function_impl *impl = call->callee->impl;
+
+            assert(impl || !"nir_shader_gather_info only works with linked shaders");
+            gather_func_info(impl, shader, visited_funcs, dead_ctx);
+            break;
+         }
+         default:
+            break;
+         }
       }
    }
 }
@@ -842,8 +930,6 @@ nir_shader_gather_info(nir_shader *shader, nir_function_impl *entrypoint)
 {
    shader->info.num_textures = 0;
    shader->info.num_images = 0;
-   shader->info.image_buffers = 0;
-   shader->info.msaa_images = 0;
    shader->info.bit_sizes_float = 0;
    shader->info.bit_sizes_int = 0;
 
@@ -856,21 +942,7 @@ nir_shader_gather_info(nir_shader *shader, nir_function_impl *entrypoint)
          continue;
 
       shader->info.num_textures += glsl_type_get_sampler_count(var->type);
-
-      unsigned num_image_slots = glsl_type_get_image_count(var->type);
-      if (num_image_slots) {
-         const struct glsl_type *image_type = glsl_without_array(var->type);
-
-         if (glsl_get_sampler_dim(image_type) == GLSL_SAMPLER_DIM_BUF) {
-            shader->info.image_buffers |=
-               BITFIELD_RANGE(shader->info.num_images, num_image_slots);
-         }
-         if (glsl_get_sampler_dim(image_type) == GLSL_SAMPLER_DIM_MS) {
-            shader->info.msaa_images |=
-               BITFIELD_RANGE(shader->info.num_images, num_image_slots);
-         }
-         shader->info.num_images += num_image_slots;
-      }
+      shader->info.num_images += glsl_type_get_image_count(var->type);
    }
 
    shader->info.inputs_read = 0;
@@ -906,26 +978,18 @@ nir_shader_gather_info(nir_shader *shader, nir_function_impl *entrypoint)
       shader->info.tess.tcs_cross_invocation_inputs_read = 0;
       shader->info.tess.tcs_cross_invocation_outputs_read = 0;
    }
+   if (shader->info.stage == MESA_SHADER_MESH) {
+      shader->info.mesh.ms_cross_invocation_output_access = 0;
+   }
 
-   shader->info.writes_memory = shader->info.has_transform_feedback_varyings;
+   if (shader->info.stage != MESA_SHADER_FRAGMENT)
+      shader->info.writes_memory = shader->info.has_transform_feedback_varyings;
 
    void *dead_ctx = ralloc_context(NULL);
-   nir_foreach_block(block, entrypoint) {
-      gather_info_block(block, shader, dead_ctx);
-   }
+   struct set *visited_funcs = _mesa_pointer_set_create(dead_ctx);
+   gather_func_info(entrypoint, shader, visited_funcs, dead_ctx);
    ralloc_free(dead_ctx);
 
-   if (shader->info.stage == MESA_SHADER_FRAGMENT &&
-       (shader->info.fs.uses_sample_qualifier ||
-        (BITSET_TEST(shader->info.system_values_read, SYSTEM_VALUE_SAMPLE_ID) ||
-         BITSET_TEST(shader->info.system_values_read, SYSTEM_VALUE_SAMPLE_POS)))) {
-      /* This shouldn't be cleared because if optimizations remove all
-       * sample-qualified inputs and that pass is run again, the sample
-       * shading must stay enabled.
-       */
-      shader->info.fs.uses_sample_shading = true;
-   }
-
    shader->info.per_primitive_outputs = 0;
    if (shader->info.stage == MESA_SHADER_MESH) {
       nir_foreach_shader_out_variable(var, shader) {
