@@ -23,6 +23,7 @@
 
 #include "nir.h"
 #include "nir_builder.h"
+#include "util/u_math.h"
 
 /**
  * \file nir_opt_intrinsics.c
@@ -44,7 +45,7 @@ lower_subgroups_64bit_split_intrinsic(nir_builder *b, nir_intrinsic_instr *intri
    intr->const_index[1] = intrin->const_index[1];
    intr->src[0] = nir_src_for_ssa(comp);
    if (nir_intrinsic_infos[intrin->intrinsic].num_srcs == 2)
-      nir_src_copy(&intr->src[1], &intrin->src[1], intr);
+      nir_src_copy(&intr->src[1], &intrin->src[1]);
 
    intr->num_components = 1;
    nir_builder_instr_insert(b, &intr->instr);
@@ -61,54 +62,45 @@ lower_subgroup_op_to_32bit(nir_builder *b, nir_intrinsic_instr *intrin)
 }
 
 static nir_ssa_def *
-ballot_type_to_uint(nir_builder *b, nir_ssa_def *value, unsigned bit_size)
+ballot_type_to_uint(nir_builder *b, nir_ssa_def *value,
+                    const nir_lower_subgroups_options *options)
 {
-   /* We only use this on uvec4 types */
+   /* Only the new-style SPIR-V subgroup instructions take a ballot result as
+    * an argument, so we only use this on uvec4 types.
+    */
    assert(value->num_components == 4 && value->bit_size == 32);
 
-   if (bit_size == 32) {
-      return nir_channel(b, value, 0);
-   } else {
-      assert(bit_size == 64);
-      return nir_pack_64_2x32_split(b, nir_channel(b, value, 0),
-                                       nir_channel(b, value, 1));
-   }
+   return nir_extract_bits(b, &value, 1, 0, options->ballot_components,
+                           options->ballot_bit_size);
 }
 
-/* Converts a uint32_t or uint64_t value to uint64_t or uvec4 */
 static nir_ssa_def *
 uint_to_ballot_type(nir_builder *b, nir_ssa_def *value,
                     unsigned num_components, unsigned bit_size)
 {
-   assert(value->num_components == 1);
-   assert(value->bit_size == 32 || value->bit_size == 64);
+   assert(util_is_power_of_two_nonzero(num_components));
+   assert(util_is_power_of_two_nonzero(value->num_components));
 
-   nir_ssa_def *zero = nir_imm_int(b, 0);
-   if (num_components > 1) {
-      /* SPIR-V uses a uvec4 for ballot values */
-      assert(num_components == 4);
-      assert(bit_size == 32);
+   unsigned total_bits = bit_size * num_components;
 
-      if (value->bit_size == 32) {
-         return nir_vec4(b, value, zero, zero, zero);
-      } else {
-         assert(value->bit_size == 64);
-         return nir_vec4(b, nir_unpack_64_2x32_split_x(b, value),
-                            nir_unpack_64_2x32_split_y(b, value),
-                            zero, zero);
-      }
-   } else {
-      /* GLSL uses a uint64_t for ballot values */
-      assert(num_components == 1);
-      assert(bit_size == 64);
+   /* If the source doesn't have enough bits, zero-pad */
+   if (total_bits > value->bit_size * value->num_components)
+      value = nir_pad_vector_imm_int(b, value, 0, total_bits / value->bit_size);
 
-      if (value->bit_size == 32) {
-         return nir_pack_64_2x32_split(b, value, zero);
-      } else {
-         assert(value->bit_size == 64);
-         return value;
-      }
-   }
+   value = nir_bitcast_vector(b, value, bit_size);
+
+   /* If the source has too many components, truncate.  This can happen if,
+    * for instance, we're implementing GL_ARB_shader_ballot or
+    * VK_EXT_shader_subgroup_ballot which have 64-bit ballot values on an
+    * architecture with a native 128-bit uvec4 ballot.  This comes up in Zink
+    * for OpenGL on Vulkan.  It's the job of the driver calling this lowering
+    * pass to ensure that it's restricted subgroup sizes sufficiently that we
+    * have enough ballot bits.
+    */
+   if (value->num_components > num_components)
+      value = nir_trim_vector(b, value, num_components);
+
+   return value;
 }
 
 static nir_ssa_def *
@@ -134,7 +126,7 @@ lower_subgroup_op_to_scalar(nir_builder *b, nir_intrinsic_instr *intrin,
       /* invocation */
       if (nir_intrinsic_infos[intrin->intrinsic].num_srcs > 1) {
          assert(nir_intrinsic_infos[intrin->intrinsic].num_srcs == 2);
-         nir_src_copy(&chan_intrin->src[1], &intrin->src[1], chan_intrin);
+         nir_src_copy(&chan_intrin->src[1], &intrin->src[1]);
       }
 
       chan_intrin->const_index[0] = intrin->const_index[0];
@@ -178,8 +170,7 @@ lower_vote_eq_to_scalar(nir_builder *b, nir_intrinsic_instr *intrin)
 }
 
 static nir_ssa_def *
-lower_vote_eq_to_ballot(nir_builder *b, nir_intrinsic_instr *intrin,
-                        const nir_lower_subgroups_options *options)
+lower_vote_eq(nir_builder *b, nir_intrinsic_instr *intrin)
 {
    assert(intrin->src[0].is_ssa);
    nir_ssa_def *value = intrin->src[0].ssa;
@@ -187,20 +178,13 @@ lower_vote_eq_to_ballot(nir_builder *b, nir_intrinsic_instr *intrin,
    /* We have to implicitly lower to scalar */
    nir_ssa_def *all_eq = NULL;
    for (unsigned i = 0; i < intrin->num_components; i++) {
-      nir_intrinsic_instr *rfi =
-         nir_intrinsic_instr_create(b->shader,
-                                    nir_intrinsic_read_first_invocation);
-      nir_ssa_dest_init(&rfi->instr, &rfi->dest,
-                        1, value->bit_size, NULL);
-      rfi->num_components = 1;
-      rfi->src[0] = nir_src_for_ssa(nir_channel(b, value, i));
-      nir_builder_instr_insert(b, &rfi->instr);
+      nir_ssa_def *rfi = nir_read_first_invocation(b, nir_channel(b, value, i));
 
       nir_ssa_def *is_eq;
       if (intrin->intrinsic == nir_intrinsic_vote_feq) {
-         is_eq = nir_feq(b, &rfi->dest.ssa, nir_channel(b, value, i));
+         is_eq = nir_feq(b, rfi, nir_channel(b, value, i));
       } else {
-         is_eq = nir_ieq(b, &rfi->dest.ssa, nir_channel(b, value, i));
+         is_eq = nir_ieq(b, rfi, nir_channel(b, value, i));
       }
 
       if (all_eq == NULL) {
@@ -210,16 +194,7 @@ lower_vote_eq_to_ballot(nir_builder *b, nir_intrinsic_instr *intrin,
       }
    }
 
-   nir_intrinsic_instr *ballot =
-      nir_intrinsic_instr_create(b->shader, nir_intrinsic_ballot);
-   nir_ssa_dest_init(&ballot->instr, &ballot->dest,
-                     1, options->ballot_bit_size, NULL);
-   ballot->num_components = 1;
-   ballot->src[0] = nir_src_for_ssa(nir_inot(b, all_eq));
-   nir_builder_instr_insert(b, &ballot->instr);
-
-   return nir_ieq(b, &ballot->dest.ssa,
-                  nir_imm_intN_t(b, 0, options->ballot_bit_size));
+   return nir_vote_all(b, 1, all_eq);
 }
 
 static nir_ssa_def *
@@ -234,7 +209,7 @@ lower_shuffle_to_swizzle(nir_builder *b, nir_intrinsic_instr *intrin,
    nir_intrinsic_instr *swizzle = nir_intrinsic_instr_create(
       b->shader, nir_intrinsic_masked_swizzle_amd);
    swizzle->num_components = intrin->num_components;
-   nir_src_copy(&swizzle->src[0], &intrin->src[0], swizzle);
+   nir_src_copy(&swizzle->src[0], &intrin->src[0]);
    nir_intrinsic_set_swizzle_mask(swizzle, (mask << 10) | 0x1f);
    nir_ssa_dest_init(&swizzle->instr, &swizzle->dest,
                      intrin->dest.ssa.num_components,
@@ -250,9 +225,11 @@ lower_shuffle_to_swizzle(nir_builder *b, nir_intrinsic_instr *intrin,
    }
 }
 
+/* Lowers "specialized" shuffles to a generic nir_intrinsic_shuffle. */
+
 static nir_ssa_def *
-lower_shuffle(nir_builder *b, nir_intrinsic_instr *intrin,
-              const nir_lower_subgroups_options *options)
+lower_to_shuffle(nir_builder *b, nir_intrinsic_instr *intrin,
+                 const nir_lower_subgroups_options *options)
 {
    if (intrin->intrinsic == nir_intrinsic_shuffle_xor &&
        options->lower_shuffle_to_swizzle_amd &&
@@ -311,7 +288,7 @@ lower_shuffle(nir_builder *b, nir_intrinsic_instr *intrin,
    nir_intrinsic_instr *shuffle =
       nir_intrinsic_instr_create(b->shader, nir_intrinsic_shuffle);
    shuffle->num_components = intrin->num_components;
-   nir_src_copy(&shuffle->src[0], &intrin->src[0], shuffle);
+   nir_src_copy(&shuffle->src[0], &intrin->src[0]);
    shuffle->src[1] = nir_src_for_ssa(index);
    nir_ssa_dest_init(&shuffle->instr, &shuffle->dest,
                      intrin->dest.ssa.num_components,
@@ -328,19 +305,252 @@ lower_shuffle(nir_builder *b, nir_intrinsic_instr *intrin,
    }
 }
 
+static const struct glsl_type *
+glsl_type_for_ssa(nir_ssa_def *def)
+{
+   const struct glsl_type *comp_type = def->bit_size == 1 ? glsl_bool_type() :
+      glsl_uintN_t_type(def->bit_size);
+   return glsl_replace_vector_type(comp_type, def->num_components);
+}
+
+/* Lower nir_intrinsic_shuffle to a waterfall loop + nir_read_invocation.
+ */
+static nir_ssa_def *
+lower_shuffle(nir_builder *b, nir_intrinsic_instr *intrin)
+{
+   assert(intrin->src[0].is_ssa);
+   assert(intrin->src[1].is_ssa);
+   nir_ssa_def *val = intrin->src[0].ssa;
+   nir_ssa_def *id = intrin->src[1].ssa;
+
+   /* The loop is something like:
+    *
+    * while (true) {
+    *    first_id = readFirstInvocation(gl_SubgroupInvocationID);
+    *    first_val = readFirstInvocation(val);
+    *    first_result = readInvocation(val, readFirstInvocation(id));
+    *    if (id == first_id)
+    *       result = first_val;
+    *    if (elect()) {
+    *       if (id > gl_SubgroupInvocationID) {
+    *          result = first_result;
+    *       }
+    *       break;
+    *    }
+    * }
+    *
+    * The idea is to guarantee, on each iteration of the loop, that anything
+    * reading from first_id gets the correct value, so that we can then kill
+    * it off by breaking out of the loop. Before doing that we also have to
+    * ensure that first_id invocation gets the correct value. It only won't be
+    * assigned the correct value already if the invocation it's reading from
+    * isn't already killed off, that is, if it's later than its own ID.
+    * Invocations where id <= gl_SubgroupInvocationID will be assigned their
+    * result in the first if, and invocations where id >
+    * gl_SubgroupInvocationID will be assigned their result in the second if.
+    *
+    * We do this more complicated loop rather than looping over all id's
+    * explicitly because at this point we don't know the "actual" subgroup
+    * size and at the moment there's no way to get at it, which means we may
+    * loop over always-inactive invocations.
+    */
+
+   nir_ssa_def *subgroup_id = nir_load_subgroup_invocation(b);
+
+   nir_variable *result =
+      nir_local_variable_create(b->impl, glsl_type_for_ssa(val), "result");
+
+   nir_loop *loop = nir_push_loop(b); {
+      nir_ssa_def *first_id = nir_read_first_invocation(b, subgroup_id);
+      nir_ssa_def *first_val = nir_read_first_invocation(b, val);
+      nir_ssa_def *first_result =
+         nir_read_invocation(b, val, nir_read_first_invocation(b, id));
+
+      nir_if *nif = nir_push_if(b, nir_ieq(b, id, first_id)); {
+         nir_store_var(b, result, first_val, BITFIELD_MASK(val->num_components));
+      } nir_pop_if(b, nif);
+
+      nir_if *nif2 = nir_push_if(b, nir_elect(b, 1)); {
+         nir_if *nif3 = nir_push_if(b, nir_ult(b, subgroup_id, id)); {
+            nir_store_var(b, result, first_result, BITFIELD_MASK(val->num_components));
+         } nir_pop_if(b, nif3);
+
+         nir_jump(b, nir_jump_break);
+      } nir_pop_if(b, nif2);
+   } nir_pop_loop(b, loop);
+
+   return nir_load_var(b, result);
+}
+
 static bool
 lower_subgroups_filter(const nir_instr *instr, const void *_options)
 {
    return instr->type == nir_instr_type_intrinsic;
 }
 
+/* Return a ballot-mask-sized value which represents "val" sign-extended and
+ * then shifted left by "shift". Only particular values for "val" are
+ * supported, see below.
+ */
 static nir_ssa_def *
-build_subgroup_mask(nir_builder *b, unsigned bit_size,
+build_ballot_imm_ishl(nir_builder *b, int64_t val, nir_ssa_def *shift,
+                      const nir_lower_subgroups_options *options)
+{
+   /* This only works if all the high bits are the same as bit 1. */
+   assert((val >> 2) == (val & 0x2 ? -1 : 0));
+
+   /* First compute the result assuming one ballot component. */
+   nir_ssa_def *result =
+      nir_ishl(b, nir_imm_intN_t(b, val, options->ballot_bit_size), shift);
+
+   if (options->ballot_components == 1)
+      return result;
+
+   /* Fix up the result when there is > 1 component. The idea is that nir_ishl
+    * masks out the high bits of the shift value already, so in case there's
+    * more than one component the component which 1 would be shifted into
+    * already has the right value and all we have to do is fixup the other
+    * components. Components below it should always be 0, and components above
+    * it must be either 0 or ~0 because of the assert above. For example, if
+    * the target ballot size is 2 x uint32, and we're shifting 1 by 33, then
+    * we'll feed 33 into ishl, which will mask it off to get 1, so we'll
+    * compute a single-component result of 2, which is correct for the second
+    * component, but the first component needs to be 0, which we get by
+    * comparing the high bits of the shift with 0 and selecting the original
+    * answer or 0 for the first component (and something similar with the
+    * second component). This idea is generalized here for any component count
+    */
+   nir_const_value min_shift[4] = { 0 };
+   for (unsigned i = 0; i < options->ballot_components; i++)
+      min_shift[i].i32 = i * options->ballot_bit_size;
+   nir_ssa_def *min_shift_val = nir_build_imm(b, options->ballot_components, 32, min_shift);
+
+   nir_const_value max_shift[4] = { 0 };
+   for (unsigned i = 0; i < options->ballot_components; i++)
+      max_shift[i].i32 = (i + 1) * options->ballot_bit_size;
+   nir_ssa_def *max_shift_val = nir_build_imm(b, options->ballot_components, 32, max_shift);
+
+   return nir_bcsel(b, nir_ult(b, shift, max_shift_val),
+                    nir_bcsel(b, nir_ult(b, shift, min_shift_val),
+                              nir_imm_intN_t(b, val >> 63, result->bit_size),
+                              result),
+                    nir_imm_intN_t(b, 0, result->bit_size));
+}
+
+static nir_ssa_def *
+build_subgroup_eq_mask(nir_builder *b,
+                       const nir_lower_subgroups_options *options)
+{
+   nir_ssa_def *subgroup_idx = nir_load_subgroup_invocation(b);
+
+   return build_ballot_imm_ishl(b, 1, subgroup_idx, options);
+}
+
+static nir_ssa_def *
+build_subgroup_ge_mask(nir_builder *b,
+                       const nir_lower_subgroups_options *options)
+{
+   nir_ssa_def *subgroup_idx = nir_load_subgroup_invocation(b);
+
+   return build_ballot_imm_ishl(b, ~0ull, subgroup_idx, options);
+}
+
+static nir_ssa_def *
+build_subgroup_gt_mask(nir_builder *b,
+                       const nir_lower_subgroups_options *options)
+{
+   nir_ssa_def *subgroup_idx = nir_load_subgroup_invocation(b);
+
+   return build_ballot_imm_ishl(b, ~1ull, subgroup_idx, options);
+}
+
+/* Return a mask which is 1 for threads up to the run-time subgroup size, i.e.
+ * 1 for the entire subgroup. SPIR-V requires us to return 0 for indices at or
+ * above the subgroup size for the masks, but gt_mask and ge_mask make them 1
+ * so we have to "and" with this mask.
+ */
+static nir_ssa_def *
+build_subgroup_mask(nir_builder *b,
                     const nir_lower_subgroups_options *options)
 {
-   return nir_ushr(b, nir_imm_intN_t(b, ~0ull, bit_size),
-                      nir_isub(b, nir_imm_int(b, bit_size),
-                                  nir_load_subgroup_size(b)));
+   nir_ssa_def *subgroup_size = nir_load_subgroup_size(b);
+
+   /* First compute the result assuming one ballot component. */
+   nir_ssa_def *result =
+      nir_ushr(b, nir_imm_intN_t(b, ~0ull, options->ballot_bit_size),
+                  nir_isub_imm(b, options->ballot_bit_size,
+                               subgroup_size));
+
+   /* Since the subgroup size and ballot bitsize are both powers of two, there
+    * are two possible cases to consider:
+    *
+    * (1) The subgroup size is less than the ballot bitsize. We need to return
+    * "result" in the first component and 0 in every other component.
+    * (2) The subgroup size is a multiple of the ballot bitsize. We need to
+    * return ~0 if the subgroup size divided by the ballot bitsize is less
+    * than or equal to the index in the vector and 0 otherwise. For example,
+    * with a target ballot type of 4 x uint32 and subgroup_size = 64 we'd need
+    * to return { ~0, ~0, 0, 0 }.
+    *
+    * In case (2) it turns out that "result" will be ~0, because
+    * "ballot_bit_size - subgroup_size" is also a multiple of
+    * "ballot_bit_size" and since nir_ushr masks the shift value it will
+    * shifted by 0. This means that the first component can just be "result"
+    * in all cases.  The other components will also get the correct value in
+    * case (1) if we just use the rule in case (2), so we'll get the correct
+    * result if we just follow (2) and then replace the first component with
+    * "result". 
+    */ 
+   nir_const_value min_idx[4] = { 0 };
+   for (unsigned i = 0; i < options->ballot_components; i++)
+      min_idx[i].i32 = i * options->ballot_bit_size;
+   nir_ssa_def *min_idx_val = nir_build_imm(b, options->ballot_components, 32, min_idx);
+
+   nir_ssa_def *result_extended =
+      nir_pad_vector_imm_int(b, result, ~0ull, options->ballot_components);
+
+   return nir_bcsel(b, nir_ult(b, min_idx_val, subgroup_size),
+                    result_extended, nir_imm_intN_t(b, 0, options->ballot_bit_size));
+}
+
+static nir_ssa_def *
+vec_bit_count(nir_builder *b, nir_ssa_def *value)
+{
+   nir_ssa_def *vec_result = nir_bit_count(b, value);
+   nir_ssa_def *result = nir_channel(b, vec_result, 0);
+   for (unsigned i = 1; i < value->num_components; i++)
+      result = nir_iadd(b, result, nir_channel(b, vec_result, i));
+   return result;
+}
+
+static nir_ssa_def *
+vec_find_lsb(nir_builder *b, nir_ssa_def *value)
+{
+   nir_ssa_def *vec_result = nir_find_lsb(b, value);
+   nir_ssa_def *result = nir_imm_int(b, -1);
+   for (int i = value->num_components - 1; i >= 0; i--) {
+      nir_ssa_def *channel = nir_channel(b, vec_result, i);
+      /* result = channel >= 0 ? (i * bitsize + channel) : result */
+      result = nir_bcsel(b, nir_ige(b, channel, nir_imm_int(b, 0)),
+                         nir_iadd_imm(b, channel, i * value->bit_size),
+                         result);
+   }
+   return result;
+}
+
+static nir_ssa_def *
+vec_find_msb(nir_builder *b, nir_ssa_def *value)
+{
+   nir_ssa_def *vec_result = nir_ufind_msb(b, value);
+   nir_ssa_def *result = nir_imm_int(b, -1);
+   for (unsigned i = 0; i < value->num_components; i++) {
+      nir_ssa_def *channel = nir_channel(b, vec_result, i);
+      /* result = channel >= 0 ? (i * bitsize + channel) : result */
+      result = nir_bcsel(b, nir_ige(b, channel, nir_imm_int(b, 0)),
+                         nir_iadd_imm(b, channel, i * value->bit_size),
+                         result);
+   }
+   return result;
 }
 
 static nir_ssa_def *
@@ -348,7 +558,7 @@ lower_dynamic_quad_broadcast(nir_builder *b, nir_intrinsic_instr *intrin,
                              const nir_lower_subgroups_options *options)
 {
    if (!options->lower_quad_broadcast_dynamic_to_const)
-      return lower_shuffle(b, intrin, options);
+      return lower_to_shuffle(b, intrin, options);
 
    nir_ssa_def *dst = NULL;
 
@@ -358,7 +568,7 @@ lower_dynamic_quad_broadcast(nir_builder *b, nir_intrinsic_instr *intrin,
 
       qbcst->num_components = intrin->num_components;
       qbcst->src[1] = nir_src_for_ssa(nir_imm_int(b, i));
-      nir_src_copy(&qbcst->src[0], &intrin->src[0], qbcst);
+      nir_src_copy(&qbcst->src[0], &intrin->src[0]);
       nir_ssa_dest_init(&qbcst->instr, &qbcst->dest,
                         intrin->dest.ssa.num_components,
                         intrin->dest.ssa.bit_size, NULL);
@@ -383,6 +593,15 @@ lower_dynamic_quad_broadcast(nir_builder *b, nir_intrinsic_instr *intrin,
    return dst;
 }
 
+static nir_ssa_def *
+lower_read_invocation_to_cond(nir_builder *b, nir_intrinsic_instr *intrin)
+{
+   return nir_read_invocation_cond_ir3(b, intrin->dest.ssa.bit_size,
+                                       intrin->src[0].ssa,
+                                       nir_ieq(b, intrin->src[1].ssa,
+                                               nir_load_subgroup_invocation(b)));
+}
+
 static nir_ssa_def *
 lower_subgroups_instr(nir_builder *b, nir_instr *instr, void *_options)
 {
@@ -401,8 +620,8 @@ lower_subgroups_instr(nir_builder *b, nir_instr *instr, void *_options)
       if (options->lower_vote_trivial)
          return nir_imm_true(b);
 
-      if (options->lower_vote_eq_to_ballot)
-         return lower_vote_eq_to_ballot(b, intrin, options);
+      if (options->lower_vote_eq)
+         return lower_vote_eq(b, intrin);
 
       if (options->lower_to_scalar && intrin->num_components > 1)
          return lower_vote_eq_to_scalar(b, intrin);
@@ -414,6 +633,14 @@ lower_subgroups_instr(nir_builder *b, nir_instr *instr, void *_options)
       break;
 
    case nir_intrinsic_read_invocation:
+      if (options->lower_to_scalar && intrin->num_components > 1)
+         return lower_subgroup_op_to_scalar(b, intrin, false);
+
+      if (options->lower_read_invocation_to_cond)
+         return lower_read_invocation_to_cond(b, intrin);
+
+      break;
+
    case nir_intrinsic_read_first_invocation:
       if (options->lower_to_scalar && intrin->num_components > 1)
          return lower_subgroup_op_to_scalar(b, intrin, false);
@@ -427,32 +654,24 @@ lower_subgroups_instr(nir_builder *b, nir_instr *instr, void *_options)
       if (!options->lower_subgroup_masks)
          return NULL;
 
-      /* If either the result or the requested bit size is 64-bits then we
-       * know that we have 64-bit types and using them will probably be more
-       * efficient than messing around with 32-bit shifts and packing.
-       */
-      const unsigned bit_size = MAX2(options->ballot_bit_size,
-                                     intrin->dest.ssa.bit_size);
-
-      nir_ssa_def *count = nir_load_subgroup_invocation(b);
       nir_ssa_def *val;
       switch (intrin->intrinsic) {
       case nir_intrinsic_load_subgroup_eq_mask:
-         val = nir_ishl(b, nir_imm_intN_t(b, 1ull, bit_size), count);
+         val = build_subgroup_eq_mask(b, options);
          break;
       case nir_intrinsic_load_subgroup_ge_mask:
-         val = nir_iand(b, nir_ishl(b, nir_imm_intN_t(b, ~0ull, bit_size), count),
-                           build_subgroup_mask(b, bit_size, options));
+         val = nir_iand(b, build_subgroup_ge_mask(b, options),
+                           build_subgroup_mask(b, options));
          break;
       case nir_intrinsic_load_subgroup_gt_mask:
-         val = nir_iand(b, nir_ishl(b, nir_imm_intN_t(b, ~1ull, bit_size), count),
-                           build_subgroup_mask(b, bit_size, options));
+         val = nir_iand(b, build_subgroup_gt_mask(b, options),
+                           build_subgroup_mask(b, options));
          break;
       case nir_intrinsic_load_subgroup_le_mask:
-         val = nir_inot(b, nir_ishl(b, nir_imm_intN_t(b, ~1ull, bit_size), count));
+         val = nir_inot(b, build_subgroup_gt_mask(b, options));
          break;
       case nir_intrinsic_load_subgroup_lt_mask:
-         val = nir_inot(b, nir_ishl(b, nir_imm_intN_t(b, ~0ull, bit_size), count));
+         val = nir_inot(b, build_subgroup_ge_mask(b, options));
          break;
       default:
          unreachable("you seriously can't tell this is unreachable?");
@@ -464,19 +683,15 @@ lower_subgroups_instr(nir_builder *b, nir_instr *instr, void *_options)
    }
 
    case nir_intrinsic_ballot: {
-      if (intrin->dest.ssa.num_components == 1 &&
+      if (intrin->dest.ssa.num_components == options->ballot_components &&
           intrin->dest.ssa.bit_size == options->ballot_bit_size)
          return NULL;
 
-      nir_intrinsic_instr *ballot =
-         nir_intrinsic_instr_create(b->shader, nir_intrinsic_ballot);
-      ballot->num_components = 1;
-      nir_ssa_dest_init(&ballot->instr, &ballot->dest,
-                        1, options->ballot_bit_size, NULL);
-      nir_src_copy(&ballot->src[0], &intrin->src[0], ballot);
-      nir_builder_instr_insert(b, &ballot->instr);
+      nir_ssa_def *ballot =
+         nir_ballot(b, options->ballot_components, options->ballot_bit_size,
+                    intrin->src[0].ssa);
 
-      return uint_to_ballot_type(b, &ballot->dest.ssa,
+      return uint_to_ballot_type(b, ballot,
                                  intrin->dest.ssa.num_components,
                                  intrin->dest.ssa.bit_size);
    }
@@ -487,7 +702,7 @@ lower_subgroups_instr(nir_builder *b, nir_instr *instr, void *_options)
    case nir_intrinsic_ballot_find_msb: {
       assert(intrin->src[0].is_ssa);
       nir_ssa_def *int_val = ballot_type_to_uint(b, intrin->src[0].ssa,
-                                                 options->ballot_bit_size);
+                                                 options);
 
       if (intrin->intrinsic != nir_intrinsic_ballot_bitfield_extract &&
           intrin->intrinsic != nir_intrinsic_ballot_find_lsb) {
@@ -510,22 +725,31 @@ lower_subgroups_instr(nir_builder *b, nir_instr *instr, void *_options)
           * valid bits, we hit the undefined results case and we can return
           * anything we want.
           */
-         int_val = nir_iand(b, int_val,
-            build_subgroup_mask(b, options->ballot_bit_size, options));
+         int_val = nir_iand(b, int_val, build_subgroup_mask(b, options));
       }
 
       switch (intrin->intrinsic) {
-      case nir_intrinsic_ballot_bitfield_extract:
+      case nir_intrinsic_ballot_bitfield_extract: {
          assert(intrin->src[1].is_ssa);
-         return nir_i2b(b, nir_iand(b, nir_ushr(b, int_val,
-                                                   intrin->src[1].ssa),
-                                       nir_imm_intN_t(b, 1, options->ballot_bit_size)));
+         nir_ssa_def *idx = intrin->src[1].ssa;
+         if (int_val->num_components > 1) {
+            /* idx will be truncated by nir_ushr, so we just need to select
+             * the right component using the bits of idx that are truncated in
+             * the shift.
+             */
+            int_val =
+               nir_vector_extract(b, int_val,
+                                  nir_udiv_imm(b, idx, int_val->bit_size));
+         }
+
+         return nir_test_mask(b, nir_ushr(b, int_val, idx), 1);
+      }
       case nir_intrinsic_ballot_bit_count_reduce:
-         return nir_bit_count(b, int_val);
+         return vec_bit_count(b, int_val);
       case nir_intrinsic_ballot_find_lsb:
-         return nir_find_lsb(b, int_val);
+         return vec_find_lsb(b, int_val);
       case nir_intrinsic_ballot_find_msb:
-         return nir_ufind_msb(b, int_val);
+         return vec_find_msb(b, int_val);
       default:
          unreachable("you seriously can't tell this is unreachable?");
       }
@@ -533,34 +757,31 @@ lower_subgroups_instr(nir_builder *b, nir_instr *instr, void *_options)
 
    case nir_intrinsic_ballot_bit_count_exclusive:
    case nir_intrinsic_ballot_bit_count_inclusive: {
-      nir_ssa_def *count = nir_load_subgroup_invocation(b);
-      nir_ssa_def *mask = nir_imm_intN_t(b, ~0ull, options->ballot_bit_size);
+      nir_ssa_def *mask;
       if (intrin->intrinsic == nir_intrinsic_ballot_bit_count_inclusive) {
-         const unsigned bits = options->ballot_bit_size;
-         mask = nir_ushr(b, mask, nir_isub(b, nir_imm_int(b, bits - 1), count));
+         mask = nir_inot(b, build_subgroup_gt_mask(b, options));
       } else {
-         mask = nir_inot(b, nir_ishl(b, mask, count));
+         mask = nir_inot(b, build_subgroup_ge_mask(b, options));
       }
 
       assert(intrin->src[0].is_ssa);
       nir_ssa_def *int_val = ballot_type_to_uint(b, intrin->src[0].ssa,
-                                                 options->ballot_bit_size);
+                                                 options);
 
-      return nir_bit_count(b, nir_iand(b, int_val, mask));
+      return vec_bit_count(b, nir_iand(b, int_val, mask));
    }
 
    case nir_intrinsic_elect: {
-      nir_intrinsic_instr *first =
-         nir_intrinsic_instr_create(b->shader,
-                                    nir_intrinsic_first_invocation);
-      nir_ssa_dest_init(&first->instr, &first->dest, 1, 32, NULL);
-      nir_builder_instr_insert(b, &first->instr);
+      if (!options->lower_elect)
+         return NULL;
 
-      return nir_ieq(b, nir_load_subgroup_invocation(b), &first->dest.ssa);
+      return nir_ieq(b, nir_load_subgroup_invocation(b), nir_first_invocation(b));
    }
 
    case nir_intrinsic_shuffle:
-      if (options->lower_to_scalar && intrin->num_components > 1)
+      if (options->lower_shuffle)
+         return lower_shuffle(b, intrin);
+      else if (options->lower_to_scalar && intrin->num_components > 1)
          return lower_subgroup_op_to_scalar(b, intrin, options->lower_shuffle_to_32bit);
       else if (options->lower_shuffle_to_32bit && intrin->src[0].ssa->bit_size == 64)
          return lower_subgroup_op_to_32bit(b, intrin);
@@ -568,8 +789,8 @@ lower_subgroups_instr(nir_builder *b, nir_instr *instr, void *_options)
    case nir_intrinsic_shuffle_xor:
    case nir_intrinsic_shuffle_up:
    case nir_intrinsic_shuffle_down:
-      if (options->lower_shuffle)
-         return lower_shuffle(b, intrin, options);
+      if (options->lower_relative_shuffle)
+         return lower_to_shuffle(b, intrin, options);
       else if (options->lower_to_scalar && intrin->num_components > 1)
          return lower_subgroup_op_to_scalar(b, intrin, options->lower_shuffle_to_32bit);
       else if (options->lower_shuffle_to_32bit && intrin->src[0].ssa->bit_size == 64)
