@@ -144,36 +144,36 @@ emit_intrinsic_atomic_ssbo(struct ir3_context *ctx, nir_intrinsic_instr *intr)
 
    switch (intr->intrinsic) {
    case nir_intrinsic_ssbo_atomic_add_ir3:
-      atomic = ir3_ATOMIC_ADD_G(b, ibo, 0, src0, 0, src1, 0);
+      atomic = ir3_ATOMIC_B_ADD(b, ibo, 0, src0, 0, src1, 0);
       break;
    case nir_intrinsic_ssbo_atomic_imin_ir3:
-      atomic = ir3_ATOMIC_MIN_G(b, ibo, 0, src0, 0, src1, 0);
+      atomic = ir3_ATOMIC_B_MIN(b, ibo, 0, src0, 0, src1, 0);
       type = TYPE_S32;
       break;
    case nir_intrinsic_ssbo_atomic_umin_ir3:
-      atomic = ir3_ATOMIC_MIN_G(b, ibo, 0, src0, 0, src1, 0);
+      atomic = ir3_ATOMIC_B_MIN(b, ibo, 0, src0, 0, src1, 0);
       break;
    case nir_intrinsic_ssbo_atomic_imax_ir3:
-      atomic = ir3_ATOMIC_MAX_G(b, ibo, 0, src0, 0, src1, 0);
+      atomic = ir3_ATOMIC_B_MAX(b, ibo, 0, src0, 0, src1, 0);
       type = TYPE_S32;
       break;
    case nir_intrinsic_ssbo_atomic_umax_ir3:
-      atomic = ir3_ATOMIC_MAX_G(b, ibo, 0, src0, 0, src1, 0);
+      atomic = ir3_ATOMIC_B_MAX(b, ibo, 0, src0, 0, src1, 0);
       break;
    case nir_intrinsic_ssbo_atomic_and_ir3:
-      atomic = ir3_ATOMIC_AND_G(b, ibo, 0, src0, 0, src1, 0);
+      atomic = ir3_ATOMIC_B_AND(b, ibo, 0, src0, 0, src1, 0);
       break;
    case nir_intrinsic_ssbo_atomic_or_ir3:
-      atomic = ir3_ATOMIC_OR_G(b, ibo, 0, src0, 0, src1, 0);
+      atomic = ir3_ATOMIC_B_OR(b, ibo, 0, src0, 0, src1, 0);
       break;
    case nir_intrinsic_ssbo_atomic_xor_ir3:
-      atomic = ir3_ATOMIC_XOR_G(b, ibo, 0, src0, 0, src1, 0);
+      atomic = ir3_ATOMIC_B_XOR(b, ibo, 0, src0, 0, src1, 0);
       break;
    case nir_intrinsic_ssbo_atomic_exchange_ir3:
-      atomic = ir3_ATOMIC_XCHG_G(b, ibo, 0, src0, 0, src1, 0);
+      atomic = ir3_ATOMIC_B_XCHG(b, ibo, 0, src0, 0, src1, 0);
       break;
    case nir_intrinsic_ssbo_atomic_comp_swap_ir3:
-      atomic = ir3_ATOMIC_CMPXCHG_G(b, ibo, 0, src0, 0, src1, 0);
+      atomic = ir3_ATOMIC_B_CMPXCHG(b, ibo, 0, src0, 0, src1, 0);
       break;
    default:
       unreachable("boo");
@@ -288,39 +288,39 @@ emit_intrinsic_atomic_image(struct ir3_context *ctx, nir_intrinsic_instr *intr)
    switch (intr->intrinsic) {
    case nir_intrinsic_image_atomic_add:
    case nir_intrinsic_bindless_image_atomic_add:
-      atomic = ir3_ATOMIC_ADD_G(b, ibo, 0, src0, 0, src1, 0);
+      atomic = ir3_ATOMIC_B_ADD(b, ibo, 0, src0, 0, src1, 0);
       break;
    case nir_intrinsic_image_atomic_imin:
    case nir_intrinsic_image_atomic_umin:
    case nir_intrinsic_bindless_image_atomic_imin:
    case nir_intrinsic_bindless_image_atomic_umin:
-      atomic = ir3_ATOMIC_MIN_G(b, ibo, 0, src0, 0, src1, 0);
+      atomic = ir3_ATOMIC_B_MIN(b, ibo, 0, src0, 0, src1, 0);
       break;
    case nir_intrinsic_image_atomic_imax:
    case nir_intrinsic_image_atomic_umax:
    case nir_intrinsic_bindless_image_atomic_imax:
    case nir_intrinsic_bindless_image_atomic_umax:
-      atomic = ir3_ATOMIC_MAX_G(b, ibo, 0, src0, 0, src1, 0);
+      atomic = ir3_ATOMIC_B_MAX(b, ibo, 0, src0, 0, src1, 0);
       break;
    case nir_intrinsic_image_atomic_and:
    case nir_intrinsic_bindless_image_atomic_and:
-      atomic = ir3_ATOMIC_AND_G(b, ibo, 0, src0, 0, src1, 0);
+      atomic = ir3_ATOMIC_B_AND(b, ibo, 0, src0, 0, src1, 0);
       break;
    case nir_intrinsic_image_atomic_or:
    case nir_intrinsic_bindless_image_atomic_or:
-      atomic = ir3_ATOMIC_OR_G(b, ibo, 0, src0, 0, src1, 0);
+      atomic = ir3_ATOMIC_B_OR(b, ibo, 0, src0, 0, src1, 0);
       break;
    case nir_intrinsic_image_atomic_xor:
    case nir_intrinsic_bindless_image_atomic_xor:
-      atomic = ir3_ATOMIC_XOR_G(b, ibo, 0, src0, 0, src1, 0);
+      atomic = ir3_ATOMIC_B_XOR(b, ibo, 0, src0, 0, src1, 0);
       break;
    case nir_intrinsic_image_atomic_exchange:
    case nir_intrinsic_bindless_image_atomic_exchange:
-      atomic = ir3_ATOMIC_XCHG_G(b, ibo, 0, src0, 0, src1, 0);
+      atomic = ir3_ATOMIC_B_XCHG(b, ibo, 0, src0, 0, src1, 0);
       break;
    case nir_intrinsic_image_atomic_comp_swap:
    case nir_intrinsic_bindless_image_atomic_comp_swap:
-      atomic = ir3_ATOMIC_CMPXCHG_G(b, ibo, 0, src0, 0, src1, 0);
+      atomic = ir3_ATOMIC_B_CMPXCHG(b, ibo, 0, src0, 0, src1, 0);
       break;
    default:
       unreachable("boo");
@@ -376,12 +376,23 @@ emit_intrinsic_load_global_ir3(struct ir3_context *ctx,
    addr = ir3_collect(b, ir3_get_src(ctx, &intr->src[0])[0],
                       ir3_get_src(ctx, &intr->src[0])[1]);
 
-   offset = ir3_get_src(ctx, &intr->src[1])[0];
+   struct ir3_instruction *load;
 
-   struct ir3_instruction *load =
-      ir3_LDG_A(b, addr, 0, offset, 0, create_immed(b, 0), 0,
-                create_immed(b, 0), 0, create_immed(b, dest_components), 0);
-   load->cat6.type = TYPE_U32;
+   bool const_offset_in_bounds = nir_src_is_const(intr->src[1]) &&
+                                 nir_src_as_int(intr->src[1]) < (1 << 13) &&
+                                 nir_src_as_int(intr->src[1]) > -(1 << 13);
+
+   if (const_offset_in_bounds) {
+      load = ir3_LDG(b, addr, 0, create_immed(b, nir_src_as_int(intr->src[1])),
+                     0, create_immed(b, dest_components), 0);
+   } else {
+      offset = ir3_get_src(ctx, &intr->src[1])[0];
+      load =
+         ir3_LDG_A(b, addr, 0, offset, 0, create_immed(b, 0), 0,
+                   create_immed(b, 0), 0, create_immed(b, dest_components), 0);
+   }
+
+   load->cat6.type = type_uint_size(intr->dest.ssa.bit_size);
    load->dsts[0]->wrmask = MASK(dest_components);
 
    load->barrier_class = IR3_BARRIER_BUFFER_R;
@@ -401,14 +412,27 @@ emit_intrinsic_store_global_ir3(struct ir3_context *ctx,
    addr = ir3_collect(b, ir3_get_src(ctx, &intr->src[1])[0],
                       ir3_get_src(ctx, &intr->src[1])[1]);
 
-   offset = ir3_get_src(ctx, &intr->src[2])[0];
-
    value = ir3_create_collect(b, ir3_get_src(ctx, &intr->src[0]), ncomp);
 
-   struct ir3_instruction *stg =
-      ir3_STG_A(b, addr, 0, offset, 0, create_immed(b, 0), 0,
-                create_immed(b, 0), 0, value, 0, create_immed(b, ncomp), 0);
-   stg->cat6.type = TYPE_U32;
+   struct ir3_instruction *stg;
+
+   bool const_offset_in_bounds = nir_src_is_const(intr->src[2]) &&
+                                 nir_src_as_int(intr->src[2]) < (1 << 13) &&
+                                 nir_src_as_int(intr->src[2]) > -(1 << 13);
+
+   if (const_offset_in_bounds) {
+      stg = ir3_STG(b, addr, 0,
+                    create_immed(b, nir_src_as_int(intr->src[2])), 0,
+                    value, 0,
+                    create_immed(b, ncomp), 0);
+   } else {
+      offset = ir3_get_src(ctx, &intr->src[2])[0];
+      stg =
+         ir3_STG_A(b, addr, 0, offset, 0, create_immed(b, 0), 0,
+                   create_immed(b, 0), 0, value, 0, create_immed(b, ncomp), 0);
+   }
+
+   stg->cat6.type = type_uint_size(intr->src[0].ssa->bit_size);
    stg->cat6.iim_val = 1;
 
    array_insert(b, b->keeps, stg);
@@ -417,6 +441,73 @@ emit_intrinsic_store_global_ir3(struct ir3_context *ctx,
    stg->barrier_conflict = IR3_BARRIER_BUFFER_R | IR3_BARRIER_BUFFER_W;
 }
 
+static struct ir3_instruction *
+emit_intrinsic_atomic_global(struct ir3_context *ctx, nir_intrinsic_instr *intr)
+{
+   struct ir3_block *b = ctx->block;
+   struct ir3_instruction *addr, *atomic, *src1;
+   struct ir3_instruction *value = ir3_get_src(ctx, &intr->src[1])[0];
+   type_t type = TYPE_U32;
+
+   addr = ir3_collect(b, ir3_get_src(ctx, &intr->src[0])[0],
+                      ir3_get_src(ctx, &intr->src[0])[1]);
+
+   if (intr->intrinsic == nir_intrinsic_global_atomic_comp_swap_ir3) {
+      struct ir3_instruction *compare = ir3_get_src(ctx, &intr->src[2])[0];
+      src1 = ir3_collect(b, compare, value);
+   } else {
+      src1 = value;
+   }
+
+   switch (intr->intrinsic) {
+   case nir_intrinsic_global_atomic_add_ir3:
+      atomic = ir3_ATOMIC_G_ADD(b, addr, 0, src1, 0);
+      break;
+   case nir_intrinsic_global_atomic_imin_ir3:
+      atomic = ir3_ATOMIC_G_MIN(b, addr, 0, src1, 0);
+      type = TYPE_S32;
+      break;
+   case nir_intrinsic_global_atomic_umin_ir3:
+      atomic = ir3_ATOMIC_G_MIN(b, addr, 0, src1, 0);
+      break;
+   case nir_intrinsic_global_atomic_imax_ir3:
+      atomic = ir3_ATOMIC_G_MAX(b, addr, 0, src1, 0);
+      type = TYPE_S32;
+      break;
+   case nir_intrinsic_global_atomic_umax_ir3:
+      atomic = ir3_ATOMIC_G_MAX(b, addr, 0, src1, 0);
+      break;
+   case nir_intrinsic_global_atomic_and_ir3:
+      atomic = ir3_ATOMIC_G_AND(b, addr, 0, src1, 0);
+      break;
+   case nir_intrinsic_global_atomic_or_ir3:
+      atomic = ir3_ATOMIC_G_OR(b, addr, 0, src1, 0);
+      break;
+   case nir_intrinsic_global_atomic_xor_ir3:
+      atomic = ir3_ATOMIC_G_XOR(b, addr, 0, src1, 0);
+      break;
+   case nir_intrinsic_global_atomic_exchange_ir3:
+      atomic = ir3_ATOMIC_G_XCHG(b, addr, 0, src1, 0);
+      break;
+   case nir_intrinsic_global_atomic_comp_swap_ir3:
+      atomic = ir3_ATOMIC_G_CMPXCHG(b, addr, 0, src1, 0);
+      break;
+   default:
+      unreachable("Unknown global atomic op");
+   }
+
+   atomic->cat6.iim_val = 1;
+   atomic->cat6.d = 1;
+   atomic->cat6.type = type;
+   atomic->barrier_class = IR3_BARRIER_BUFFER_W;
+   atomic->barrier_conflict = IR3_BARRIER_BUFFER_R | IR3_BARRIER_BUFFER_W;
+
+   /* even if nothing consume the result, we can't DCE the instruction: */
+   array_insert(b, b->keeps, atomic);
+
+   return atomic;
+}
+
 const struct ir3_context_funcs ir3_a6xx_funcs = {
    .emit_intrinsic_load_ssbo = emit_intrinsic_load_ssbo,
    .emit_intrinsic_store_ssbo = emit_intrinsic_store_ssbo,
@@ -427,4 +518,5 @@ const struct ir3_context_funcs ir3_a6xx_funcs = {
    .emit_intrinsic_image_size = emit_intrinsic_image_size,
    .emit_intrinsic_load_global_ir3 = emit_intrinsic_load_global_ir3,
    .emit_intrinsic_store_global_ir3 = emit_intrinsic_store_global_ir3,
+   .emit_intrinsic_atomic_global = emit_intrinsic_atomic_global,
 };
