@@ -29,7 +29,7 @@
 #include "pipe/p_compiler.h"
 #include "util/u_memory.h"
 #include "util/u_string.h"
-#include "util/u_format.h"
+#include "util/format/u_format.h"
 #include "tgsi/tgsi_dump.h"
 
 #include <inttypes.h>
@@ -48,7 +48,7 @@ util_stream_writef(FILE *stream, const char *format, ...)
    unsigned len;
    va_list ap;
    va_start(ap, format);
-   len = util_vsnprintf(buf, sizeof(buf), format, ap);
+   len = vsnprintf(buf, sizeof(buf), format, ap);
    va_end(ap);
    fwrite(buf, len, 1, stream);
 }
@@ -368,7 +368,8 @@ util_dump_rasterizer_state(FILE *stream, const struct pipe_rasterizer_state *sta
    util_dump_member(stream, bool, state, half_pixel_center);
    util_dump_member(stream, bool, state, bottom_edge_rule);
    util_dump_member(stream, bool, state, rasterizer_discard);
-   util_dump_member(stream, bool, state, depth_clip);
+   util_dump_member(stream, bool, state, depth_clip_near);
+   util_dump_member(stream, bool, state, depth_clip_far);
    util_dump_member(stream, bool, state, clip_halfz);
    util_dump_member(stream, uint, state, clip_plane_enable);
 
@@ -461,12 +462,37 @@ util_dump_clip_state(FILE *stream, const struct pipe_clip_state *state)
    util_dump_struct_end(stream);
 }
 
+void
+util_dump_stream_output_info(FILE *stream,
+                             const struct pipe_stream_output_info *state)
+{
+   if (!state) {
+      util_dump_null(stream);
+      return;
+   }
+
+   util_dump_struct_begin(stream, "pipe_stream_output_info");
+   util_dump_member(stream, uint, state, num_outputs);
+   util_dump_array(stream, uint, state->stride,
+                   ARRAY_SIZE(state->stride));
+   util_dump_array_begin(stream);
+   for (unsigned i = 0; i < state->num_outputs; ++i) {
+      util_dump_elem_begin(stream);
+      util_dump_struct_begin(stream, ""); /* anonymous */
+      util_dump_member(stream, uint, &state->output[i], register_index);
+      util_dump_member(stream, uint, &state->output[i], start_component);
+      util_dump_member(stream, uint, &state->output[i], num_components);
+      util_dump_member(stream, uint, &state->output[i], output_buffer);
+      util_dump_struct_end(stream);
+      util_dump_elem_end(stream);
+   }
+   util_dump_array_end(stream);
+   util_dump_struct_end(stream);
+}
 
 void
 util_dump_shader_state(FILE *stream, const struct pipe_shader_state *state)
 {
-   unsigned i;
-
    if (!state) {
       util_dump_null(stream);
       return;
@@ -484,23 +510,7 @@ util_dump_shader_state(FILE *stream, const struct pipe_shader_state *state)
 
    if (state->stream_output.num_outputs) {
       util_dump_member_begin(stream, "stream_output");
-      util_dump_struct_begin(stream, "pipe_stream_output_info");
-      util_dump_member(stream, uint, &state->stream_output, num_outputs);
-      util_dump_array(stream, uint, state->stream_output.stride,
-                      ARRAY_SIZE(state->stream_output.stride));
-      util_dump_array_begin(stream);
-      for(i = 0; i < state->stream_output.num_outputs; ++i) {
-         util_dump_elem_begin(stream);
-         util_dump_struct_begin(stream, ""); /* anonymous */
-         util_dump_member(stream, uint, &state->stream_output.output[i], register_index);
-         util_dump_member(stream, uint, &state->stream_output.output[i], start_component);
-         util_dump_member(stream, uint, &state->stream_output.output[i], num_components);
-         util_dump_member(stream, uint, &state->stream_output.output[i], output_buffer);
-         util_dump_struct_end(stream);
-         util_dump_elem_end(stream);
-      }
-      util_dump_array_end(stream);
-      util_dump_struct_end(stream);
+      util_dump_stream_output_info(stream, &state->stream_output);
       util_dump_member_end(stream);
    }
 
@@ -520,15 +530,11 @@ util_dump_depth_stencil_alpha_state(FILE *stream, const struct pipe_depth_stenci
 
    util_dump_struct_begin(stream, "pipe_depth_stencil_alpha_state");
 
-   util_dump_member_begin(stream, "depth");
-   util_dump_struct_begin(stream, "pipe_depth_state");
-   util_dump_member(stream, bool, &state->depth, enabled);
-   if (state->depth.enabled) {
-      util_dump_member(stream, bool, &state->depth, writemask);
-      util_dump_member(stream, enum_func, &state->depth, func);
+   util_dump_member(stream, bool, state, depth_enabled);
+   if (state->depth_enabled) {
+      util_dump_member(stream, bool, state, depth_writemask);
+      util_dump_member(stream, enum_func, state, depth_func);
    }
-   util_dump_struct_end(stream);
-   util_dump_member_end(stream);
 
    util_dump_member_begin(stream, "stencil");
    util_dump_array_begin(stream);
@@ -553,15 +559,11 @@ util_dump_depth_stencil_alpha_state(FILE *stream, const struct pipe_depth_stenci
    util_dump_array_end(stream);
    util_dump_member_end(stream);
 
-   util_dump_member_begin(stream, "alpha");
-   util_dump_struct_begin(stream, "pipe_alpha_state");
-   util_dump_member(stream, bool, &state->alpha, enabled);
-   if (state->alpha.enabled) {
-      util_dump_member(stream, enum_func, &state->alpha, func);
-      util_dump_member(stream, float, &state->alpha, ref_value);
+   util_dump_member(stream, bool, state, alpha_enabled);
+   if (state->alpha_enabled) {
+      util_dump_member(stream, enum_func, state, alpha_func);
+      util_dump_member(stream, float, state, alpha_ref_value);
    }
-   util_dump_struct_end(stream);
-   util_dump_member_end(stream);
 
    util_dump_struct_end(stream);
 }
@@ -602,6 +604,7 @@ util_dump_blend_state(FILE *stream, const struct pipe_blend_state *state)
    util_dump_member(stream, bool, state, dither);
    util_dump_member(stream, bool, state, alpha_to_coverage);
    util_dump_member(stream, bool, state, alpha_to_one);
+   util_dump_member(stream, uint, state, max_rt);
 
    util_dump_member(stream, bool, state, logicop_enable);
    if (state->logicop_enable) {
@@ -612,7 +615,7 @@ util_dump_blend_state(FILE *stream, const struct pipe_blend_state *state)
 
       util_dump_member_begin(stream, "rt");
       if (state->independent_blend_enable)
-         valid_entries = PIPE_MAX_COLOR_BUFS;
+         valid_entries = state->max_rt + 1;
       util_dump_struct_array(stream, rt_blend_state, state->rt, valid_entries);
       util_dump_member_end(stream);
    }
@@ -788,7 +791,7 @@ util_dump_sampler_view(FILE *stream, const struct pipe_sampler_view *state)
    else {
       util_dump_member(stream, uint, state, u.tex.first_layer);
       util_dump_member(stream, uint, state, u.tex.last_layer);
-      util_dump_member(stream, uint, state, u.tex.last_level);
+      util_dump_member(stream, uint, state, u.tex.first_level);
       util_dump_member(stream, uint, state, u.tex.last_level);
    }
 
@@ -915,17 +918,10 @@ util_dump_draw_info(FILE *stream, const struct pipe_draw_info *state)
    util_dump_member(stream, uint, state, has_user_indices);
 
    util_dump_member(stream, enum_prim_mode, state, mode);
-   util_dump_member(stream, uint, state, start);
-   util_dump_member(stream, uint, state, count);
 
    util_dump_member(stream, uint, state, start_instance);
    util_dump_member(stream, uint, state, instance_count);
 
-   util_dump_member(stream, uint, state, drawid);
-
-   util_dump_member(stream, uint, state, vertices_per_patch);
-
-   util_dump_member(stream, int,  state, index_bias);
    util_dump_member(stream, uint, state, min_index);
    util_dump_member(stream, uint, state, max_index);
 
@@ -939,19 +935,36 @@ util_dump_draw_info(FILE *stream, const struct pipe_draw_info *state)
       else
          util_dump_member(stream, ptr, state, index.resource);
    }
-   util_dump_member(stream, ptr, state, count_from_stream_output);
+   util_dump_struct_end(stream);
+}
 
-   if (!state->indirect) {
-      util_dump_member(stream, ptr, state, indirect);
-   } else {
-      util_dump_member(stream, uint, state, indirect->offset);
-      util_dump_member(stream, uint, state, indirect->stride);
-      util_dump_member(stream, uint, state, indirect->draw_count);
-      util_dump_member(stream, uint, state, indirect->indirect_draw_count_offset);
-      util_dump_member(stream, ptr, state, indirect->buffer);
-      util_dump_member(stream, ptr, state, indirect->indirect_draw_count);
+void
+util_dump_draw_start_count_bias(FILE *stream, const struct pipe_draw_start_count_bias *state)
+{
+   util_dump_struct_begin(stream, "pipe_draw_start_count_bias");
+   util_dump_member(stream, uint, state, start);
+   util_dump_member(stream, uint, state, count);
+   util_dump_member(stream, int,  state, index_bias);
+   util_dump_struct_end(stream);
+}
+
+void
+util_dump_draw_indirect_info(FILE *stream,
+                             const struct pipe_draw_indirect_info *state)
+{
+   if (!state) {
+      util_dump_null(stream);
+      return;
    }
 
+   util_dump_struct_begin(stream, "pipe_draw_indirect_info");
+   util_dump_member(stream, uint, state, offset);
+   util_dump_member(stream, uint, state, stride);
+   util_dump_member(stream, uint, state, draw_count);
+   util_dump_member(stream, uint, state, indirect_draw_count_offset);
+   util_dump_member(stream, ptr, state, buffer);
+   util_dump_member(stream, ptr, state, indirect_draw_count);
+   util_dump_member(stream, ptr, state, count_from_stream_output);
    util_dump_struct_end(stream);
 }
 
