@@ -38,6 +38,7 @@
 #include "util/slab.h"
 #include "util/u_dynarray.h"
 #include "util/u_helpers.h"
+#include "util/u_queue.h"
 #include "compiler/nir/nir.h"
 
 struct etna_bo;
@@ -53,6 +54,11 @@ enum viv_features_word {
    viv_chipMinorFeatures5 = 6,
    viv_chipMinorFeatures6 = 7,
    viv_chipMinorFeatures7 = 8,
+   viv_chipMinorFeatures8 = 9,
+   viv_chipMinorFeatures9 = 10,
+   viv_chipMinorFeatures10 = 11,
+   viv_chipMinorFeatures11 = 12,
+   viv_chipMinorFeatures12 = 13,
    VIV_FEATURES_WORD_COUNT /* Must be last */
 };
 
@@ -87,7 +93,13 @@ struct etna_screen {
    uint32_t drm_version;
 
    struct etna_compiler *compiler;
-   nir_shader_compiler_options options;
+   struct util_queue shader_compiler_queue;
+
+   /* dummy render target for GPUs that can't fully disable the color pipe */
+   struct etna_reloc dummy_rt_reloc;
+
+   /* dummy texture descriptor */
+   struct etna_reloc dummy_desc_reloc;
 };
 
 static inline struct etna_screen *
@@ -104,4 +116,16 @@ struct pipe_screen *
 etna_screen_create(struct etna_device *dev, struct etna_gpu *gpu,
                    struct renderonly *ro);
 
+static inline size_t
+etna_screen_get_tile_size(struct etna_screen *screen, uint8_t ts_mode)
+{
+   if (!VIV_FEATURE(screen, chipMinorFeatures6, CACHE128B256BPERLINE))
+      return 64;
+
+   if (ts_mode == TS_MODE_256B)
+      return 256;
+   else
+      return 128;
+}
+
 #endif
