@@ -30,19 +30,19 @@
 #include "../r600_pipe.h"
 #include "../r600_shader.h"
 
+
 #include "util/u_prim.h"
 
-#include "sfn_instruction_tex.h"
-
-#include "sfn_shader_vertex.h"
-#include "sfn_shader_fragment.h"
-#include "sfn_shader_geometry.h"
-#include "sfn_shader_compute.h"
-#include "sfn_shader_tcs.h"
-#include "sfn_shader_tess_eval.h"
+#include "sfn_shader.h"
+#include "sfn_assembler.h"
+#include "sfn_debug.h"
+#include "sfn_liverangeevaluator.h"
 #include "sfn_nir_lower_fs_out_to_vector.h"
-#include "sfn_ir_to_assembly.h"
 #include "sfn_nir_lower_alu.h"
+#include "sfn_nir_lower_tex.h"
+#include "sfn_optimizer.h"
+#include "sfn_ra.h"
+#include "sfn_scheduler.h"
 
 #include <vector>
 
@@ -54,7 +54,6 @@ using std::vector;
 NirLowerInstruction::NirLowerInstruction():
 	b(nullptr)
 {
-
 }
 
 bool NirLowerInstruction::filter_instr(const nir_instr *instr, const void *data)
@@ -78,264 +77,11 @@ bool NirLowerInstruction::run(nir_shader *shader)
                                         (void *)this);
 }
 
-
-ShaderFromNir::ShaderFromNir():sh(nullptr),
-   chip_class(CLASS_UNKNOWN),
-   m_current_if_id(0),
-   m_current_loop_id(0),
-   scratch_size(0)
-{
-}
-
-bool ShaderFromNir::lower(const nir_shader *shader, r600_pipe_shader *pipe_shader,
-                          r600_pipe_shader_selector *sel, r600_shader_key& key,
-                          struct r600_shader* gs_shader, enum chip_class _chip_class)
-{
-   sh = shader;
-   chip_class = _chip_class;
-   assert(sh);
-
-   switch (shader->info.stage) {
-   case MESA_SHADER_VERTEX:
-      impl.reset(new VertexShaderFromNir(pipe_shader, *sel, key, gs_shader, chip_class));
-      break;
-   case MESA_SHADER_TESS_CTRL:
-      sfn_log << SfnLog::trans << "Start TCS\n";
-      impl.reset(new TcsShaderFromNir(pipe_shader, *sel, key, chip_class));
-      break;
-   case MESA_SHADER_TESS_EVAL:
-      sfn_log << SfnLog::trans << "Start TESS_EVAL\n";
-      impl.reset(new TEvalShaderFromNir(pipe_shader, *sel, key, gs_shader, chip_class));
-      break;
-   case MESA_SHADER_GEOMETRY:
-      sfn_log << SfnLog::trans << "Start GS\n";
-      impl.reset(new GeometryShaderFromNir(pipe_shader, *sel, key, chip_class));
-      break;
-   case MESA_SHADER_FRAGMENT:
-      sfn_log << SfnLog::trans << "Start FS\n";
-      impl.reset(new FragmentShaderFromNir(*shader, pipe_shader->shader, *sel, key, chip_class));
-      break;
-   case MESA_SHADER_COMPUTE:
-      sfn_log << SfnLog::trans << "Start CS\n";
-      impl.reset(new ComputeShaderFromNir(pipe_shader, *sel, key, chip_class));
-      break;
-   default:
-      return false;
-   }
-
-   sfn_log << SfnLog::trans << "Process declarations\n";
-   if (!process_declaration())
-      return false;
-
-   // at this point all functions should be inlined
-   const nir_function *func = reinterpret_cast<const nir_function *>(exec_list_get_head_const(&sh->functions));
-
-   sfn_log << SfnLog::trans << "Scan shader\n";
-
-   if (sfn_log.has_debug_flag(SfnLog::instr))
-      nir_print_shader(const_cast<nir_shader *>(shader), stderr);
-
-   nir_foreach_block(block, func->impl) {
-      nir_foreach_instr(instr, block) {
-         if (!impl->scan_instruction(instr)) {
-            fprintf(stderr, "Unhandled sysvalue access ");
-            nir_print_instr(instr, stderr);
-            fprintf(stderr, "\n");
-            return false;
-         }
-      }
-   }
-
-   sfn_log << SfnLog::trans << "Reserve registers\n";
-   if (!impl->allocate_reserved_registers()) {
-      return false;
-   }
-
-   ValuePool::array_list arrays;
-   sfn_log << SfnLog::trans << "Allocate local registers\n";
-   foreach_list_typed(nir_register, reg, node, &func->impl->registers) {
-      impl->allocate_local_register(*reg, arrays);
-   }
-
-   sfn_log << SfnLog::trans << "Emit shader start\n";
-   impl->allocate_arrays(arrays);
-
-   impl->emit_shader_start();
-
-   sfn_log << SfnLog::trans << "Process shader \n";
-   foreach_list_typed(nir_cf_node, node, node, &func->impl->body) {
-      if (!process_cf_node(node))
-         return false;
-   }
-
-   // Add optimizations here
-   sfn_log << SfnLog::trans << "Finalize\n";
-   impl->finalize();
-
-   impl->get_array_info(pipe_shader->shader);
-
-   if (!sfn_log.has_debug_flag(SfnLog::nomerge)) {
-      sfn_log << SfnLog::trans << "Merge registers\n";
-      impl->remap_registers();
-   }
-
-   sfn_log << SfnLog::trans << "Finished translating to R600 IR\n";
-   return true;
-}
-
-Shader ShaderFromNir::shader() const
-{
-   return Shader{impl->m_output, impl->get_temp_registers()};
-}
-
-
-bool ShaderFromNir::process_cf_node(nir_cf_node *node)
-{
-   SFN_TRACE_FUNC(SfnLog::flow, "CF");
-   switch (node->type) {
-   case nir_cf_node_block:
-      return process_block(nir_cf_node_as_block(node));
-   case nir_cf_node_if:
-      return process_if(nir_cf_node_as_if(node));
-   case nir_cf_node_loop:
-      return process_loop(nir_cf_node_as_loop(node));
-   default:
-      return false;
-   }
-}
-
-bool ShaderFromNir::process_if(nir_if *if_stmt)
-{
-   SFN_TRACE_FUNC(SfnLog::flow, "IF");
-
-   if (!impl->emit_if_start(m_current_if_id, if_stmt))
-      return false;
-
-   int if_id = m_current_if_id++;
-   m_if_stack.push(if_id);
-
-   foreach_list_typed(nir_cf_node, n, node, &if_stmt->then_list)
-         if (!process_cf_node(n)) return false;
-
-   if (!if_stmt->then_list.is_empty()) {
-      if (!impl->emit_else_start(if_id))
-         return false;
-
-      foreach_list_typed(nir_cf_node, n, node, &if_stmt->else_list)
-            if (!process_cf_node(n)) return false;
-   }
-
-   if (!impl->emit_ifelse_end(if_id))
-      return false;
-
-   m_if_stack.pop();
-   return true;
-}
-
-bool ShaderFromNir::process_loop(nir_loop *node)
-{
-   SFN_TRACE_FUNC(SfnLog::flow, "LOOP");
-   int loop_id = m_current_loop_id++;
-
-   if (!impl->emit_loop_start(loop_id))
-      return false;
-
-   foreach_list_typed(nir_cf_node, n, node, &node->body)
-         if (!process_cf_node(n)) return false;
-
-   if (!impl->emit_loop_end(loop_id))
-      return false;
-
-   return true;
-}
-
-bool ShaderFromNir::process_block(nir_block *block)
-{
-   SFN_TRACE_FUNC(SfnLog::flow, "BLOCK");
-   nir_foreach_instr(instr, block) {
-      int r = emit_instruction(instr);
-      if (!r) {
-         sfn_log << SfnLog::err << "R600: Unsupported instruction: "
-                 << *instr << "\n";
-         return false;
-      }
-   }
-   return true;
-}
-
-
-ShaderFromNir::~ShaderFromNir()
-{
-}
-
-pipe_shader_type ShaderFromNir::processor_type() const
-{
-   return impl->m_processor_type;
-}
-
-
-bool ShaderFromNir::emit_instruction(nir_instr *instr)
-{
-   assert(impl);
-
-   sfn_log << SfnLog::instr << "Read instruction " << *instr << "\n";
-
-   switch (instr->type) {
-   case nir_instr_type_alu:
-      return impl->emit_alu_instruction(instr);
-   case nir_instr_type_deref:
-      return impl->emit_deref_instruction(nir_instr_as_deref(instr));
-   case nir_instr_type_intrinsic:
-      return impl->emit_intrinsic_instruction(nir_instr_as_intrinsic(instr));
-   case nir_instr_type_load_const: /* const values are loaded when needed */
-      return true;
-   case nir_instr_type_tex:
-      return impl->emit_tex_instruction(instr);
-   case nir_instr_type_jump:
-      return impl->emit_jump_instruction(nir_instr_as_jump(instr));
-   default:
-      fprintf(stderr, "R600: %s: ShaderFromNir Unsupported instruction: type %d:'", __func__, instr->type);
-      nir_print_instr(instr, stderr);
-      fprintf(stderr, "'\n");
-      return false;
-   case nir_instr_type_ssa_undef:
-      return impl->create_undef(nir_instr_as_ssa_undef(instr));
-      return true;
-   }
-}
-
-bool ShaderFromNir::process_declaration()
-{
-   impl->set_shader_info(sh);
-
-   if (!impl->scan_inputs_read(sh))
-      return false;
-
-   // scan declarations
-   nir_foreach_variable_with_modes(variable, sh, nir_var_uniform |
-                                                 nir_var_mem_ubo |
-                                                 nir_var_mem_ssbo) {
-      if (!impl->process_uniforms(variable)) {
-         fprintf(stderr, "R600: error parsing outputs variable %s\n", variable->name);
-         return false;
-      }
-   }
-
-   return true;
-}
-
-const std::vector<InstructionBlock>& ShaderFromNir::shader_ir() const
-{
-   assert(impl);
-   return impl->m_output;
-}
-
-
 AssemblyFromShader::~AssemblyFromShader()
 {
 }
 
-bool AssemblyFromShader::lower(const std::vector<InstructionBlock>& ir)
+bool AssemblyFromShader::lower(const Shader& ir)
 {
    return do_lower(ir);
 }
@@ -444,6 +190,75 @@ void sort_fsoutput(nir_shader *shader)
    exec_list_append(&shader->variables, &new_list);
 }
 
+class LowerClipvertexWrite : public NirLowerInstruction {
+
+public:
+   LowerClipvertexWrite(int noutputs, pipe_stream_output_info& so_info) :
+      m_clipplane1(noutputs),
+      m_clipvtx(noutputs + 1),
+      m_so_info(so_info){}
+private:
+   bool filter(const nir_instr *instr) const override {
+      if (instr->type != nir_instr_type_intrinsic)
+         return false;
+
+      auto intr = nir_instr_as_intrinsic(instr);
+      if (intr->intrinsic != nir_intrinsic_store_output)
+         return false;
+
+      return nir_intrinsic_io_semantics(intr).location == VARYING_SLOT_CLIP_VERTEX;
+   }
+
+   nir_ssa_def *lower(nir_instr *instr) override {
+
+      auto intr = nir_instr_as_intrinsic(instr);
+      nir_ssa_def *output[8] = {nullptr};
+
+      // for UBO loads we correct the buffer ID by adding 1
+      auto buf_id = nir_imm_int(b, R600_BUFFER_INFO_CONST_BUFFER - 1);
+
+      assert(intr->src[0].is_ssa);
+      auto clip_vtx = intr->src[0].ssa;
+
+      for (int i = 0; i < 8; ++i) {
+         auto sel = nir_imm_int(b, i);
+         auto mrow = nir_load_ubo_vec4(b, 4, 32, buf_id, sel);
+         output[i] = nir_fdot4(b, clip_vtx, mrow);
+      }
+
+      unsigned clip_vertex_index = nir_intrinsic_base(intr);
+
+      for (int i = 0; i < 2; ++i) {
+         auto clip_i = nir_vec(b, &output[4 * i], 4);
+         auto store = nir_store_output(b, clip_i,  intr->src[1].ssa);
+         nir_intrinsic_set_write_mask(store, 0xf);
+         nir_intrinsic_set_base(store, clip_vertex_index);
+         nir_io_semantics semantic = nir_intrinsic_io_semantics(intr);
+         semantic.location = VARYING_SLOT_CLIP_DIST0 + i;
+         semantic.no_varying = 1;
+
+         if (i > 0)
+            nir_intrinsic_set_base(store, m_clipplane1);
+         nir_intrinsic_set_write_mask(store, 0xf);
+         nir_intrinsic_set_io_semantics(store, semantic);
+      }
+      nir_intrinsic_set_base(intr, m_clipvtx);
+
+      nir_ssa_def *result = NIR_LOWER_INSTR_PROGRESS_REPLACE;
+      for (unsigned  i = 0; i < m_so_info.num_outputs; ++i) {
+         if (m_so_info.output[i].register_index == clip_vertex_index) {
+            m_so_info.output[i].register_index = m_clipvtx;
+            result = NIR_LOWER_INSTR_PROGRESS;
+         }
+      }
+      return result;
+   }
+   int m_clipplane1;
+   int m_clipvtx;
+   pipe_stream_output_info& m_so_info;
+};
+
+
 }
 
 static nir_intrinsic_op
@@ -531,6 +346,19 @@ r600_lower_deref_instr(nir_builder *b, nir_instr *instr_, UNUSED void *cb_data)
    return true;
 }
 
+
+static bool
+r600_lower_clipvertex_to_clipdist(nir_shader *sh,
+                                  pipe_stream_output_info& so_info)
+{
+   if (!(sh->info.outputs_written & VARYING_BIT_CLIP_VERTEX))
+      return false;
+
+   int noutputs = util_bitcount64(sh->info.outputs_written);
+   bool result = r600::LowerClipvertexWrite(noutputs, so_info).run(sh);
+   return result;
+}
+
 static bool
 r600_nir_lower_atomics(nir_shader *shader)
 {
@@ -557,7 +385,6 @@ r600_nir_lower_atomics(nir_shader *shader)
                                        nir_metadata_dominance,
                                        NULL);
 }
-using r600::r600_nir_lower_int_tg4;
 using r600::r600_lower_scratch_addresses;
 using r600::r600_lower_fs_out_to_vector;
 using r600::r600_lower_ubo_to_align16;
@@ -676,6 +503,7 @@ r600_lower_shared_io(nir_shader *nir)
 static nir_ssa_def *
 r600_lower_fs_pos_input_impl(nir_builder *b, nir_instr *instr, void *_options)
 {
+   (void)_options;
    auto old_ir = nir_instr_as_intrinsic(instr);
    auto load = nir_intrinsic_instr_create(b->shader, nir_intrinsic_load_input);
    nir_ssa_dest_init(&load->instr, &load->dest,
@@ -693,6 +521,8 @@ r600_lower_fs_pos_input_impl(nir_builder *b, nir_instr *instr, void *_options)
 
 bool r600_lower_fs_pos_input_filter(const nir_instr *instr, const void *_options)
 {
+   (void)_options;
+
    if (instr->type != nir_instr_type_intrinsic)
       return false;
 
@@ -713,7 +543,7 @@ bool r600_lower_fs_pos_input(nir_shader *shader)
 };
 
 static bool
-optimize_once(nir_shader *shader, bool vectorize)
+optimize_once(nir_shader *shader)
 {
    bool progress = false;
    NIR_PASS(progress, shader, nir_lower_vars_to_ssa);
@@ -722,9 +552,6 @@ optimize_once(nir_shader *shader, bool vectorize)
    NIR_PASS(progress, shader, nir_opt_algebraic);
    NIR_PASS(progress, shader, nir_opt_constant_folding);
    NIR_PASS(progress, shader, nir_opt_copy_prop_vars);
-   if (vectorize)
-      NIR_PASS(progress, shader, nir_opt_vectorize, NULL, NULL);
-
    NIR_PASS(progress, shader, nir_opt_remove_phis);
 
    if (nir_opt_trivial_continues(shader)) {
@@ -733,7 +560,7 @@ optimize_once(nir_shader *shader, bool vectorize)
            NIR_PASS(progress, shader, nir_opt_dce);
    }
 
-   NIR_PASS(progress, shader, nir_opt_if, false);
+   NIR_PASS(progress, shader, nir_opt_if, nir_opt_if_optimize_phi_true_false);
    NIR_PASS(progress, shader, nir_opt_dead_cf);
    NIR_PASS(progress, shader, nir_opt_cse);
    NIR_PASS(progress, shader, nir_opt_peephole_select, 200, true, true);
@@ -758,6 +585,22 @@ bool has_saturate(const nir_function *func)
    return false;
 }
 
+static bool r600_is_last_vertex_stage(nir_shader *nir, const r600_shader_key& key)
+{
+   if (nir->info.stage == MESA_SHADER_GEOMETRY)
+      return true;
+
+   if (nir->info.stage == MESA_SHADER_TESS_EVAL &&
+       !key.tes.as_es)
+      return true;
+
+   if (nir->info.stage == MESA_SHADER_VERTEX &&
+       !key.vs.as_es && !key.vs.as_ls)
+      return true;
+
+   return false;
+}
+
 extern "C"
 bool r600_lower_to_scalar_instr_filter(const nir_instr *instr, const void *)
 {
@@ -777,31 +620,35 @@ bool r600_lower_to_scalar_instr_filter(const nir_instr *instr, const void *)
    case nir_op_fdot2:
    case nir_op_fdot3:
    case nir_op_fdot4:
+      return nir_src_bit_size(alu->src[0].src) == 64;
    case nir_op_cube_r600:
       return false;
-   case nir_op_bany_fnequal2:
-   case nir_op_ball_fequal2:
-   case nir_op_bany_inequal2:
-   case nir_op_ball_iequal2:
-      return nir_src_bit_size(alu->src[0].src) != 64;
    default:
       return true;
    }
 }
 
+
+class MallocPoolRelease {
+public:
+   ~MallocPoolRelease() {
+      r600::release_pool();
+   }
+};
+
 int r600_shader_from_nir(struct r600_context *rctx,
                          struct r600_pipe_shader *pipeshader,
                          r600_shader_key *key)
 {
-   char filename[4000];
+   MallocPoolRelease pool_release;
+
    struct r600_pipe_shader_selector *sel = pipeshader->selector;
 
-   bool lower_64bit = ((sel->nir->options->lower_int64_options ||
+   bool lower_64bit = (rctx->b.gfx_level < CAYMAN  &&
+                       (sel->nir->options->lower_int64_options ||
                         sel->nir->options->lower_doubles_options) &&
                        (sel->nir->info.bit_sizes_float | sel->nir->info.bit_sizes_int) & 64);
 
-   r600::ShaderFromNir convert;
-
    if (rctx->screen->b.debug_flags & DBG_PREOPT_IR) {
       fprintf(stderr, "PRE-OPT-NIR-----------.------------------------------\n");
       nir_print_shader(sel->nir, stderr);
@@ -813,22 +660,20 @@ int r600_shader_from_nir(struct r600_context *rctx,
    /* Cayman seems very crashy about accessing images that don't exists or are
     * accessed out of range, this lowering seems to help (but it can also be
     * another problem */
-   if (sel->nir->info.num_images > 0 && rctx->b.chip_class == CAYMAN)
-       NIR_PASS_V(sel->nir, r600_legalize_image_load_store);
 
-   NIR_PASS_V(sel->nir, nir_lower_vars_to_ssa);
    NIR_PASS_V(sel->nir, nir_lower_regs_to_ssa);
    nir_lower_idiv_options idiv_options = {0};
    idiv_options.imprecise_32bit_lowering = sel->nir->info.stage != MESA_SHADER_COMPUTE;
    idiv_options.allow_fp16 = true;
 
    NIR_PASS_V(sel->nir, nir_lower_idiv, &idiv_options);
-   NIR_PASS_V(sel->nir, r600_nir_lower_trigen);
+   NIR_PASS_V(sel->nir, r600_nir_lower_trigen, rctx->b.gfx_level);
    NIR_PASS_V(sel->nir, nir_lower_phis_to_scalar, false);
+   NIR_PASS_V(sel->nir, nir_lower_undef_to_zero);
 
    if (lower_64bit)
       NIR_PASS_V(sel->nir, nir_lower_int64);
-   while(optimize_once(sel->nir, false));
+   while(optimize_once(sel->nir));
 
    NIR_PASS_V(sel->nir, r600_lower_shared_io);
    NIR_PASS_V(sel->nir, r600_nir_lower_atomics);
@@ -836,10 +681,12 @@ int r600_shader_from_nir(struct r600_context *rctx,
    struct nir_lower_tex_options lower_tex_options = {0};
    lower_tex_options.lower_txp = ~0u;
    lower_tex_options.lower_txf_offset = true;
+   lower_tex_options.lower_invalid_implicit_lod = true;
+   lower_tex_options.lower_tg4_offsets = true;
 
    NIR_PASS_V(sel->nir, nir_lower_tex, &lower_tex_options);
-   NIR_PASS_V(sel->nir, r600::r600_nir_lower_txl_txf_array_or_cube);
-   NIR_PASS_V(sel->nir, r600::r600_nir_lower_cube_to_2darray);
+   NIR_PASS_V(sel->nir, r600_nir_lower_txl_txf_array_or_cube);
+   NIR_PASS_V(sel->nir, r600_nir_lower_cube_to_2darray);
 
    NIR_PASS_V(sel->nir, r600_nir_lower_pack_unpack_2x16);
 
@@ -850,30 +697,11 @@ int r600_shader_from_nir(struct r600_context *rctx,
       NIR_PASS_V(sel->nir, nir_lower_fragcoord_wtrans);
       NIR_PASS_V(sel->nir, r600_lower_fs_out_to_vector);
    }
+   nir_variable_mode io_modes = nir_var_uniform |
+                                nir_var_shader_in |
+                                nir_var_shader_out;
 
-   nir_variable_mode io_modes = nir_var_uniform | nir_var_shader_in;
-
-   //if (sel->nir->info.stage != MESA_SHADER_FRAGMENT)
-      io_modes |= nir_var_shader_out;
-
-   if (sel->nir->info.stage == MESA_SHADER_FRAGMENT) {
-
-      /* Lower IO to temporaries late, because otherwise we get into trouble
-       * with the glsl 4.40 interpolateAt swizzle tests. There seems to be a bug
-       * somewhere that results in the input alweas reading from the same temp
-       * regardless of interpolation when the lowering is done early */
-      NIR_PASS_V(sel->nir, nir_lower_io_to_temporaries, nir_shader_get_entrypoint(sel->nir),
-              true, true);
-
-      /* Since we're doing nir_lower_io_to_temporaries late, we need
-       * to lower all the copy_deref's introduced by
-       * lower_io_to_temporaries before calling nir_lower_io.
-       */
-      NIR_PASS_V(sel->nir, nir_split_var_copies);
-      NIR_PASS_V(sel->nir, nir_lower_var_copies);
-      NIR_PASS_V(sel->nir, nir_lower_global_vars_to_local);
-   }
-
+   NIR_PASS_V(sel->nir, nir_opt_combine_stores, nir_var_shader_out);
    NIR_PASS_V(sel->nir, nir_lower_io, io_modes, r600_glsl_type_size,
                  nir_lower_io_lower_64bit_to_32);
 
@@ -899,6 +727,9 @@ int r600_shader_from_nir(struct r600_context *rctx,
 
    auto sh = nir_shader_clone(sel->nir, sel->nir);
 
+   if (r600_is_last_vertex_stage(sh, *key))
+      r600_lower_clipvertex_to_clipdist(sh, sel->so);
+
    if (sh->info.stage == MESA_SHADER_TESS_CTRL ||
        sh->info.stage == MESA_SHADER_TESS_EVAL ||
        (sh->info.stage == MESA_SHADER_VERTEX && key->vs.as_ls)) {
@@ -915,14 +746,26 @@ int r600_shader_from_nir(struct r600_context *rctx,
       NIR_PASS_V(sh, r600_lower_tess_coord, u_tess_prim_from_shader(sh->info.tess._primitive_mode));
    }
 
+   NIR_PASS_V(sh, nir_lower_alu_to_scalar, r600_lower_to_scalar_instr_filter, NULL);
+   NIR_PASS_V(sh, nir_lower_phis_to_scalar, false);
+   NIR_PASS_V(sh, nir_lower_alu_to_scalar, r600_lower_to_scalar_instr_filter, NULL);
+
+   NIR_PASS_V(sh, r600::r600_nir_split_64bit_io);
+   NIR_PASS_V(sh, r600::r600_split_64bit_alu_and_phi);
+   NIR_PASS_V(sh, nir_split_64bit_vec3_and_vec4);
+   NIR_PASS_V(sh, nir_lower_int64);
+
    NIR_PASS_V(sh, nir_lower_ubo_vec4);
+
    if (lower_64bit)
       NIR_PASS_V(sh, r600::r600_nir_64_to_vec2);
 
+   NIR_PASS_V(sh, r600::r600_split_64bit_uniforms_and_ubo);
    /* Lower to scalar to let some optimization work out better */
-   while(optimize_once(sh, false));
+   while(optimize_once(sh));
 
-   NIR_PASS_V(sh, r600::r600_merge_vec2_stores);
+   if (lower_64bit)
+      NIR_PASS_V(sh, r600::r600_merge_vec2_stores);
 
    NIR_PASS_V(sh, nir_remove_dead_variables, nir_var_shader_in, NULL);
    NIR_PASS_V(sh, nir_remove_dead_variables,  nir_var_shader_out, NULL);
@@ -933,7 +776,7 @@ int r600_shader_from_nir(struct r600_context *rctx,
               40,
               r600_get_natural_size_align_bytes);
 
-   while (optimize_once(sh, true));
+   while (optimize_once(sh));
 
    NIR_PASS_V(sh, nir_lower_bool_to_int32);
    NIR_PASS_V(sh, r600_nir_lower_int_tg4);
@@ -944,8 +787,6 @@ int r600_shader_from_nir(struct r600_context *rctx,
 
    NIR_PASS_V(sh, nir_lower_locals_to_regs);
 
-   //NIR_PASS_V(sh, nir_opt_algebraic);
-   //NIR_PASS_V(sh, nir_copy_prop);
    NIR_PASS_V(sh, nir_lower_to_source_mods,
 	      (nir_lower_to_source_mods_flags)(nir_lower_float_source_mods |
 					       nir_lower_64bit_source_mods));
@@ -973,35 +814,68 @@ int r600_shader_from_nir(struct r600_context *rctx,
       pipeshader->shader.cc_dist_mask = (1 <<  (sh->info.cull_distance_array_size +
                                                 sh->info.clip_distance_array_size)) - 1;
    }
-
-   struct r600_shader* gs_shader = nullptr;
+   struct r600_shader* gs_shader = nullptr;   
    if (rctx->gs_shader)
       gs_shader = &rctx->gs_shader->current->shader;
    r600_screen *rscreen = rctx->screen;
 
-   bool r = convert.lower(sh, pipeshader, sel, *key, gs_shader, rscreen->b.chip_class);
-   if (!r || rctx->screen->b.debug_flags & DBG_ALL_SHADERS) {
-      static int shnr = 0;
+   r600::Shader *shader = r600::Shader::translate_from_nir(sh, &sel->so, gs_shader,
+                                                           *key, rctx->isa->hw_class);
 
-      snprintf(filename, 4000, "nir-%s_%d.inc", sh->info.name, shnr++);
+   assert(shader);
+   if (!shader)
+      return -2;
 
-      if (access(filename, F_OK) == -1) {
-         FILE *f = fopen(filename, "w");
+   pipeshader->enabled_stream_buffers_mask = shader->enabled_stream_buffers_mask();
+   pipeshader->selector->info.file_count[TGSI_FILE_HW_ATOMIC] += shader->atomic_file_count();
+   pipeshader->selector->info.writes_memory = shader->has_flag(r600::Shader::sh_writes_memory);
 
-         if (f) {
-            fprintf(f, "const char *shader_blob_%s = {\nR\"(", sh->info.name);
-            nir_print_shader(sh, f);
-            fprintf(f, ")\";\n");
-            fclose(f);
-         }
-      }
-      if (!r)
-         return -2;
+   if (r600::sfn_log.has_debug_flag(r600::SfnLog::steps)) {
+      std::cerr << "Shader after conversion from nir\n";
+      shader->print(std::cerr);
    }
 
-   auto shader = convert.shader();
+   if (!r600::sfn_log.has_debug_flag(r600::SfnLog::noopt)) {
+      optimize(*shader);
 
-   r600_bytecode_init(&pipeshader->shader.bc, rscreen->b.chip_class, rscreen->b.family,
+      if (r600::sfn_log.has_debug_flag(r600::SfnLog::steps)) {
+         std::cerr << "Shader after optimization\n";
+         shader->print(std::cerr);
+      }
+   }
+
+   auto scheduled_shader = r600::schedule(shader);
+   if (r600::sfn_log.has_debug_flag(r600::SfnLog::steps)) {
+      std::cerr << "Shader after scheduling\n";
+      shader->print(std::cerr);
+   }
+
+   if (!r600::sfn_log.has_debug_flag(r600::SfnLog::nomerge)) {
+
+      if (r600::sfn_log.has_debug_flag(r600::SfnLog::merge)) {
+         r600::sfn_log << r600::SfnLog::merge << "Shader before RA\n";
+         scheduled_shader->print(std::cerr);
+      }
+
+      r600::sfn_log << r600::SfnLog::trans << "Merge registers\n";
+      auto lrm = r600::LiveRangeEvaluator().run(*scheduled_shader);
+
+      if (!r600::register_allocation(lrm)) {
+         R600_ERR("%s: Register allocation failed\n", __func__);
+         /* For now crash if the shader could not be benerated */
+         assert(0);
+         return -1;
+      } else if (r600::sfn_log.has_debug_flag(r600::SfnLog::merge) ||
+                 r600::sfn_log.has_debug_flag(r600::SfnLog::steps)) {
+         r600::sfn_log << "Shader after RA\n";
+         scheduled_shader->print(std::cerr);
+      }
+   }
+
+   scheduled_shader->get_shader_info(&pipeshader->shader);
+   pipeshader->shader.uses_doubles = sh->info.bit_sizes_float & 64 ? 1 : 0;
+
+   r600_bytecode_init(&pipeshader->shader.bc, rscreen->b.gfx_level, rscreen->b.family,
                       rscreen->has_compressed_msaa_texturing);
 
    r600::sfn_log << r600::SfnLog::shader_info
@@ -1011,9 +885,13 @@ int r600_shader_from_nir(struct r600_context *rctx,
    pipeshader->shader.bc.type = pipeshader->shader.processor_type;
    pipeshader->shader.bc.isa = rctx->isa;
 
-   r600::AssemblyFromShaderLegacy afs(&pipeshader->shader, key);
-   if (!afs.lower(shader.m_ir)) {
+   r600::Assembler afs(&pipeshader->shader, *key);
+   if (!afs.lower(scheduled_shader)) {
       R600_ERR("%s: Lowering to assembly failed\n", __func__);
+
+      scheduled_shader->print(std::cerr);
+      /* For now crash if the shader could not be benerated */
+      assert(0);
       return -1;
    }
 
@@ -1024,8 +902,5 @@ int r600_shader_from_nir(struct r600_context *rctx,
    } else {
       r600::sfn_log << r600::SfnLog::shader_info << "This is not a Geometry shader\n";
    }
-   if (pipeshader->shader.bc.ngpr < 6)
-      pipeshader->shader.bc.ngpr = 6;
-
    return 0;
 }
