@@ -25,10 +25,11 @@
 #ifndef SI_PM4_H
 #define SI_PM4_H
 
-#include "radeon/radeon_winsys.h"
+#include "winsys/radeon_winsys.h"
 
-#define SI_PM4_MAX_DW 176
-#define SI_PM4_MAX_BO 3
+#ifdef __cplusplus
+extern "C" {
+#endif
 
 // forward defines
 struct si_context;
@@ -41,42 +42,35 @@ struct si_atom {
 };
 
 struct si_pm4_state {
-   /* optional indirect buffer */
-   struct si_resource *indirect_buffer;
-
    /* PKT3_SET_*_REG handling */
-   unsigned last_opcode;
-   unsigned last_reg;
-   unsigned last_pm4;
-
-   /* commands for the DE */
-   unsigned ndw;
-   uint32_t pm4[SI_PM4_MAX_DW];
-
-   /* BO's referenced by this state */
-   unsigned nbo;
-   struct si_resource *bo[SI_PM4_MAX_BO];
-   enum radeon_bo_usage bo_usage[SI_PM4_MAX_BO];
-   enum radeon_bo_priority bo_priority[SI_PM4_MAX_BO];
+   uint16_t last_reg;   /* register offset in dwords */
+   uint16_t last_pm4;
+   uint16_t ndw;        /* number of dwords in pm4 */
+   uint8_t last_opcode;
 
    /* For shader states only */
-   struct si_shader *shader;
+   bool is_shader;
    struct si_atom atom;
+
+   /* commands for the DE */
+   uint16_t max_dw;
+
+   /* This must be the last field because the array can continue after the structure. */
+   uint32_t pm4[64];
 };
 
-void si_pm4_cmd_begin(struct si_pm4_state *state, unsigned opcode);
 void si_pm4_cmd_add(struct si_pm4_state *state, uint32_t dw);
-void si_pm4_cmd_end(struct si_pm4_state *state, bool predicate);
-
 void si_pm4_set_reg(struct si_pm4_state *state, unsigned reg, uint32_t val);
-void si_pm4_add_bo(struct si_pm4_state *state, struct si_resource *bo, enum radeon_bo_usage usage,
-                   enum radeon_bo_priority priority);
-void si_pm4_upload_indirect_buffer(struct si_context *sctx, struct si_pm4_state *state);
+void si_pm4_set_reg_idx3(struct si_pm4_state *state, unsigned reg, uint32_t val);
 
 void si_pm4_clear_state(struct si_pm4_state *state);
 void si_pm4_free_state(struct si_context *sctx, struct si_pm4_state *state, unsigned idx);
 
 void si_pm4_emit(struct si_context *sctx, struct si_pm4_state *state);
-void si_pm4_reset_emitted(struct si_context *sctx);
+void si_pm4_reset_emitted(struct si_context *sctx, bool first_cs);
+
+#ifdef __cplusplus
+}
+#endif
 
 #endif
