@@ -113,7 +113,7 @@ softpipe_shader_db(struct pipe_context *pipe, const struct tgsi_token *tokens)
 
    struct tgsi_shader_info info;
    tgsi_scan_shader(tokens, &info);
-   pipe_debug_message(&softpipe->debug, SHADER_INFO, "%s shader: %d inst, %d loops, %d temps, %d const, %d imm",
+   util_debug_message(&softpipe->debug, SHADER_INFO, "%s shader: %d inst, %d loops, %d temps, %d const, %d imm",
                       _mesa_shader_stage_to_abbrev(tgsi_processor_to_shader_stage(info.processor)),
                       info.num_instructions,
                       info.opcode_count[TGSI_OPCODE_BGNLOOP],
