@@ -682,15 +682,24 @@ static void *virgl_shader_encoder(struct pipe_context *ctx,
    const struct tgsi_token *ntt_tokens = NULL;
    struct tgsi_token *new_tokens;
    int ret;
+   bool is_separable = false;
 
    if (shader->type == PIPE_SHADER_IR_NIR) {
+      struct nir_to_tgsi_options options = {
+         .unoptimized_ra = true,
+         .lower_fabs = true
+      };
       nir_shader *s = nir_shader_clone(NULL, shader->ir.nir);
-      ntt_tokens = tokens = nir_to_tgsi(s, vctx->base.screen); /* takes ownership */
+
+      /* Propagare the separable shader property to the host, unless
+       * it is an internal shader - these are marked separable even though they are not. */
+      is_separable = s->info.separate_shader && !s->info.internal;
+      ntt_tokens = tokens = nir_to_tgsi_options(s, vctx->base.screen, &options); /* takes ownership */
    } else {
       tokens = shader->tokens;
    }
 
-   new_tokens = virgl_tgsi_transform((struct virgl_screen *)vctx->base.screen, tokens);
+   new_tokens = virgl_tgsi_transform((struct virgl_screen *)vctx->base.screen, tokens, is_separable);
    if (!new_tokens)
       return NULL;
 
@@ -1366,23 +1375,32 @@ static void *virgl_create_compute_state(struct pipe_context *ctx,
    int ret;
 
    if (state->ir_type == PIPE_SHADER_IR_NIR) {
+      struct nir_to_tgsi_options options = {
+         .unoptimized_ra = true,
+         .lower_fabs = true
+      };
       nir_shader *s = nir_shader_clone(NULL, state->prog);
-      ntt_tokens = tokens = nir_to_tgsi(s, vctx->base.screen); /* takes ownership */
+      ntt_tokens = tokens = nir_to_tgsi_options(s, vctx->base.screen, &options); /* takes ownership */
    } else {
       tokens = state->prog;
    }
 
+   void *new_tokens = virgl_tgsi_transform((struct virgl_screen *)vctx->base.screen, tokens, false);
+   if (!new_tokens)
+      return NULL;
+
    handle = virgl_object_assign_handle();
    ret = virgl_encode_shader_state(vctx, handle, PIPE_SHADER_COMPUTE,
                                    &so_info,
                                    state->req_local_mem,
-                                   tokens);
+                                   new_tokens);
    if (ret) {
       FREE((void *)ntt_tokens);
       return NULL;
    }
 
    FREE((void *)ntt_tokens);
+   FREE(new_tokens);
 
    return (void *)(unsigned long)handle;
 }
