@@ -35,7 +35,7 @@
 #include "util/u_draw_quad.h"
 
 #define floatsEqual(x, y) (fabsf(x - y) <= 0.00001f * MIN2(fabsf(x), fabsf(y)))
-#define floatIsZero(x) (floatsEqual((x) + 1, 1))
+#define floatIsZero(x) (floatsEqual((x) + 1.0f, 1.0f))
 
 #define NUM_COMPONENTS 4
 
@@ -49,7 +49,7 @@ static inline boolean
 is_affine(const float *matrix)
 {
     return floatIsZero(matrix[2]) && floatIsZero(matrix[5])
-	&& floatsEqual(matrix[8], 1);
+	&& floatsEqual(matrix[8], 1.0f);
 }
 
 static inline void
@@ -335,7 +335,7 @@ renderer_bind_destination(struct xa_context *r,
     fb.height = surface->height;
     fb.nr_cbufs = 1;
     fb.cbufs[0] = surface;
-    fb.zsbuf = 0;
+    fb.zsbuf = NULL;
 
     /* Viewport just touches the bit we're interested in:
      */
@@ -446,7 +446,7 @@ renderer_copy_prepare(struct xa_context *r,
 	u_sampler_view_default_template(&templ,
 					src_texture, src_texture->format);
 	src_view = pipe->create_sampler_view(pipe, src_texture, &templ);
-	cso_set_sampler_views(r->cso, PIPE_SHADER_FRAGMENT, 1, &src_view);
+	pipe->set_sampler_views(pipe, PIPE_SHADER_FRAGMENT, 0, 1, 0, false, &src_view);
 	pipe_sampler_view_reference(&src_view, NULL);
     }
 
