@@ -515,7 +515,8 @@ enum pipe_flush_flags
 #define PIPE_BIND_LINEAR      (1 << 21)
 #define PIPE_BIND_PROTECTED   (1 << 22) /* Resource will be protected/encrypted */
 #define PIPE_BIND_SAMPLER_REDUCTION_MINMAX (1 << 23) /* PIPE_CAP_SAMPLER_REDUCTION_MINMAX */
-#define PIPE_BIND_DRI_PRIME   (1 << 24)
+/* Resource is the DRI_PRIME blit destination. Only set on on the render GPU. */
+#define PIPE_BIND_PRIME_BLIT_DST (1 << 24)
 
 
 /**
@@ -529,7 +530,8 @@ enum pipe_flush_flags
 #define PIPE_RESOURCE_FLAG_ENCRYPTED             (1 << 5)
 #define PIPE_RESOURCE_FLAG_DONT_OVER_ALLOCATE    (1 << 6)
 #define PIPE_RESOURCE_FLAG_DONT_MAP_DIRECTLY     (1 << 7) /* for small visible VRAM */
-#define PIPE_RESOURCE_FLAG_DRV_PRIV    (1 << 8) /* driver/winsys private */
+#define PIPE_RESOURCE_FLAG_UNMAPPABLE            (1 << 8) /* implies staging transfers due to VK interop */
+#define PIPE_RESOURCE_FLAG_DRV_PRIV              (1 << 9) /* driver/winsys private */
 #define PIPE_RESOURCE_FLAG_FRONTEND_PRIV         (1 << 24) /* gallium frontend private */
 
 /**
@@ -760,15 +762,16 @@ enum pipe_cap
    /** different blend funcs per rendertarget */
    PIPE_CAP_INDEP_BLEND_FUNC,
    PIPE_CAP_MAX_TEXTURE_ARRAY_LAYERS,
-   PIPE_CAP_TGSI_FS_COORD_ORIGIN_UPPER_LEFT,
-   PIPE_CAP_TGSI_FS_COORD_ORIGIN_LOWER_LEFT,
-   PIPE_CAP_TGSI_FS_COORD_PIXEL_CENTER_HALF_INTEGER,
-   PIPE_CAP_TGSI_FS_COORD_PIXEL_CENTER_INTEGER,
+   PIPE_CAP_FS_COORD_ORIGIN_UPPER_LEFT,
+   PIPE_CAP_FS_COORD_ORIGIN_LOWER_LEFT,
+   PIPE_CAP_FS_COORD_PIXEL_CENTER_HALF_INTEGER,
+   PIPE_CAP_FS_COORD_PIXEL_CENTER_INTEGER,
+   PIPE_CAP_POINT_COORD_ORIGIN_UPPER_LEFT,
    PIPE_CAP_DEPTH_CLIP_DISABLE,
    PIPE_CAP_DEPTH_CLIP_DISABLE_SEPARATE,
    PIPE_CAP_DEPTH_CLAMP_ENABLE,
    PIPE_CAP_SHADER_STENCIL_EXPORT,
-   PIPE_CAP_TGSI_INSTANCEID,
+   PIPE_CAP_VS_INSTANCEID,
    PIPE_CAP_VERTEX_ELEMENT_INSTANCE_DIVISOR,
    PIPE_CAP_FRAGMENT_COLOR_CLAMPED,
    PIPE_CAP_MIXED_COLORBUFFER_FORMATS,
@@ -808,11 +811,11 @@ enum pipe_cap
    PIPE_CAP_TEXTURE_TRANSFER_MODES,
    PIPE_CAP_QUERY_PIPELINE_STATISTICS,
    PIPE_CAP_TEXTURE_BORDER_COLOR_QUIRK,
-   PIPE_CAP_MAX_TEXTURE_BUFFER_SIZE,
+   PIPE_CAP_MAX_TEXEL_BUFFER_ELEMENTS_UINT,
    PIPE_CAP_MAX_VIEWPORTS,
    PIPE_CAP_ENDIANNESS,
    PIPE_CAP_MIXED_FRAMEBUFFER_SIZES,
-   PIPE_CAP_TGSI_VS_LAYER_VIEWPORT,
+   PIPE_CAP_VS_LAYER_VIEWPORT,
    PIPE_CAP_MAX_GEOMETRY_OUTPUT_VERTICES,
    PIPE_CAP_MAX_GEOMETRY_TOTAL_OUTPUT_COMPONENTS,
    PIPE_CAP_MAX_TEXTURE_GATHER_COMPONENTS,
@@ -824,10 +827,10 @@ enum pipe_cap
    PIPE_CAP_MAX_TEXTURE_GATHER_OFFSET,
    PIPE_CAP_SAMPLE_SHADING,
    PIPE_CAP_TEXTURE_GATHER_OFFSETS,
-   PIPE_CAP_TGSI_VS_WINDOW_SPACE_POSITION,
+   PIPE_CAP_VS_WINDOW_SPACE_POSITION,
    PIPE_CAP_MAX_VERTEX_STREAMS,
    PIPE_CAP_DRAW_INDIRECT,
-   PIPE_CAP_TGSI_FS_FINE_DERIVATIVE,
+   PIPE_CAP_FS_FINE_DERIVATIVE,
    PIPE_CAP_VENDOR_ID,
    PIPE_CAP_DEVICE_ID,
    PIPE_CAP_ACCELERATED,
@@ -847,19 +850,20 @@ enum pipe_cap
    PIPE_CAP_TEXTURE_FLOAT_LINEAR,
    PIPE_CAP_TEXTURE_HALF_FLOAT_LINEAR,
    PIPE_CAP_DEPTH_BOUNDS_TEST,
-   PIPE_CAP_TGSI_TXQS,
+   PIPE_CAP_TEXTURE_QUERY_SAMPLES,
    PIPE_CAP_FORCE_PERSAMPLE_INTERP,
    PIPE_CAP_SHAREABLE_SHADERS,
    PIPE_CAP_COPY_BETWEEN_COMPRESSED_AND_PLAIN_FORMATS,
    PIPE_CAP_CLEAR_TEXTURE,
    PIPE_CAP_CLEAR_SCISSORED,
    PIPE_CAP_DRAW_PARAMETERS,
-   PIPE_CAP_TGSI_PACK_HALF_FLOAT,
+   PIPE_CAP_SHADER_PACK_HALF_FLOAT,
    PIPE_CAP_MULTI_DRAW_INDIRECT,
    PIPE_CAP_MULTI_DRAW_INDIRECT_PARAMS,
-   PIPE_CAP_TGSI_FS_POSITION_IS_SYSVAL,
-   PIPE_CAP_TGSI_FS_POINT_IS_SYSVAL,
-   PIPE_CAP_TGSI_FS_FACE_IS_INTEGER_SYSVAL,
+   PIPE_CAP_MULTI_DRAW_INDIRECT_PARTIAL_STRIDE,
+   PIPE_CAP_FS_POSITION_IS_SYSVAL,
+   PIPE_CAP_FS_POINT_IS_SYSVAL,
+   PIPE_CAP_FS_FACE_IS_INTEGER_SYSVAL,
    PIPE_CAP_SHADER_BUFFER_OFFSET_ALIGNMENT,
    PIPE_CAP_INVALIDATE_BUFFER,
    PIPE_CAP_GENERATE_MIPMAP,
@@ -874,29 +878,29 @@ enum pipe_cap
    PIPE_CAP_FRAMEBUFFER_NO_ATTACHMENT,
    PIPE_CAP_ROBUST_BUFFER_ACCESS_BEHAVIOR,
    PIPE_CAP_CULL_DISTANCE,
-   PIPE_CAP_TGSI_VOTE,
+   PIPE_CAP_CULL_DISTANCE_NOCOMBINE,
+   PIPE_CAP_SHADER_GROUP_VOTE,
    PIPE_CAP_MAX_WINDOW_RECTANGLES,
    PIPE_CAP_POLYGON_OFFSET_UNITS_UNSCALED,
    PIPE_CAP_VIEWPORT_SUBPIXEL_BITS,
    PIPE_CAP_RASTERIZER_SUBPIXEL_BITS,
    PIPE_CAP_MIXED_COLOR_DEPTH_BITS,
-   PIPE_CAP_TGSI_ARRAY_COMPONENTS,
+   PIPE_CAP_SHADER_ARRAY_COMPONENTS,
    PIPE_CAP_STREAM_OUTPUT_INTERLEAVE_BUFFERS,
-   PIPE_CAP_TGSI_CAN_READ_OUTPUTS,
+   PIPE_CAP_SHADER_CAN_READ_OUTPUTS,
    PIPE_CAP_NATIVE_FENCE_FD,
-   PIPE_CAP_GLSL_OPTIMIZE_CONSERVATIVELY,
    PIPE_CAP_GLSL_TESS_LEVELS_AS_INPUTS,
    PIPE_CAP_FBFETCH,
-   PIPE_CAP_TGSI_MUL_ZERO_WINS,
+   PIPE_CAP_LEGACY_MATH_RULES,
    PIPE_CAP_DOUBLES,
    PIPE_CAP_INT64,
    PIPE_CAP_INT64_DIVMOD,
    PIPE_CAP_TGSI_TEX_TXF_LZ,
-   PIPE_CAP_TGSI_CLOCK,
+   PIPE_CAP_SHADER_CLOCK,
    PIPE_CAP_POLYGON_MODE_FILL_RECTANGLE,
    PIPE_CAP_SPARSE_BUFFER_PAGE_SIZE,
-   PIPE_CAP_TGSI_BALLOT,
-   PIPE_CAP_TGSI_TES_LAYER_VIEWPORT,
+   PIPE_CAP_SHADER_BALLOT,
+   PIPE_CAP_TES_LAYER_VIEWPORT,
    PIPE_CAP_CAN_BIND_CONST_BUFFER_AS_VERTEX,
    PIPE_CAP_ALLOW_MAPPED_BUFFERS_DURING_EXECUTION,
    PIPE_CAP_POST_DEPTH_COVERAGE,
@@ -922,7 +926,7 @@ enum pipe_cap
    PIPE_CAP_CONSERVATIVE_RASTER_INNER_COVERAGE,
    PIPE_CAP_PROGRAMMABLE_SAMPLE_LOCATIONS,
    PIPE_CAP_MAX_GS_INVOCATIONS,
-   PIPE_CAP_MAX_SHADER_BUFFER_SIZE,
+   PIPE_CAP_MAX_SHADER_BUFFER_SIZE_UINT,
    PIPE_CAP_TEXTURE_MIRROR_CLAMP_TO_EDGE,
    PIPE_CAP_MAX_COMBINED_SHADER_BUFFERS,
    PIPE_CAP_MAX_COMBINED_HW_ATOMIC_COUNTERS,
@@ -930,7 +934,7 @@ enum pipe_cap
    PIPE_CAP_MAX_TEXTURE_UPLOAD_MEMORY_BUDGET,
    PIPE_CAP_MAX_VERTEX_ELEMENT_SRC_OFFSET,
    PIPE_CAP_SURFACE_SAMPLE_COUNT,
-   PIPE_CAP_TGSI_ATOMFADD,
+   PIPE_CAP_IMAGE_ATOMIC_FLOAT_ADD,
    PIPE_CAP_QUERY_PIPELINE_STATISTICS_SINGLE,
    PIPE_CAP_RGB_OVERRIDE_DST_ALPHA_BLEND,
    PIPE_CAP_DEST_SURFACE_SRGB_CONTROL,
@@ -938,7 +942,6 @@ enum pipe_cap
    PIPE_CAP_MAX_VARYINGS,
    PIPE_CAP_COMPUTE_GRID_INFO_LAST_BLOCK,
    PIPE_CAP_COMPUTE_SHADER_DERIVATIVES,
-   PIPE_CAP_TGSI_SKIP_SHRINK_IO_ARRAYS,
    PIPE_CAP_IMAGE_LOAD_FORMATTED,
    PIPE_CAP_IMAGE_STORE_FORMATTED,
    PIPE_CAP_THROTTLE,
@@ -951,10 +954,9 @@ enum pipe_cap
    PIPE_CAP_TGSI_DIV,
    PIPE_CAP_FRAGMENT_SHADER_TEXTURE_LOD,
    PIPE_CAP_FRAGMENT_SHADER_DERIVATIVES,
-   PIPE_CAP_VERTEX_SHADER_SATURATE,
    PIPE_CAP_TEXTURE_SHADOW_LOD,
    PIPE_CAP_SHADER_SAMPLES_IDENTICAL,
-   PIPE_CAP_TGSI_ATOMINC_WRAP,
+   PIPE_CAP_IMAGE_ATOMIC_INC_WRAP,
    PIPE_CAP_PREFER_IMM_ARRAYS_AS_CONSTBUF,
    PIPE_CAP_GL_SPIRV,
    PIPE_CAP_GL_SPIRV_VARIABLE_POINTERS,
@@ -1005,6 +1007,12 @@ enum pipe_cap
    PIPE_CAP_SPARSE_TEXTURE_FULL_ARRAY_CUBE_MIPMAPS,
    PIPE_CAP_QUERY_SPARSE_TEXTURE_RESIDENCY,
    PIPE_CAP_CLAMP_SPARSE_TEXTURE_LOD,
+   PIPE_CAP_ALLOW_DRAW_OUT_OF_ORDER,
+   PIPE_CAP_MAX_CONSTANT_BUFFER_SIZE_UINT,
+   PIPE_CAP_HARDWARE_GL_SELECT,
+   PIPE_CAP_DITHERING,
+   PIPE_CAP_FBFETCH_ZS,
+   PIPE_CAP_TIMELINE_SEMAPHORE_IMPORT,
 
    PIPE_CAP_LAST,
    /* XXX do not add caps after PIPE_CAP_LAST! */
@@ -1029,6 +1037,7 @@ enum pipe_texture_transfer_mode {
 
 #define PIPE_QUIRK_TEXTURE_BORDER_COLOR_SWIZZLE_NV50 (1 << 0)
 #define PIPE_QUIRK_TEXTURE_BORDER_COLOR_SWIZZLE_R600 (1 << 1)
+#define PIPE_QUIRK_TEXTURE_BORDER_COLOR_SWIZZLE_FREEDRENO (1 << 2)
 
 enum pipe_endian
 {
@@ -1074,11 +1083,11 @@ enum pipe_shader_cap
    PIPE_SHADER_CAP_MAX_CONTROL_FLOW_DEPTH,
    PIPE_SHADER_CAP_MAX_INPUTS,
    PIPE_SHADER_CAP_MAX_OUTPUTS,
-   PIPE_SHADER_CAP_MAX_CONST_BUFFER_SIZE,
+   PIPE_SHADER_CAP_MAX_CONST_BUFFER0_SIZE,
    PIPE_SHADER_CAP_MAX_CONST_BUFFERS,
    PIPE_SHADER_CAP_MAX_TEMPS,
    /* boolean caps */
-   PIPE_SHADER_CAP_TGSI_CONT_SUPPORTED,
+   PIPE_SHADER_CAP_CONT_SUPPORTED,
    PIPE_SHADER_CAP_INDIRECT_INPUT_ADDR,
    PIPE_SHADER_CAP_INDIRECT_OUTPUT_ADDR,
    PIPE_SHADER_CAP_INDIRECT_TEMP_ADDR,
@@ -1095,17 +1104,13 @@ enum pipe_shader_cap
    PIPE_SHADER_CAP_PREFERRED_IR,
    PIPE_SHADER_CAP_TGSI_SQRT_SUPPORTED,
    PIPE_SHADER_CAP_MAX_SAMPLER_VIEWS,
-   PIPE_SHADER_CAP_TGSI_DROUND_SUPPORTED, /* all rounding modes */
-   PIPE_SHADER_CAP_TGSI_DFRACEXP_DLDEXP_SUPPORTED,
-   PIPE_SHADER_CAP_TGSI_FMA_SUPPORTED,
+   PIPE_SHADER_CAP_DROUND_SUPPORTED, /* all rounding modes */
+   PIPE_SHADER_CAP_DFRACEXP_DLDEXP_SUPPORTED,
    PIPE_SHADER_CAP_TGSI_ANY_INOUT_DECL_RANGE,
-   PIPE_SHADER_CAP_MAX_UNROLL_ITERATIONS_HINT,
    PIPE_SHADER_CAP_MAX_SHADER_BUFFERS,
    PIPE_SHADER_CAP_SUPPORTED_IRS,
    PIPE_SHADER_CAP_MAX_SHADER_IMAGES,
-   PIPE_SHADER_CAP_LOWER_IF_THRESHOLD,
-   PIPE_SHADER_CAP_TGSI_SKIP_MERGE_REGISTERS,
-   PIPE_SHADER_CAP_TGSI_LDEXP_SUPPORTED,
+   PIPE_SHADER_CAP_LDEXP_SUPPORTED,
    PIPE_SHADER_CAP_MAX_HW_ATOMIC_COUNTERS,
    PIPE_SHADER_CAP_MAX_HW_ATOMIC_COUNTER_BUFFERS,
 };
@@ -1210,17 +1215,22 @@ struct pipe_query_data_timestamp_disjoint
  */
 struct pipe_query_data_pipeline_statistics
 {
-   uint64_t ia_vertices;    /**< Num vertices read by the vertex fetcher. */
-   uint64_t ia_primitives;  /**< Num primitives read by the vertex fetcher. */
-   uint64_t vs_invocations; /**< Num vertex shader invocations. */
-   uint64_t gs_invocations; /**< Num geometry shader invocations. */
-   uint64_t gs_primitives;  /**< Num primitives output by a geometry shader. */
-   uint64_t c_invocations;  /**< Num primitives sent to the rasterizer. */
-   uint64_t c_primitives;   /**< Num primitives that were rendered. */
-   uint64_t ps_invocations; /**< Num pixel shader invocations. */
-   uint64_t hs_invocations; /**< Num hull shader invocations. */
-   uint64_t ds_invocations; /**< Num domain shader invocations. */
-   uint64_t cs_invocations; /**< Num compute shader invocations. */
+   union {
+      struct {
+         uint64_t ia_vertices;    /**< Num vertices read by the vertex fetcher. */
+         uint64_t ia_primitives;  /**< Num primitives read by the vertex fetcher. */
+         uint64_t vs_invocations; /**< Num vertex shader invocations. */
+         uint64_t gs_invocations; /**< Num geometry shader invocations. */
+         uint64_t gs_primitives;  /**< Num primitives output by a geometry shader. */
+         uint64_t c_invocations;  /**< Num primitives sent to the rasterizer. */
+         uint64_t c_primitives;   /**< Num primitives that were rendered. */
+         uint64_t ps_invocations; /**< Num pixel shader invocations. */
+         uint64_t hs_invocations; /**< Num hull shader invocations. */
+         uint64_t ds_invocations; /**< Num domain shader invocations. */
+         uint64_t cs_invocations; /**< Num compute shader invocations. */
+      };
+      uint64_t counters[11];
+   };
 };
 
 /**
@@ -1284,6 +1294,12 @@ enum pipe_query_value_type
    PIPE_QUERY_TYPE_U64,
 };
 
+enum pipe_query_flags
+{
+   PIPE_QUERY_WAIT = (1 << 0),
+   PIPE_QUERY_PARTIAL = (1 << 1),
+};
+
 union pipe_color_union
 {
    float f[4];
@@ -1349,6 +1365,7 @@ enum pipe_fd_type
 {
    PIPE_FD_TYPE_NATIVE_SYNC,
    PIPE_FD_TYPE_SYNCOBJ,
+   PIPE_FD_TYPE_TIMELINE_SEMAPHORE,
 };
 
 /**
@@ -1375,6 +1392,7 @@ enum pipe_perf_counter_data_type
 };
 
 #define PIPE_UUID_SIZE 16
+#define PIPE_LUID_SIZE 8
 
 #ifdef PIPE_OS_UNIX
 #define PIPE_MEMORY_FD
