@@ -27,7 +27,6 @@
 
 #include "radeon_drm_bo.h"
 #include "radeon_drm_cs.h"
-#include "radeon_drm_public.h"
 
 #include "util/os_file.h"
 #include "util/u_cpu_detect.h"
@@ -154,9 +153,9 @@ static bool do_winsys_init(struct radeon_drm_winsys *ws)
    /* Get DRM version. */
    version = drmGetVersion(ws->fd);
    if (version->version_major != 2 ||
-       version->version_minor < 12) {
+       version->version_minor < 50) {
       fprintf(stderr, "%s: DRM version is %d.%d.%d but this driver is "
-                      "only compatible with 2.12.0 (kernel 3.2) or later.\n",
+                      "only compatible with 2.50.0 (kernel 4.12) or later.\n",
               __FUNCTION__,
               version->version_major,
               version->version_minor,
@@ -213,7 +212,7 @@ static bool do_winsys_init(struct radeon_drm_winsys *ws)
    case CHIP_RS400:
    case CHIP_RC410:
    case CHIP_RS480:
-      ws->info.chip_class = R300;
+      ws->info.gfx_level = R300;
       break;
    case CHIP_R420:     /* R4xx-based cores. */
    case CHIP_R423:
@@ -224,7 +223,7 @@ static bool do_winsys_init(struct radeon_drm_winsys *ws)
    case CHIP_RS600:
    case CHIP_RS690:
    case CHIP_RS740:
-      ws->info.chip_class = R400;
+      ws->info.gfx_level = R400;
       break;
    case CHIP_RV515:    /* R5xx-based cores. */
    case CHIP_R520:
@@ -232,7 +231,7 @@ static bool do_winsys_init(struct radeon_drm_winsys *ws)
    case CHIP_R580:
    case CHIP_RV560:
    case CHIP_RV570:
-      ws->info.chip_class = R500;
+      ws->info.gfx_level = R500;
       break;
    case CHIP_R600:
    case CHIP_RV610:
@@ -242,13 +241,13 @@ static bool do_winsys_init(struct radeon_drm_winsys *ws)
    case CHIP_RV635:
    case CHIP_RS780:
    case CHIP_RS880:
-      ws->info.chip_class = R600;
+      ws->info.gfx_level = R600;
       break;
    case CHIP_RV770:
    case CHIP_RV730:
    case CHIP_RV710:
    case CHIP_RV740:
-      ws->info.chip_class = R700;
+      ws->info.gfx_level = R700;
       break;
    case CHIP_CEDAR:
    case CHIP_REDWOOD:
@@ -261,24 +260,24 @@ static bool do_winsys_init(struct radeon_drm_winsys *ws)
    case CHIP_BARTS:
    case CHIP_TURKS:
    case CHIP_CAICOS:
-      ws->info.chip_class = EVERGREEN;
+      ws->info.gfx_level = EVERGREEN;
       break;
    case CHIP_CAYMAN:
    case CHIP_ARUBA:
-      ws->info.chip_class = CAYMAN;
+      ws->info.gfx_level = CAYMAN;
       break;
    case CHIP_TAHITI:
    case CHIP_PITCAIRN:
    case CHIP_VERDE:
    case CHIP_OLAND:
    case CHIP_HAINAN:
-      ws->info.chip_class = GFX6;
+      ws->info.gfx_level = GFX6;
       break;
    case CHIP_BONAIRE:
    case CHIP_KAVERI:
    case CHIP_KABINI:
    case CHIP_HAWAII:
-      ws->info.chip_class = GFX7;
+      ws->info.gfx_level = GFX7;
       break;
    }
 
@@ -305,36 +304,31 @@ static bool do_winsys_init(struct radeon_drm_winsys *ws)
       ws->info.has_dedicated_vram = true;
    }
 
-   ws->info.num_rings[RING_GFX] = 1;
+   ws->info.ip[AMD_IP_GFX].num_queues = 1;
    /* Check for dma */
-   ws->info.num_rings[RING_DMA] = 0;
+   ws->info.ip[AMD_IP_SDMA].num_queues = 0;
    /* DMA is disabled on R700. There is IB corruption and hangs. */
-   if (ws->info.chip_class >= EVERGREEN && ws->info.drm_minor >= 27) {
-      ws->info.num_rings[RING_DMA] = 1;
+   if (ws->info.gfx_level >= EVERGREEN) {
+      ws->info.ip[AMD_IP_SDMA].num_queues = 1;
    }
 
    /* Check for UVD and VCE */
-   ws->info.has_video_hw.uvd_decode = false;
-   ws->info.has_video_hw.vce_encode = false;
    ws->info.vce_fw_version = 0x00000000;
-   if (ws->info.drm_minor >= 32) {
-      uint32_t value = RADEON_CS_RING_UVD;
-      if (radeon_get_drm_value(ws->fd, RADEON_INFO_RING_WORKING,
-                               "UVD Ring working", &value)) {
-         ws->info.has_video_hw.uvd_decode = value;
-         ws->info.num_rings[RING_UVD] = 1;
-      }
 
-      value = RADEON_CS_RING_VCE;
-      if (radeon_get_drm_value(ws->fd, RADEON_INFO_RING_WORKING,
-                               NULL, &value) && value) {
+   uint32_t value = RADEON_CS_RING_UVD;
+   if (radeon_get_drm_value(ws->fd, RADEON_INFO_RING_WORKING,
+                            "UVD Ring working", &value)) {
+      ws->info.ip[AMD_IP_UVD].num_queues = 1;
+   }
 
-         if (radeon_get_drm_value(ws->fd, RADEON_INFO_VCE_FW_VERSION,
-                                  "VCE FW version", &value)) {
-            ws->info.vce_fw_version = value;
-            ws->info.num_rings[RING_VCE] = 1;
-            ws->info.has_video_hw.vce_encode = true;
-         }
+   value = RADEON_CS_RING_VCE;
+   if (radeon_get_drm_value(ws->fd, RADEON_INFO_RING_WORKING,
+                            NULL, &value) && value) {
+
+      if (radeon_get_drm_value(ws->fd, RADEON_INFO_VCE_FW_VERSION,
+                               "VCE FW version", &value)) {
+         ws->info.vce_fw_version = value;
+         ws->info.ip[AMD_IP_VCE].num_queues = 1;
       }
    }
 
@@ -361,34 +355,26 @@ static bool do_winsys_init(struct radeon_drm_winsys *ws)
               retval);
       return false;
    }
-   ws->info.gart_size = gem_info.gart_size;
-   ws->info.vram_size = gem_info.vram_size;
-   ws->info.vram_vis_size = gem_info.vram_visible;
-   /* Older versions of the kernel driver reported incorrect values, and
-    * didn't support more than 256MB of visible VRAM anyway
-    */
-   if (ws->info.drm_minor < 49)
-      ws->info.vram_vis_size = MIN2(ws->info.vram_vis_size, 256*1024*1024);
-
-   ws->info.gart_size_kb = DIV_ROUND_UP(ws->info.gart_size, 1024);
-   ws->info.vram_size_kb = DIV_ROUND_UP(ws->info.vram_size, 1024);
+   ws->info.gart_size_kb = DIV_ROUND_UP(gem_info.gart_size, 1024);
+   ws->info.vram_size_kb = DIV_ROUND_UP(gem_info.vram_size, 1024);
+   ws->info.vram_vis_size_kb = DIV_ROUND_UP(gem_info.vram_visible, 1024);
 
    /* Radeon allocates all buffers contiguously, which makes large allocations
     * unlikely to succeed. */
    if (ws->info.has_dedicated_vram)
-      ws->info.max_alloc_size = ws->info.vram_size * 0.7;
+      ws->info.max_heap_size_kb = ws->info.vram_size_kb;
    else
-      ws->info.max_alloc_size = ws->info.gart_size * 0.7;
+      ws->info.max_heap_size_kb = ws->info.gart_size_kb;
 
-   if (ws->info.drm_minor < 40)
-      ws->info.max_alloc_size = MIN2(ws->info.max_alloc_size, 256*1024*1024);
-   /* Both 32-bit and 64-bit address spaces only have 4GB. */
-   ws->info.max_alloc_size = MIN2(ws->info.max_alloc_size, 3ull*1024*1024*1024);
+   /* Both 32-bit and 64-bit address spaces only have 4GB.
+    * This is a limitation of the VM allocator in the winsys.
+    */
+   ws->info.max_heap_size_kb = MIN2(ws->info.max_heap_size_kb, 4 * 1024 * 1024); /* 4 GB */
 
    /* Get max clock frequency info and convert it to MHz */
    radeon_get_drm_value(ws->fd, RADEON_INFO_MAX_SCLK, NULL,
-                        &ws->info.max_shader_clock);
-   ws->info.max_shader_clock /= 1000;
+                        &ws->info.max_gpu_freq_mhz);
+   ws->info.max_gpu_freq_mhz /= 1000;
 
    ws->num_cpus = sysconf(_SC_NPROCESSORS_ONLN);
 
@@ -420,18 +406,18 @@ static bool do_winsys_init(struct radeon_drm_winsys *ws)
                            &tiling_config);
 
       ws->info.r600_num_banks =
-            ws->info.chip_class >= EVERGREEN ?
+            ws->info.gfx_level >= EVERGREEN ?
                4 << ((tiling_config & 0xf0) >> 4) :
                     4 << ((tiling_config & 0x30) >> 4);
 
       ws->info.pipe_interleave_bytes =
-            ws->info.chip_class >= EVERGREEN ?
+            ws->info.gfx_level >= EVERGREEN ?
                256 << ((tiling_config & 0xf00) >> 8) :
                       256 << ((tiling_config & 0xc0) >> 6);
 
       if (!ws->info.pipe_interleave_bytes)
          ws->info.pipe_interleave_bytes =
-               ws->info.chip_class >= EVERGREEN ? 512 : 256;
+               ws->info.gfx_level >= EVERGREEN ? 512 : 256;
 
       radeon_get_drm_value(ws->fd, RADEON_INFO_NUM_TILE_PIPES, NULL,
                            &ws->info.num_tile_pipes);
@@ -459,19 +445,19 @@ static bool do_winsys_init(struct radeon_drm_winsys *ws)
                               &ws->info.enabled_rb_mask);
 
       ws->info.r600_has_virtual_memory = false;
-      if (ws->info.drm_minor >= 13) {
-         uint32_t ib_vm_max_size;
 
-         ws->info.r600_has_virtual_memory = true;
-         if (!radeon_get_drm_value(ws->fd, RADEON_INFO_VA_START, NULL,
-                                   &ws->va_start))
-            ws->info.r600_has_virtual_memory = false;
-         if (!radeon_get_drm_value(ws->fd, RADEON_INFO_IB_VM_MAX_SIZE, NULL,
-                                   &ib_vm_max_size))
-            ws->info.r600_has_virtual_memory = false;
-         radeon_get_drm_value(ws->fd, RADEON_INFO_VA_UNMAP_WORKING, NULL,
-                              &ws->va_unmap_working);
-      }
+      uint32_t ib_vm_max_size;
+
+      ws->info.r600_has_virtual_memory = true;
+      if (!radeon_get_drm_value(ws->fd, RADEON_INFO_VA_START, NULL,
+                                &ws->va_start))
+         ws->info.r600_has_virtual_memory = false;
+      if (!radeon_get_drm_value(ws->fd, RADEON_INFO_IB_VM_MAX_SIZE, NULL,
+                                &ib_vm_max_size))
+         ws->info.r600_has_virtual_memory = false;
+      radeon_get_drm_value(ws->fd, RADEON_INFO_VA_UNMAP_WORKING, NULL,
+                           &ws->va_unmap_working);
+
       if (ws->gen == DRV_R600 && !debug_get_bool_option("RADEON_VA", false))
          ws->info.r600_has_virtual_memory = false;
    }
@@ -483,9 +469,9 @@ static bool do_winsys_init(struct radeon_drm_winsys *ws)
                         &ws->info.r600_max_quad_pipes);
 
    /* All GPUs have at least one compute unit */
-   ws->info.num_good_compute_units = 1;
+   ws->info.num_cu = 1;
    radeon_get_drm_value(ws->fd, RADEON_INFO_ACTIVE_CU_COUNT, NULL,
-                        &ws->info.num_good_compute_units);
+                        &ws->info.num_cu);
 
    radeon_get_drm_value(ws->fd, RADEON_INFO_MAX_SE, NULL,
                         &ws->info.max_se);
@@ -541,7 +527,7 @@ static bool do_winsys_init(struct radeon_drm_winsys *ws)
                         &ws->info.max_sa_per_se);
    if (ws->gen == DRV_SI) {
       ws->info.max_good_cu_per_sa =
-      ws->info.min_good_cu_per_sa = ws->info.num_good_compute_units /
+      ws->info.min_good_cu_per_sa = ws->info.num_cu /
                                     (ws->info.max_se * ws->info.max_sa_per_se);
    }
 
@@ -555,7 +541,7 @@ static bool do_winsys_init(struct radeon_drm_winsys *ws)
       return false;
    }
 
-   if (ws->info.chip_class == GFX7) {
+   if (ws->info.gfx_level == GFX7) {
       if (!radeon_get_drm_value(ws->fd, RADEON_INFO_CIK_MACROTILE_MODE_ARRAY, NULL,
                                 ws->info.cik_macrotile_mode_array)) {
          fprintf(stderr, "radeon: Kernel 3.13 is required for Sea Islands support.\n");
@@ -563,7 +549,7 @@ static bool do_winsys_init(struct radeon_drm_winsys *ws)
       }
    }
 
-   if (ws->info.chip_class >= GFX6) {
+   if (ws->info.gfx_level >= GFX6) {
       if (!radeon_get_drm_value(ws->fd, RADEON_INFO_SI_TILE_MODE_ARRAY, NULL,
                                 ws->info.si_tile_mode_array)) {
          fprintf(stderr, "radeon: Kernel 3.10 is required for Southern Islands support.\n");
@@ -574,33 +560,14 @@ static bool do_winsys_init(struct radeon_drm_winsys *ws)
    /* Hawaii with old firmware needs type2 nop packet.
     * accel_working2 with value 3 indicates the new firmware.
     */
-   ws->info.gfx_ib_pad_with_type2 = ws->info.chip_class <= GFX6 ||
+   ws->info.gfx_ib_pad_with_type2 = ws->info.gfx_level <= GFX6 ||
                                     (ws->info.family == CHIP_HAWAII &&
                                      ws->accel_working2 < 3);
    ws->info.tcc_cache_line_size = 64; /* TC L2 line size on GCN */
    ws->info.ib_alignment = 4096;
-   ws->info.kernel_flushes_hdp_before_ib = ws->info.drm_minor >= 40;
-   /* HTILE is broken with 1D tiling on old kernels and GFX7. */
-   ws->info.htile_cmask_support_1d_tiling = ws->info.chip_class != GFX7 ||
-                                                                   ws->info.drm_minor >= 38;
-   ws->info.si_TA_CS_BC_BASE_ADDR_allowed = ws->info.drm_minor >= 48;
    ws->info.has_bo_metadata = false;
-   ws->info.has_gpu_reset_status_query = ws->info.drm_minor >= 43;
    ws->info.has_eqaa_surface_allocator = false;
-   ws->info.has_format_bc1_through_bc7 = ws->info.drm_minor >= 31;
-   ws->info.kernel_flushes_tc_l2_after_ib = true;
-   /* Old kernels disallowed register writes via COPY_DATA
-    * that are used for indirect compute dispatches. */
-   ws->info.has_indirect_compute_dispatch = ws->info.chip_class == GFX7 ||
-                                            (ws->info.chip_class == GFX6 &&
-                                             ws->info.drm_minor >= 45);
-   /* GFX6 doesn't support unaligned loads. */
-   ws->info.has_unaligned_shader_loads = ws->info.chip_class == GFX7 &&
-                                         ws->info.drm_minor >= 50;
    ws->info.has_sparse_vm_mappings = false;
-   /* 2D tiling on GFX7 is supported since DRM 2.35.0 */
-   ws->info.has_2d_tiling = ws->info.chip_class <= GFX6 || ws->info.drm_minor >= 35;
-   ws->info.has_read_registers_query = ws->info.drm_minor >= 42;
    ws->info.max_alignment = 1024*1024;
    ws->info.has_graphics = true;
    ws->info.cpdma_prefetch_writes_memory = true;
@@ -685,9 +652,6 @@ uint32_t radeon_drm_get_gpu_reset_counter(struct radeon_drm_winsys *ws)
 {
    uint64_t retval = 0;
 
-   if (!ws->info.has_gpu_reset_status_query)
-      return 0;
-
    radeon_get_drm_value(ws->fd, RADEON_INFO_GPU_RESET_COUNTER,
                         "gpu-reset-counter", (uint32_t*)&retval);
    return retval;
@@ -713,7 +677,7 @@ static uint64_t radeon_query_value(struct radeon_winsys *rws,
    case RADEON_NUM_MAPPED_BUFFERS:
       return ws->num_mapped_buffers;
    case RADEON_TIMESTAMP:
-      if (ws->info.drm_minor < 20 || ws->gen < DRV_R600) {
+      if (ws->gen < DRV_R600) {
          assert(0);
          return 0;
       }
@@ -852,9 +816,9 @@ radeon_drm_winsys_create(int fd, const struct pipe_screen_config *config,
    if (!do_winsys_init(ws))
       goto fail1;
 
-   pb_cache_init(&ws->bo_cache, RADEON_MAX_CACHED_HEAPS,
+   pb_cache_init(&ws->bo_cache, RADEON_NUM_HEAPS,
                  500000, ws->check_vm ? 1.0f : 2.0f, 0,
-                 MIN2(ws->info.vram_size, ws->info.gart_size), NULL,
+                 (uint64_t)MIN2(ws->info.vram_size_kb, ws->info.gart_size_kb) * 1024, NULL,
                  radeon_bo_destroy,
                  radeon_bo_can_reclaim);
 
@@ -865,7 +829,7 @@ radeon_drm_winsys_create(int fd, const struct pipe_screen_config *config,
        */
       if (!pb_slabs_init(&ws->bo_slabs,
                          RADEON_SLAB_MIN_SIZE_LOG2, RADEON_SLAB_MAX_SIZE_LOG2,
-                         RADEON_MAX_SLAB_HEAPS, false,
+                         RADEON_NUM_HEAPS, false,
                          ws,
                          radeon_bo_can_reclaim_slab,
                          radeon_bo_slab_alloc,
@@ -924,17 +888,8 @@ radeon_drm_winsys_create(int fd, const struct pipe_screen_config *config,
    ws->vm32.start = ws->va_start;
    ws->vm32.end = 1ull << 32;
 
-   /* The maximum is 8GB of virtual address space limited by the kernel.
-    * It's obviously not enough for bigger cards, like Hawaiis with 4GB
-    * and 8GB of physical memory and 4GB of GART.
-    *
-    * Older kernels set the limit to 4GB, which is even worse, so they only
-    * have 32-bit address space.
-    */
-   if (ws->info.drm_minor >= 41) {
-      ws->vm64.start = 1ull << 32;
-      ws->vm64.end = 1ull << 33;
-   }
+   ws->vm64.start = 1ull << 32;
+   ws->vm64.end = 1ull << 33;
 
    /* TTM aligns the BO size to the CPU page size */
    ws->info.gart_page_size = sysconf(_SC_PAGESIZE);
